/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.actionEnforcement.impl;

import ro.utcn.dsrl.actionEnforcementModule.actionEnforcement.ContextActionEnforcer;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.impl.StubServiceCenterManagement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.utils.commonDtos.PhysicalHost;
import ro.utcn.dsrl.utils.commonDtos.VirtualDiskInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualNetworkInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class DefaultContextActionEnforcer implements ContextActionEnforcer {

    private ServiceCenterManagement serviceCenterManagement;
    private static Pattern taskNamePattern = Pattern.compile("index_\\d*_");

    public DefaultContextActionEnforcer(ServiceCenterManagement delegate) {
        this.serviceCenterManagement = delegate;
    }


    public void executeRemoveActivityAction(ContextAction contextAction) throws ApplicationException {
        RemoveActivity action = (RemoveActivity) contextAction;
        ApplicationActivity task = action.getActivity();

        if (serviceCenterManagement != null) {
            if (!(serviceCenterManagement instanceof StubServiceCenterManagement)) {
                BufferedReader reader = null;
                try {
                    //          String path = server.getHddResources().iterator().next().getPhysicalPath();
                    GlobalLoopLogger.getLogger().info("Removing ...");
                    VirtualNetworkInfo network = new VirtualNetworkInfo();
                    String taskName = task.getName().replaceAll("\\.", "_").split("#")[1];
                    Matcher nameMatcher = taskNamePattern.matcher(taskName);
                    if (nameMatcher.find()) {
                        taskName = taskName.replace(taskName.substring(nameMatcher.start(), nameMatcher.end()), "_");
                    }

                    VirtualTaskInfo taskInfo = new VirtualTaskInfo(taskName);

                    taskInfo.setId(task.getId());
                    serviceCenterManagement.removeVirtualMachine(taskInfo);

                } catch (Exception ex) {
                    GlobalLoopLogger.getLogger().debug(ex.toString());
                    throw new ApplicationException(ex.toString(), ex.getCause());
                }

            }

        } else {
            throw new ApplicationException("Proxy is null");
        }
    }

    public VirtualTaskInfo executeDeployActivityAction(ContextAction contextAction, String virtualMachineConf) throws ApplicationException {

        DeployActivity action = (DeployActivity) contextAction;
        ServiceCenterServer server = (ServiceCenterServer) action.getResourceTo();
        ApplicationActivity task = action.getActivity();
        ApplicationActivityNetworkInfo networkInfo = task.getNetworkInfo();
        VirtualTaskInfo taskInfo = null;

        if (serviceCenterManagement != null) {
//            if (!(serviceCenterManagement instanceof StubServiceCenterManagement)) {
            BufferedReader reader = null;
            try {
//            String path = server.getHddResources().iterator().next().getPhysicalPath();
                GlobalLoopLogger.getLogger().info("Deploying ...");
                VirtualNetworkInfo network = new VirtualNetworkInfo();
                String taskName = task.getName().replaceAll("\\.", "_").split("#")[1];
                Matcher nameMatcher = taskNamePattern.matcher(taskName);
                if (nameMatcher.find()) {
                    taskName = taskName.replace(taskName.substring(nameMatcher.start(), nameMatcher.end()), "_");
                }
                taskInfo = new VirtualTaskInfo(taskName);


                if (!GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
                    //TODO: to be fixed. Sa nu citesc chiar asa direct template cu path sau ceva se tat acolo. k acuma in EPKNB i pathu

                    String vmTemplate = "";
                    reader = new BufferedReader(new FileReader(virtualMachineConf));
                    String line = "";
                    while ((line = reader.readLine()) != null) {
                        vmTemplate += line + "\n";
                    }
                    taskInfo.setVmTemplate(vmTemplate);
                    network.setIp(networkInfo.getIpAddress());
                    network.setVncPassword("123456");
                    network.setVncPort(networkInfo.getVncPort());
                    taskInfo.setNetworkInfo(network);
                    for (Object info : task.getDiskInfo()) {
                        ApplicationActivityDiskInfo diskInfo = (ApplicationActivityDiskInfo) info;
                        VirtualDiskInfo virtualDiskInfo = new VirtualDiskInfo();
                        virtualDiskInfo.setFormat(diskInfo.getDiskFormat());
                        virtualDiskInfo.setSize(diskInfo.getDiskSize());
                        virtualDiskInfo.setSource(diskInfo.getPhysicalPath());
                        virtualDiskInfo.setType(diskInfo.getDiskType());
                        taskInfo.addDiskDto(virtualDiskInfo);
                    }
                }

                CPU serverCPU = server.getCpuResources().iterator().next();
                //de aici trebe sa reiasa 1 pt fiecare core folosit. de ex 3 x 3000 = 6 x 1500 = 3 final
                double requestedCPUFreq = (float) task.getCpuRequiredMaxValue() * task.getNumberOfCoresRequiredValue() * serverCPU.getCpuCoreCount() / serverCPU.getMaximumWorkLoad();
                taskInfo.setRequestedCPU(requestedCPUFreq);
                taskInfo.setRequestedMemory((int) task.getMemRequiredMaxValue());
                PhysicalHost host = new PhysicalHost();
                host.setHostname(server.getIpAddress());
                host.setIm(PhysicalHost.IM_KVM);
                host.setTm(PhysicalHost.TM_SSH);
                host.setVmm(PhysicalHost.VMM_KVM);
                host.setId(server.getId());
                VirtualTaskInfo info;
                try {
                    if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
                        taskInfo.setId(task.getId());
                        info = serviceCenterManagement.deployVirtualMachine(taskInfo, host);
                    } else {
                        info = serviceCenterManagement.createAndDeployVirtualMachine(taskInfo, host);
                    }

                    taskInfo.setId(info.getId());
                    task.setId(info.getId());
                    networkInfo.setId(info.getNetworkInfo().getId());
//                  delegate.restartInitialVM(info.getNetworkInfo().getIp());
                } catch (ServiceCenterAccessException ex) {
                    GlobalLoopLogger.getLogger().debug(ex.toString());
                    throw new ApplicationException(ex.toString(), ex.getCause());
                }

                 
            } catch (Exception ex) {
                GlobalLoopLogger.getLogger().debug(ex.toString());
//                    throw new ApplicationException(ex.toString(), ex.getCause());
            }

//            }


        } else {
            throw new ApplicationException("Proxy is null");
        }

        return taskInfo;
    }

    public void executeMigrateActivityAction(ContextAction a) throws ApplicationException {
        MigrateActivity action = (MigrateActivity) a;
        ServiceCenterServer newServer = (ServiceCenterServer) action.getResourceTo();
        ServiceCenterServer oldServer = (ServiceCenterServer) action.getResourceFrom();
        ApplicationActivity task = action.getActivity();

        if (serviceCenterManagement != null) {
            VirtualTaskInfo taskInfo = new VirtualTaskInfo(task.getName());
            taskInfo.setId(task.getId());
            PhysicalHost host = new PhysicalHost();
            host.setId(newServer.getId());
            try {
                serviceCenterManagement.migrateVirtualMachine(taskInfo, host);
            } catch (ServiceCenterAccessException ex) {
                GlobalLoopLogger.error(ex);
                throw new ApplicationException(ex.toString(), ex.getCause());
            }

            
        } else {
            throw new ApplicationException("Proxy is null");
        }
    }

//    public void executeMigrateActivityAction(ContextAction a, String virtualMachineConf) throws ApplicationException {
//        MigrateActivity action = (MigrateActivity) a;
//        ServiceCenterServer newServer = (ServiceCenterServer) action.getResourceTo();
//        ApplicationActivity task = action.getActivity();
//        if (serviceCenterManagement != null) {
//            VirtualTaskInfo taskInfo = new VirtualTaskInfo(task.getName());
//            taskInfo.setId(task.getId());
//            PhysicalHost hostSrc = new PhysicalHost();
//            PhysicalHost hostDst = new PhysicalHost();
//            hostSrc.setId(action.getResourceFrom().getId());
//            hostDst.setId(action.getResourceTo().getId());
//
//            //firstly delete
//            try {
//                GlobalLoopLogger.getLogger().info("Deploying ...");
//                serviceCenterManagement.removeVirtualMachine(taskInfo);
//
//
//            } catch (Exception ex) {
//                GlobalLoopLogger.getLogger().debug(ex.toString());
//                throw new ApplicationException(ex.toString(), ex.getCause());
//            }
//
//            //deploy on dst
//            ServiceCenterServer server = (ServiceCenterServer) action.getResourceTo();
//            task = action.getActivity();
//            ApplicationActivityNetworkInfo networkInfo = task.getNetworkInfo();
//
//
//            if (serviceCenterManagement != null) {
//                if (!(serviceCenterManagement instanceof StubServiceCenterManagement)) {
//                    BufferedReader reader = null;
//                    try {
////            String path = server.getHddResources().iterator().next().getPhysicalPath();
//                        GlobalLoopLogger.getLogger().info("Deploying ...");
//                        VirtualNetworkInfo network = new VirtualNetworkInfo();
//                        String taskName = task.getName().replaceAll("\\.", "_").split("#")[1];
//                        Matcher nameMatcher = taskNamePattern.matcher(taskName);
//                        if (nameMatcher.find()) {
//                            taskName = taskName.replace(taskName.substring(nameMatcher.start(), nameMatcher.end()), "_");
//                        }
//
//                        taskInfo = new VirtualTaskInfo(taskName);
//
//                        if (!GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
//                            //TODO: to be fixed. Sa nu citesc chiar asa direct template cu path sau ceva se tat acolo. k acuma in EPKNB i pathu
//
//                            String vmTemplate = "";
//                            reader = new BufferedReader(new FileReader(virtualMachineConf));
//                            String line = "";
//                            while ((line = reader.readLine()) != null) {
//                                vmTemplate += line + "\n";
//                            }
//                            taskInfo.setVmTemplate(vmTemplate);
//                            network.setIp(networkInfo.getIpAddress());
//                            network.setVncPassword("123456");
//                            network.setVncPort(networkInfo.getVncPort());
//                            taskInfo.setNetworkInfo(network);
//                            for (Object info : task.getDiskInfo()) {
//                                ApplicationActivityDiskInfo diskInfo = (ApplicationActivityDiskInfo) info;
//                                VirtualDiskInfo virtualDiskInfo = new VirtualDiskInfo();
//                                virtualDiskInfo.setFormat(diskInfo.getDiskFormat());
//                                virtualDiskInfo.setSize(diskInfo.getDiskSize());
//                                virtualDiskInfo.setSource(diskInfo.getPhysicalPath());
//                                virtualDiskInfo.setType(diskInfo.getDiskType());
//                                taskInfo.addDiskDto(virtualDiskInfo);
//                            }
//                        }
//
//                        CPU serverCPU = server.getCpuResources().iterator().next();
//                        //de aici trebe sa reiasa 1 pt fiecare core folosit. de ex 3 x 3000 = 6 x 1500 = 3 final
//                        double requestedCPUFreq = (float) task.getCpuRequiredMaxValue() * task.getNumberOfCoresRequiredValue() * serverCPU.getCpuCoreCount() / serverCPU.getMaximumWorkLoad();
//                        taskInfo.setRequestedCPU(requestedCPUFreq);
//                        taskInfo.setRequestedMemory((int) task.getMemRequiredMaxValue());
//                        PhysicalHost host = new PhysicalHost();
//                        host.setHostname(server.getIpAddress());
//                        host.setIm(PhysicalHost.IM_KVM);
//                        host.setTm(PhysicalHost.TM_SSH);
//                        host.setVmm(PhysicalHost.VMM_KVM);
//                        host.setId(server.getId());
//                        VirtualTaskInfo info;
//                        try {
//                            if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
//                                taskInfo.setId(task.getId());
//                                info = serviceCenterManagement.deployVirtualMachine(taskInfo, host);
//                            } else {
//                                info = serviceCenterManagement.createAndDeployVirtualMachine(taskInfo, host);
//                            }
//
//                            task.setId(info.getId());
//                            networkInfo.setId(info.getNetworkInfo().getId());
////                    delegate.restartInitialVM(info.getNetworkInfo().getIp());
//                        } catch (ServiceCenterAccessException ex) {
//                            GlobalLoopLogger.getLogger().debug(ex.toString());
//                            throw new ApplicationException(ex.toString(), ex.getCause());
//                        }
//                    } catch (Exception ex) {
//                        GlobalLoopLogger.getLogger().debug(ex.toString());
//                        throw new ApplicationException(ex.toString(), ex.getCause());
//                    }
//
//                }
//            }
//        }
//    }

    public void executeSetServerStateAction(ContextAction a) throws ApplicationException {
        SetServerStateActivity action = (SetServerStateActivity) a;
        for (ContextResource resource : action.getResources()) {
            ServiceCenterServer server = (ServiceCenterServer) resource;
            PhysicalHost host = new PhysicalHost();
            host.setHostname(server.getIpAddress());
            host.setMac(server.getMacAddress());

            host.setId(server.getId());
            try {
                if (action.getTargetServerState() == 0) {
                    serviceCenterManagement.sendServerToSleep(host);
                    serviceCenterManagement.disableHost(host);

                     

                } else {
                    serviceCenterManagement.wakeUpServer(host);
                    serviceCenterManagement.enableHost(host);

                 

                }
            } catch (ServiceCenterAccessException ex) {
                GlobalLoopLogger.getLogger().debug(ex.toString());
                throw new ApplicationException(ex.toString(), ex.getCause());
            }


        }
    }


    public void terminate() {
//		if(currentlyRunningThread != null & currentlyRunningThread.isAlive()){
//			currentlyRunningThread.interrupt();
        serviceCenterManagement.terminate();
//			try {
//				currentlyRunningThread.join();
//			} catch (InterruptedException e) {
//				GlobalLoopLogger.error(e);
//			}
    }


}
