/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.agents;

import org.apache.log4j.Logger;

import ro.utcn.dsrl.actionEnforcementModule.actionEnforcement.ContextActionEnforcer;
import ro.utcn.dsrl.actionEnforcementModule.agents.behaviors.APEAReceiveMessageBehavior;
import ro.utcn.dsrl.actionEnforcementModule.agents.behaviors.APEAExecuteActionPlanBehavior;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.ContextAccessAndProcessingAPI;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.utils.actionEnforcement.ActionPlanDto;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import jade.core.Agent;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */




public class ActionPlanEnforcementAgent extends Agent implements Interruptable {

    private EACMInstance eacmInstance;
    private ContextActionEnforcer actionEnforcer;
    private Logger logger;
    private Thread actionEnforcementThread;
    private APEAExecuteActionPlanBehavior currentlyRunningBehavior;
    private ActionPlanEnforcementAgent thisAgent;
    private ContextAccessAndProcessingAPI contextAccessAndProcessingAPI;
    private CAPInteractions CAPInteractions;

    protected void setup() {
        thisAgent = this;
        Object[] args = getArguments();
        if (args != null && args.length > 3) {
            eacmInstance = (EACMInstance) args[0];
            contextAccessAndProcessingAPI = (ContextAccessAndProcessingAPI) args[1];
            actionEnforcer = (ContextActionEnforcer) args[2];
            CAPInteractions = (CAPInteractions) args[3];
        } else {
            GlobalLoopLogger.getLogger().info("ActionPlanEnforcementAgent not enoguh parameters received.Terminating");
            this.takeDown();
        }
        logger = GlobalLoopLogger.getLogger();
        addBehaviour(new APEAReceiveMessageBehavior(this, eacmInstance));
        GlobalLoopLogger.getLogger().info("AEA started");
    }

    public void executeActionPlan(final ActionPlanDto actionPlanDto) {
        actionEnforcementThread = new Thread("ActionEnforcementThread") {


            public void run() {
                currentlyRunningBehavior = new APEAExecuteActionPlanBehavior(thisAgent, eacmInstance,
                        contextAccessAndProcessingAPI, actionEnforcer, CAPInteractions, actionPlanDto, logger);
                currentlyRunningBehavior.action();
            }


        };
        actionEnforcementThread.start();
        try {
            actionEnforcementThread.join();
        } catch (InterruptedException e) {
            GlobalLoopLogger.error(e);
        }

    }

    public void terminate() {
        if (actionEnforcementThread != null && actionEnforcementThread.isAlive()) {
            actionEnforcementThread.interrupt();
            currentlyRunningBehavior.terminate();
            try {
                actionEnforcementThread.join();
            } catch (InterruptedException e) {
                GlobalLoopLogger.error(e);
            }
        }


    }
}
