/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement;

import java.util.Collection;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public interface CAPInteractions {

    /**
     * 
     * @param dto information needed to uniquely identify the service in the service pool
     * @return information about a service from the service pool
     */
    public abstract ActivityDto getService(ActivityDto dto) throws ApplicationException;

    /**
     * 
     * @param dto the new service to be added to  the service pool
     */
    public abstract void addService(ActivityDto dto) throws ApplicationException;

    /**
     * 
     * @param dto information needed to uniquely identify the service in the service pool
     * Removes a service from the service pool
     */
    public abstract void removeService(ActivityDto dto) throws ApplicationException;

    /**
     * 
     * @param dto information needed to uniquely identify a service in the deployed services pool
     * @return information about a deployed service ( a service running on the testbed )
     */
    public abstract ActivityDto getDeployedService(ActivityDto dto) throws ApplicationException;

    /**
     * 
     * @param dto information about a service running on the testbed which will be added in the deployed
     * services list
     */
    public abstract void addDeployedService(ActivityDto dto) throws ApplicationException;

    /**
     * 
     * @param dto
     */
    public abstract void removeDeployedService(ActivityDto dto) throws ApplicationException;

    public abstract void updateContextModelWithMonitoredData(EACMInstance localEACMInstance) throws ApplicationException;

    public abstract void persistContextSituation(ContextSituationDto contextSituationDto) throws ApplicationException;

    public abstract void persistContextSituations(Collection<ContextSituationDto> contextSituationDtos) throws ApplicationException;
    //public abstract List<ActivityDto> getAllServices() throws ApplicationException;
}
