/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement;

import ro.utcn.dsrl.utils.commonDtos.PhysicalHost;
import ro.utcn.dsrl.utils.commonDtos.ServerInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;

import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public interface ServiceCenterManagement extends Interruptable {

    /**
     * Reads from OpenNebula the currently pending (undeployed) virtual machines
     * A problem is that for OpenNebula one declares CPU required by VM as 1.0 = 1 CORE.
     * This means that we can't find out the CPU frequency a VM requires.
     * !!!! To solve this problem, I have added an option in general configuration mentioning
     * VM_DEFAULT_CPU_FREQUENCY.
     */
    List<VirtualTaskInfo> getPendingVirtualMachines() throws ServiceCenterAccessException;

    List<VirtualTaskInfo> getAllVirtualMachines() throws ServiceCenterAccessException;

    List<ServerInfo> getDefinedServers() throws ServiceCenterAccessException;

    VirtualTaskInfo getVMInfo(Integer vmID) throws ServiceCenterAccessException;

    ServerInfo getServerInfo(PhysicalHost physicalHost) throws ServiceCenterAccessException;

    VirtualTaskInfo createAndDeployVirtualMachine(VirtualTaskInfo infoVirtual, PhysicalHost physicalHost) throws ServiceCenterAccessException;

    VirtualTaskInfo deployVirtualMachine(VirtualTaskInfo infoVirtual, PhysicalHost physicalHost) throws ServiceCenterAccessException;

    void removeVirtualMachine(VirtualTaskInfo infoVirtual) throws ServiceCenterAccessException;

    void startVirtualMachine(VirtualTaskInfo info) throws ServiceCenterAccessException;

    void stopVirtualMachine(VirtualTaskInfo info) throws ServiceCenterAccessException;

    void migrateVirtualMachine(VirtualTaskInfo info, PhysicalHost destination) throws ServiceCenterAccessException;

    PhysicalHost addHost(PhysicalHost host) throws ServiceCenterAccessException;

    void removeHost(PhysicalHost host) throws ServiceCenterAccessException;

    void enableHost(PhysicalHost host) throws ServiceCenterAccessException;

    void disableHost(PhysicalHost host) throws ServiceCenterAccessException;

    void deleteVirtualMachine(VirtualTaskInfo info) throws ServiceCenterAccessException;

    //    void modifyVirtualMachine(String vmName, int memory, int procPercentage, int cores) throws ServiceCenterAccessException;
//
    void wakeUpServer(PhysicalHost host) throws ServiceCenterAccessException;

    void sendServerToSleep(PhysicalHost host) throws ServiceCenterAccessException;

    String getEnergyConsumptionInfo(PhysicalHost host) throws ServiceCenterAccessException;

    void restartInitialVM(String ip);

    void terminateRunningOperations() throws ServiceCenterAccessException;

    boolean serverIsAlive(String ip) throws ServiceCenterAccessException;

    VirtualTaskInfo createVirtualMachine(VirtualTaskInfo taskInfo) throws ServiceCenterAccessException;
}
