/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.impl;

import java.util.Collection;
import java.util.Iterator;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.ContextAccessAndProcessingAPI;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.ContextModelAccessException;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.List;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class DefaultCAPInteractions implements CAPInteractions {

    private ContextAccessAndProcessingAPI contextAccessAndProcessingAPI;

    public DefaultCAPInteractions(ContextAccessAndProcessingAPI contextAccessAndProcessingAPI) {
        this.contextAccessAndProcessingAPI = contextAccessAndProcessingAPI;
    }

    public ActivityDto getService(ActivityDto dto) throws ApplicationException {
        try {
            return contextAccessAndProcessingAPI.getService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }

    }

    // public List<ActivityDto> getAllServices() throws ApplicationException {
    //    try {
    //        return contextAccessAndProcessingAPI.getAllServices();
    //    } catch (ContextModelAccessException e) {
    //        GlobalLoopLogger.error(e);
    //        throw new ApplicationException(e.toString(), e.getCause());
    //    }
    //}
    public void addService(ActivityDto dto) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.addService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void removeService(ActivityDto dto) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.removeService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public ActivityDto getDeployedService(ActivityDto dto) throws ApplicationException {
        try {
            return contextAccessAndProcessingAPI.getDeployedService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void addDeployedService(ActivityDto dto) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.addDeployedService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void removeDeployedService(ActivityDto dto) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.removeDeployedService(dto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void updateContextModelWithMonitoredData(EACMInstance localEACMInstance) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.updateContextModelWithMonitoredData();
            EACMInstance remoteEACMInstance = contextAccessAndProcessingAPI.getCurrentContextInstance();
            for (ServiceCenterServer server : remoteEACMInstance.getAllServiceCenterServerInstances()) {

                ServiceCenterServer serverFromOntology = localEACMInstance.getServiceCenterServer(server.getName());

                Collection<CPU> cpus = server.getCpuResources();
                Iterator<CPU> cpuIterator = serverFromOntology.getCpuResources().iterator();
                for (CPU cpu : cpus) {
                    CPU localCPU = cpuIterator.next();
                    localCPU.setCurrentWorkLoad(cpu.getCurrentWorkLoad());
                }


                Collection<MEM> mems = server.getMemResources();
                Iterator<MEM> memIterator = serverFromOntology.getMemResources().iterator();
                for (MEM memory : mems) {
                    MEM localMEM = memIterator.next();
                    localMEM.setCurrentWorkLoad(memory.getCurrentWorkLoad());
                }

            }
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void persistContextSituation(ContextSituationDto contextSituationDto) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.persistHistoricalContextSituation(contextSituationDto);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }

    public void persistContextSituations(Collection<ContextSituationDto> contextSituationDtos) throws ApplicationException {
        try {
            contextAccessAndProcessingAPI.persistHistoricalContextSituations(contextSituationDtos);
        } catch (ContextModelAccessException e) {
            GlobalLoopLogger.error(e);
            throw new ApplicationException(e.toString(), e.getCause());
        }
    }
    
    
}
