/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.impl;

import java.util.ArrayList;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.utils.commonDtos.PhysicalHost;
import ro.utcn.dsrl.utils.commonDtos.ServerInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualNetworkInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;

import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class StubServiceCenterManagement implements ServiceCenterManagement {


    public List<VirtualTaskInfo> getPendingVirtualMachines() throws ServiceCenterAccessException {
        return  new OpenNebulaServiceCenterManagement().getPendingVirtualMachines();
    }

    public List<VirtualTaskInfo> getAllVirtualMachines() throws ServiceCenterAccessException {
        return new OpenNebulaServiceCenterManagement().getAllVirtualMachines();
    }

    public List<ServerInfo> getDefinedServers() throws ServiceCenterAccessException {
        return new OpenNebulaServiceCenterManagement().getDefinedServers();
    }

    public VirtualTaskInfo getVMInfo(Integer vmID) throws ServiceCenterAccessException {
        return new OpenNebulaServiceCenterManagement().getVMInfo(vmID);
    }

    public ServerInfo getServerInfo(PhysicalHost physicalHost)
            throws ServiceCenterAccessException {
        return new OpenNebulaServiceCenterManagement().getServerInfo(physicalHost);
    }

    public VirtualTaskInfo createAndDeployVirtualMachine(VirtualTaskInfo infoVirtual, PhysicalHost physicalHost) throws ServiceCenterAccessException {
       return new OpenNebulaServiceCenterManagement().createAndDeployVirtualMachine(infoVirtual, physicalHost);
    }

    public VirtualTaskInfo createVirtualMachine(VirtualTaskInfo infoVirtual) throws ServiceCenterAccessException {
       return new OpenNebulaServiceCenterManagement().createVirtualMachine(infoVirtual);
    }


    public VirtualTaskInfo deployVirtualMachine(VirtualTaskInfo infoVirtual,
                                                PhysicalHost physicalHost) throws ServiceCenterAccessException {
         VirtualNetworkInfo virtualNetworkInfo = new VirtualNetworkInfo();
        virtualNetworkInfo.setId(0);
      infoVirtual.setId(0);
        infoVirtual.setNetworkInfo(virtualNetworkInfo);
        return infoVirtual;
    }

    public void removeVirtualMachine(VirtualTaskInfo infoVirtual) throws ServiceCenterAccessException {
        new OpenNebulaServiceCenterManagement().removeVirtualMachine(infoVirtual);
    }


    public void startVirtualMachine(VirtualTaskInfo info)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void stopVirtualMachine(VirtualTaskInfo info)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void migrateVirtualMachine(VirtualTaskInfo info,
                                      PhysicalHost destination) throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public PhysicalHost addHost(PhysicalHost host)
            throws ServiceCenterAccessException {
        host.setId(0);
        return host;
    }


    public void removeHost(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void enableHost(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void disableHost(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void deleteVirtualMachine(VirtualTaskInfo info)
            throws ServiceCenterAccessException {
       new OpenNebulaServiceCenterManagement().deleteVirtualMachine(info);

    }


    public void wakeUpServer(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public void sendServerToSleep(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }


    public String getEnergyConsumptionInfo(PhysicalHost host)
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub
        return "90";
    }


    public void restartInitialVM(String ip) {
        // TODO Auto-generated method stub

    }


    public void terminateRunningOperations()
            throws ServiceCenterAccessException {
        // TODO Auto-generated method stub

    }

    public boolean serverIsAlive(String ip) throws ServiceCenterAccessException {
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public void terminate() {
        // TODO Auto-generated method stub

    }

}
