/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionPlanningModule.agents;

import ro.utcn.dsrl.actionPlanningModule.agents.behaviors.APAPlanningBehavior;
import ro.utcn.dsrl.actionPlanningModule.agents.behaviors.APAReceiveMessageBehavior;
import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.ContextEntropyEvaluator;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import jade.core.Agent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class ActionPlanningAgent extends Agent implements Interruptable {

    private EACMInstance eacmInstance;
    private PlanningEngine planningEngine;
    private APAPlanningBehavior planningBehavior;
    private ContextEntropyEvaluator entropyEvaluator;
    private CAPInteractions capInteractions;

    private Thread currentlyRunningThread;


    protected void setup() {
        Object[] args = getArguments();
        if (args != null && args.length > 3) {
            eacmInstance = (EACMInstance) args[0];
            entropyEvaluator = (ContextEntropyEvaluator) args[1];
            planningEngine = (PlanningEngine) args[2];
            capInteractions = (CAPInteractions) args[3];
        } else {
            GlobalLoopLogger.getLogger().info("ActionPlanningAgent not enoguh parameters received.Terminating");
            this.takeDown();
        }

        addBehaviour(new APAReceiveMessageBehavior(this, eacmInstance));
        GlobalLoopLogger.getLogger().info("PA started");
    }

    public void generateActionPlan() {
        planningBehavior = new APAPlanningBehavior(this, eacmInstance, entropyEvaluator, planningEngine,capInteractions);
        planningBehavior.action();
    }

    public void terminate() {
        if (currentlyRunningThread != null && currentlyRunningThread.isAlive()) {
            currentlyRunningThread.interrupt();
            planningBehavior.terminate();
            try {
                currentlyRunningThread.join();
            } catch (InterruptedException e) {
                GlobalLoopLogger.error(e);
            }
        }

    }
}
