/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionPlanningModule.reasoning.negotiation.impl;

import ro.utcn.dsrl.actionPlanningModule.reasoning.negotiation.Negotiator;
import ro.utcn.dsrl.utils.misc.GlobalVars;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


/**
 * Class which implements Factory Method design pattern.
 * Used to create diferent Negotiator instances.
 * The creator methods are synchronized because only one instance of each utils.negotiator is
 * allowed. There is no need to be more negotiators of the same type because they are generic
 * utility classes.
 */
public final class NegotiatorFactory {
    private static NashNegotiator nashNegotiator;

    private NegotiatorFactory() {
    }

    public static synchronized Negotiator getNashNegotiator() {
        NashNegotiator negotiator = nashNegotiator;
        if (negotiator == null) {
            nashNegotiator = new NashNegotiator();
            negotiator = nashNegotiator;
        }
        return negotiator;
    }

    
}
