/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.actionPlanningModule.utils.actionExecutionSimulation.impl;

import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.actionPlanningModule.utils.actionExecutionSimulation.IActionSimulationUtil;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultDeployActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultMigrateActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultRemoveActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultSetServerStateActivity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Queue;

/**
 * Created by IntelliJ IDEA.
 * User: oneadmin
 * Date: 11/8/11
 * Time: 1:59 PM
 * To change this template use File | Settings | File Templates.
 */
public class DefaultActionSimulationUtil implements IActionSimulationUtil {

    private PlanningEngine planningEngine;

    public DefaultActionSimulationUtil(PlanningEngine planningEngine) {
        this.planningEngine = planningEngine;
    }

    public void simulateActionExecution(EACMInstance eacmInstance, ContextAction contextAction) {

        //a lot of if-then-else to check if in Cluster mode
        //there are sub-actions which need to be performed or :-@
        if (contextAction instanceof DefaultDeployActivity) {
            DefaultDeployActivity deployActivityAction = (DefaultDeployActivity) contextAction;
            ApplicationActivity applicationActivity = deployActivityAction.getActivity();
            ServiceCenterServer destinationServer = (ServiceCenterServer) deployActivityAction.getResourceTo();
            if (destinationServer.isCluster()) {
                if (!deployActivityAction.hasExecutedSubActions()) {

                    Collection<ServiceCenterITComputingResource> serverResources = destinationServer.getResources();
                    Collection<ApplicationActivity> runningActivities = destinationServer.getRunningActivities();

                    Collection<GPI_KPI_Policy> policiesForEvaluation = new ArrayList<GPI_KPI_Policy>();

                    for (ServiceCenterITComputingResource serviceCenterServer : serverResources) {
                        policiesForEvaluation.add(((ServiceCenterServer) serviceCenterServer).getServiceCenterServerPolicy());
                    }
                    for (ApplicationActivity runningActivity : runningActivities) {
                        policiesForEvaluation.add((GPI_KPI_Policy) runningActivity.getActivityPolicies().iterator().next());
                    }
//                Collection<QoSPolicy> qoSPolicies = modelAccess.getAllQoSPolicyInstances();
//                for (QoSPolicy qoSPolicy : qoSPolicies) {
//                    if (!((ApplicationActivity) qoSPolicy.getPolicySubject().iterator().next()).isRunning()) {
//                        policiesForEvaluation.add(qoSPolicy);
//                    }
//                }
                    policiesForEvaluation.add((GPI_KPI_Policy) applicationActivity.getActivityPolicies().iterator().next());
//                if (planningEngine == null) {
//                    return;
//                }
                    Queue<ContextAction> contextActions = planningEngine.generateActionPlan(policiesForEvaluation).getActions();
                    deployActivityAction.setExecutedSubActions(contextActions);
                    for (ContextAction subAction : contextActions) {
                        subAction.execute(eacmInstance);
//                    if (contextAction instanceof DeployActivity) {
//                        ApplicationActivity aa = ((DeployActivity) contextAction).getActivity();
//                        synchronized (this) {
//                            if (!server.getRunningActivities().contains(activity)) {
//                                server.addRunningActivity(aa);
//                            }
//                        }
//                    }
                    }
                } else {
                    for (ContextAction subAction : deployActivityAction.getExecutedSubActions()) {
                        contextAction.execute(eacmInstance);
//                    if (contextAction instanceof DeployActivity) {
//                        ApplicationActivity aa = ((DeployActivity) contextAction).getActivity();
//                        synchronized (this) {
//                            if (!server.getRunningActivities().contains(activity)) {
//                                server.addRunningActivity(aa);
//                            }
//                        }
//                    }
                    }
                }
            }
        } else if (contextAction instanceof DefaultRemoveActivity) {
            DefaultRemoveActivity defaultRemoveActivity = (DefaultRemoveActivity) contextAction;
            ApplicationActivity applicationActivity = defaultRemoveActivity.getActivity();
            ServiceCenterServer destinationServer = (ServiceCenterServer) defaultRemoveActivity.getResources().toArray()[0];
            if (destinationServer.isCluster()) {
                if (!defaultRemoveActivity.hasExecutedSubActions()) {
                    Collection<ServiceCenterITComputingResource> serverResources = destinationServer.getResources();
                    Collection<ServiceCenterServer> attachedComplexResources = new ArrayList<ServiceCenterServer>(serverResources.size());
                    for (ContextResource contextResource : serverResources) {
                        if (contextResource instanceof ServiceCenterServer) {
                            attachedComplexResources.add((ServiceCenterServer) contextResource);
                        }
                    }


                    for (ServiceCenterServer serviceCenterServer : attachedComplexResources) {
                        if (serviceCenterServer.hasRunningActivities() && serviceCenterServer.getRunningActivities().contains(applicationActivity)) {
                            RemoveActivity removeActivity = eacmInstance.createRemoveActivity("Remove "
                                    + applicationActivity.getName() + "_from_"
                                    + serviceCenterServer.getName());
                            removeActivity.addResource(serviceCenterServer);
                            removeActivity.setActivity(applicationActivity);
                            removeActivity.execute(eacmInstance);
                            defaultRemoveActivity.addExecutedSubActions(removeActivity);
                            if (serviceCenterServer.isCluster()) {
                                Collection<GPI_KPI_Policy> policiesForEvaluation = new ArrayList<GPI_KPI_Policy>();
                                for (ServiceCenterITComputingResource s : serviceCenterServer.getResources()) {
                                    policiesForEvaluation.add(((ServiceCenterServer) s).getServiceCenterServerPolicy());
                                }

                                Queue<ContextAction> contextActions = planningEngine.generateActionPlan(policiesForEvaluation).getActions();
                                defaultRemoveActivity.setExecutedSubActions(contextActions);
                                for (ContextAction subAction : contextActions) {
                                    subAction.execute(eacmInstance);
                                }
                            }
                            break;
                        }
                    }
                } else {
                    for (ContextAction subAction : defaultRemoveActivity.getExecutedSubActions()) {
                        subAction.execute(eacmInstance);
                    }

                }
            }

        } else if (contextAction instanceof DefaultMigrateActivity) {
            DefaultMigrateActivity defaultMigrateActivity = (DefaultMigrateActivity) contextAction;
            ApplicationActivity applicationActivity = defaultMigrateActivity.getActivity();
            ServiceCenterServer sourceServer = (ServiceCenterServer) defaultMigrateActivity.getResourceFrom();
            ServiceCenterServer destinationServer = (ServiceCenterServer) defaultMigrateActivity.getResourceTo();
            if (!defaultMigrateActivity.hasExecutedSubActions()) {

                //remove activity part
                RemoveActivity removeActivity = eacmInstance.createRemoveActivity("Remove "
                        + applicationActivity.getName() + "_from_"
                        + sourceServer.getName());     //:)))
                removeActivity.addResource(sourceServer);
                removeActivity.setActivity(applicationActivity);
                this.simulateActionExecution(eacmInstance, removeActivity);
                defaultMigrateActivity.addExecutedSubActions(removeActivity);

                //deploy part
                DeployActivity deployActivity = eacmInstance.createDeployActivity("Deploy "
                        + applicationActivity.getName() + "_to_"
                        + destinationServer.getName());
                deployActivity.addResource(destinationServer);
                deployActivity.setActivity(applicationActivity);
                this.simulateActionExecution(eacmInstance, removeActivity);
                defaultMigrateActivity.addExecutedSubActions(deployActivity);

            } else {
                for (ContextAction subAction : defaultMigrateActivity.getExecutedSubActions()) {
                    subAction.execute(eacmInstance);
                }

            }
        } else if (contextAction instanceof DefaultSetServerStateActivity){
             DefaultSetServerStateActivity defaultSetServerStateActivity = (DefaultSetServerStateActivity) contextAction;

        }

        //no matter what action, it must be executed. I guess
        
        contextAction.execute(eacmInstance);
    }

    public void undoActionSimulatedExecution(EACMInstance eacmInstance, ContextAction contextAction) {
        contextAction.undo(eacmInstance);
    }
}
