/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.workloadScheduler.workloadGenerator;

import jade.core.Agent;
import ro.utcn.dsrl.utils.exposedFunctionality.jaxbBindingClasses.*;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import javax.xml.bind.JAXB;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class FileInputWorkloadGenerator extends GenericWorkloadGenerator implements WorkloadGeneratorI {

    private static final DateFormat dateFormat = new SimpleDateFormat("_dd_MM_HH_mm_");
    private static int BEGINNING_ARRIVAL = 0;
    //interval at which new tasks are generated in seconds
    private static int WORKLOAD_GENERATION_INTERVAL_MAX = 400;
    private static int WORKLOAD_GENERATION_INTERVAL_MIN = 200;

    //the minimum and maximum value for the workload duration(task running time until the task terminates) in seconds
    private static int TASK_DURATION_INTERVAL_MAX = 600;
    private static int TASK_DURATION_INTERVAL_MIN = 320;

    //number of tasks to be generated at one time
    private static int NUMBER_OF_GENERATED_TASKS_MAX = 10;
    private static int NUMBER_OF_GENERATED_TASKS_MIN = 3;

    private Random random = new Random();

    private int generatedTaskIndex = 0;

    private Agent agent;

    private boolean pauseGenerating = false;

    private File generatedSchedule;
    private WorkloadSchedule workloadSchedule;
    private Date agentCreationDate;
    private Map<Long, List<ApplicationDescription>> generatedScheduleMap;

    private long previousKey = 0;


    public FileInputWorkloadGenerator(String path) {
        generatedSchedule = new File(path);

        workloadSchedule = new WorkloadSchedule();
        workloadSchedule.setApplications(new JaxbPair[0]);
        generatedScheduleMap = new LinkedHashMap<Long, List<ApplicationDescription>>();

        try {
            workloadSchedule = JAXB.unmarshal(new FileInputStream(generatedSchedule),
                    WorkloadSchedule.class);
            JaxbPair[] schedule = workloadSchedule.getApplications();
            for (JaxbPair jaxbPair : schedule) {
                if (generatedScheduleMap.containsKey(jaxbPair.getStartDelay())) {
                    generatedScheduleMap.get(jaxbPair.getStartDelay()).add(jaxbPair.getApplication());
                } else {
                    List<ApplicationDescription> serializables = new ArrayList<ApplicationDescription>();
                    serializables.add(jaxbPair.getApplication());
                    generatedScheduleMap.put(jaxbPair.getStartDelay(), serializables);
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public Serializable[] generateTasks() {
//         if (pauseGenerating) {
//            return;
//        }
////
      if(generatedScheduleMap.size() == 0 ){
          GlobalLoopLogger.getLogger().info("Workload Input File parsed");
          System.exit(1);
      }
        Long firstKey = generatedScheduleMap.keySet().iterator().next();
        List<ApplicationDescription> content = generatedScheduleMap.remove(firstKey);

        try {
            GlobalLoopLogger.getLogger().info("Waiting for schedule for " + (firstKey - previousKey) / 1000 + " seconds");
            Thread.sleep(0);
        } catch (InterruptedException e) {
            GlobalLoopLogger.error(e);
        }

        previousKey = firstKey;
        List<ActivityDto> dtos = new ArrayList<ActivityDto>();

        for (ApplicationDescription applicationDescription : content) {
            for (ActivityDescription activityDescription : applicationDescription.getActivities()) {
                for (ActivityService activityService : activityDescription.getServices()) {
                    String serviceName = activityService.getServiceName();
                    try {
                        if (serviceName.contains("deleted")) {
                            ActivityDto activityDto = null;

                            activityDto = (ActivityDto) possibleTaskPool.get(serviceName.substring(0, serviceName.indexOf("deleted"))).clone();

                            activityDto.setIp(activityService.getIp());
                            activityDto.setServiceName(activityDto.getServiceName() + "deleted");
                            dtos.add(activityDto);
                        } else {
                            ActivityDto activityDto = (ActivityDto) possibleTaskPool.get(serviceName).clone();
                            activityDto.setIp(activityService.getIp());
                            dtos.add(activityDto);
                        }
                    } catch (CloneNotSupportedException e) {
                        GlobalLoopLogger.error(e);
                    }
                }
            }
        }


        Serializable[] generatedTasks = new Serializable[dtos.size()];
        generatedTasks = dtos.toArray(generatedTasks);
        return generatedTasks;


    }

}
