/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.workloadScheduler.workloadGenerator;

import jade.core.Agent;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.io.Serializable;
import java.util.*;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class RandomWorkloadGenerator extends GenericWorkloadGenerator implements WorkloadGeneratorI {

    private static int BEGINNING_ARRIVAL = 0;
    //interval at which new tasks are generated in milliseconds
    private static int WORKLOAD_GENERATION_INTERVAL_MAX = 3000;//00;
    private static int WORKLOAD_GENERATION_INTERVAL_MIN = 1500;//00;
    //the minimum and maximum value for the workload duration(task running time until the task terminates) in seconds
    private static int TASK_DURATION_INTERVAL_MAX = 600;
    private static int TASK_DURATION_INTERVAL_MIN = 320;
    //number of tasks to be generated at one time
    private static int NUMBER_OF_GENERATED_TASKS_MAX = 7;
    private static int NUMBER_OF_GENERATED_TASKS_MIN = 3;
    private Random random = new Random();
    private int generatedTaskIndex = 0;
    private Agent agent;
    private boolean pauseGenerating = false;
    private Date agentCreationDate;
    private List<ActivityDto> generatedScheduleList;
    private EACMInstance eacmInstance;

    public static void main(String[] args) {
        Random random = new Random();
        // RandomWorkloadGenerator randomWorkloadGenerator = new RandomWorkloadGenerator();
        for (int i = 0; i < 10; i++) {
            System.out.println(random.nextInt(2 + random.nextInt(2)));
        }
    }

    public RandomWorkloadGenerator(EACMInstance eacmInstance) {
        this.eacmInstance = eacmInstance;
        generatedScheduleList = new ArrayList<ActivityDto>();
        agentCreationDate = new Date();
    }

    public Serializable[] generateTasks() {
//        if (pauseGenerating) {
//            return;
//        }
////        final Timer timer = new Timer(true);
//
////        TimerTask task = new TimerTask() {
////            int executionCount = 0;
//
////            public void run() {

        int generatedTaskNo = random.nextInt(NUMBER_OF_GENERATED_TASKS_MAX - NUMBER_OF_GENERATED_TASKS_MIN) + NUMBER_OF_GENERATED_TASKS_MIN;
        System.out.println("Number of generated tasks is ________________ " + generatedTaskNo);
        ArrayList<ActivityDto> generatedList = new ArrayList<ActivityDto>(generatedTaskNo);

        int runningTasksCount = eacmInstance.getAllApplicationActivityInstances().size();

        int randomRangeMax = 0;

        if (runningTasksCount > 11) {
            randomRangeMax = 1;
        } else if (runningTasksCount > 7) {
            randomRangeMax = 2;
        } else if (runningTasksCount > 4) {
            randomRangeMax = 3;
        } else {
            randomRangeMax = 4;
        }
        int numberOfGeneratedTaks = generatedScheduleList.size();
        
        //odata delete, apoi add, apoi delete, sa nu se incurce 
       
         
        for (int i = 0; i < generatedTaskNo; i++) {
            ActivityDto generated = null;
            int currentRand = random.nextInt(randomRangeMax);
            System.out.println("Random_ " + currentRand);
            if (currentRand == 0 && numberOfGeneratedTaks > 0) {
                int randomEl = random.nextInt(numberOfGeneratedTaks);
                ActivityDto activityDtoToBeDeleted = generatedScheduleList.get(randomEl);
                ApplicationActivity applicationActivity = eacmInstance.getApplicationActivity(activityDtoToBeDeleted.getTaskName() + "_" + activityDtoToBeDeleted.getIp());
                if (applicationActivity != null && applicationActivity.isRunning()) {

                    generated = generatedScheduleList.remove(randomEl);
                    generated.setServiceName(generated.getServiceName() + "deleted");
                    GlobalLoopLogger.getLogger().info("Generating to delete " + generated.getServiceName());
                    generatedList.add(generated);
                    numberOfGeneratedTaks--;
                }
            } else {

                int maximumTaskType = possibleTaskPool.size();
                // for (int i = 0; i < generatedTaskNo; i++) {
                int generatedTaskTypeListIndex = random.nextInt(maximumTaskType);
                int timeToLive = random.nextInt(TASK_DURATION_INTERVAL_MAX - TASK_DURATION_INTERVAL_MIN) + TASK_DURATION_INTERVAL_MIN;


                try {
                    generated = (ActivityDto) (new ArrayList<ActivityDto>(possibleTaskPool.values()).get(generatedTaskTypeListIndex)).clone();
                } catch (CloneNotSupportedException e) {
                    GlobalLoopLogger.error(e);
                    e.printStackTrace();
                }

                int generatedTaskCount = generatedTaskIndex++;
                generated.setTaskName(generated.getTaskName() + "_" + generatedTaskCount);
                String ip = "192.168.1." + (generatedTaskCount + 100);//TODO mai generic ipu
                //                    generated.setTaskName(generated.getTaskName() + "_Generated_Index_" + generatedTaskCount);
                generated.setIp(ip);
                generated.setVncPort(generatedTaskCount);
                generated.setLife(timeToLive);
                generatedScheduleList.add(generated);
                generatedList.add(generated);

            }

        }
        Serializable[] generatedTasks = null;
        int generatedCount = generatedList.size();
        if (generatedCount > 0) {
            generatedTasks = new Serializable[]{generatedCount};
            generatedTasks = generatedList.toArray(generatedTasks);
        } else {
            generatedTasks = new Serializable[0];
        }
        return generatedTasks;
    }
}
