/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.impl;

import edu.stanford.smi.protege.exception.OntologyLoadException;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.ContextAccessAndProcessingAPI;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.modelAPI.OntologyEACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.ContextModelAccessException;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.IndividualNotFoundException;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.utils.Pair;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.utils.PolicyEvaluator;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.utils.knowledgeBaseInterraction.impl.ContextSituationAccess;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class ContextAccessAndProcessing implements ContextAccessAndProcessingAPI {

    private OntologyEACMInstance currentContextInstanceOntology;

    public ContextAccessAndProcessing() {
        try {
            createContextInstance();
        } catch (Exception e) {
            GlobalLoopLogger.getLogger().debug(e.toString());
        }
    }

    /* public List<ActivityDto> getAllServices() throws ContextModelAccessException {
    List<ActivityDto> activityDtos ;
    try {
    activityDtos = kbInteraction.getAllServices();
    } catch (RemoteException e) {
    throw new ContextModelAccessException(e.toString(), e.getCause());
    }
    return activityDtos;
    } */
    /**
     * @param resource     - Resource to compute workload for.
     *                     The resource <b>name</b> is used for finding it in the OntologyEACMInstance.
     * @param eacmInstance
     * @return
     * @throws ContextModelAccessException
     */
    public Double getLoadForSimpleResource(EACMInstance eacmInstance, SimpleResource resource)
            throws ContextModelAccessException {
        SimpleResource eacmResource = eacmInstance.getSimpleResource(resource.getName());
        if (eacmResource == null) {
            throw new ContextModelAccessException("SimpleResource " + resource.getName()
                    + " not found in EACM Instance");
        }
        return eacmResource.getCurrentWorkLoad();
    }

    public Double getLoadForSimpleResource(EACMInstance eacmInstance, String resourceName)
            throws ContextModelAccessException {
        SimpleResource eacmResource = eacmInstance.getSimpleResource(resourceName);
        if (eacmResource == null) {
            throw new ContextModelAccessException("SimpleResource " + resourceName
                    + " not found in EACM Instance");
        }
        return eacmResource.getCurrentWorkLoad();
    }

    /**
     * @param complexResource Resource to compute workload for.
     *                        The resource <b>name</b> is used for finding it in the OntologyEACMInstance.
     * @param eacmInstance
     * @return
     * @throws ContextModelAccessException
     */
    public List<Pair<SimpleResource, Double>> getLoadForComplexResource(EACMInstance eacmInstance, ComplexResource complexResource)
            throws ContextModelAccessException {
        ComplexResource eacmComplexResource = eacmInstance.getComplexResource(complexResource.getName());
        if (eacmComplexResource == null) {
            throw new ContextModelAccessException("ComplexResource " + eacmComplexResource.getName() + " not found in EACM Instance");
        }
        List<Pair<SimpleResource, Double>> complexResourceWorkload = new ArrayList<Pair<SimpleResource, Double>>();
        for (ServiceCenterITComputingResource simpleResource : eacmComplexResource.getResources()) {
            complexResourceWorkload.add(new Pair<SimpleResource, Double>((SimpleResource) simpleResource, simpleResource.getCurrentWorkLoad()));
        }
        return complexResourceWorkload;
    }

    public List<Pair<SimpleResource, Double>> getLoadForComplexResource(EACMInstance eacmInstance, String complexResourceName)
            throws ContextModelAccessException {
        ComplexResource eacmComplexResource = eacmInstance.getComplexResource(complexResourceName);
        if (eacmComplexResource == null) {
            throw new ContextModelAccessException("ComplexResource " + complexResourceName + " not found in EACM Instance");
        }
        List<Pair<SimpleResource, Double>> complexResourceWorkload = new ArrayList<Pair<SimpleResource, Double>>();
        for (ServiceCenterITComputingResource simpleResource : eacmComplexResource.getResources()) {
            complexResourceWorkload.add(
                    new Pair<SimpleResource, Double>(
                    (SimpleResource) simpleResource, simpleResource.getCurrentWorkLoad()));
        }
        return complexResourceWorkload;
    }

    public Boolean evaluateMetricDH_UE(EACMInstance eacmInstance, Number threshold)
            throws ContextModelAccessException {
        Collection<ServiceCenterServer> serverCollection =
                eacmInstance.getAllServiceCenterServerInstances();
        return serverCollection.size() <= threshold.intValue();
    }

    public Boolean evaluateMetricDH_UR(EACMInstance eacmInstance, Number threshold)
            throws ContextModelAccessException {

        PolicyEvaluator policyEvaluator = new PolicyEvaluator(((OntologyEACMInstance) eacmInstance).getOwlModel().getOntModel());

        Collection<ServiceCenterServer> serverCollection =
                eacmInstance.getAllServiceCenterServerInstances();
        int runningServers = 0;
        int usefulRunningServers = 0;
        for (ServiceCenterServer server : serverCollection) {
            if (server.getIsActive()) {
                runningServers++;
            }
            try {
                //evaluate serverIsUseful using SWRL
                if (policyEvaluator.evaluateProperty(server, eacmInstance.getIsServerUsefulProperty())) {
                    usefulRunningServers++;
                }
            } catch (IndividualNotFoundException e) {
                throw new ContextModelAccessException(e.toString(), e.getCause());
            }
        }
        return (runningServers == 0)
                || (runningServers / usefulRunningServers < threshold.doubleValue());

    }

    public void setGPI_KPI_Threshold(EACMInstance instance, GPI_KPI_Policy policy,
            Number thresholdMinValue, Number thresholdMaxValue) throws ContextModelAccessException {
        policy.setGPI_KPI_Threshold_Max(thresholdMaxValue.floatValue());
        policy.setGPI_KPI_Threshold_Min(thresholdMinValue.floatValue());
    }

    public void setGPI_KPI_Threshold(EACMInstance instance, String policyName,
            Number thresholdMinValue, Number thresholdMaxValue) throws ContextModelAccessException {
        GPI_KPI_Policy gpi_kpi_Policy = instance.getGPI_KPI_Policy(policyName);
        if (gpi_kpi_Policy == null) {
            throw new ContextModelAccessException("GPI_KPI policy \"" + policyName
                    + "\" does not exist");
        }
        gpi_kpi_Policy.setGPI_KPI_Threshold_Max(thresholdMaxValue.floatValue());
        gpi_kpi_Policy.setGPI_KPI_Threshold_Min(thresholdMinValue.floatValue());
    }

    public void setGPI_KPI_Value(EACMInstance instance, GPI_KPI_Policy policy,
            Number value) throws ContextModelAccessException {
        policy.setGPI_KPI_value(value.floatValue());
    }

    public void setGPI_KPI_Value(EACMInstance instance, String policyName,
            Number value) throws ContextModelAccessException {
        GPI_KPI_Policy gpi_kpi_Policy = instance.getGPI_KPI_Policy(policyName);
        if (gpi_kpi_Policy == null) {
            throw new ContextModelAccessException("GPI_KPI policy \"" + policyName
                    + "\" does not exist");
        }
        gpi_kpi_Policy.setGPI_KPI_value(value.floatValue());
    }

    public EACMInstance getCurrentContextInstance() {
        return currentContextInstanceOntology;
    }

    public EACMInstance loadContextInstance(String ontologyFileServerIP)
            throws ContextModelAccessException {
        String ontologyFile = "./ontology/EnergyAwareContextModel/EACM.owl";
        File file = new File(ontologyFile);
        OWLModel model = null;
        try {
            model = ProtegeOWL.createJenaOWLModelFromURI(file.toURI().toString());
        } catch (OntologyLoadException e) {
            GlobalLoopLogger.error(e);
            GlobalLoopLogger.getLogger().info(e.toString());
            System.exit(1);
        }
        return new OntologyEACMInstance(model);
    }

    public EACMInstance createContextInstance()
            throws ContextModelAccessException {
        currentContextInstanceOntology = new OntologyEACMInstance();
        return currentContextInstanceOntology;
    }

    public void saveContextInstance(String saveDirectoryPath)
            throws ContextModelAccessException, ApplicationException {
        File dir = new File(saveDirectoryPath);
        if (!dir.exists() || !dir.isDirectory()) {
            throw new ContextModelAccessException("Path \"" + saveDirectoryPath
                    + "\" is not a valid directory.");
        }

        try {
            File file = new File(dir.getPath() + "/EACMInstance_" + new SimpleDateFormat("HH:mm:ss__dd:MM:yy").format(new java.util.Date()).replaceAll(":", "_"));
            FileOutputStream fileOutputStream = new FileOutputStream(file);
            currentContextInstanceOntology.getOwlModel().getOntModel().write(fileOutputStream);
            fileOutputStream.flush();
            fileOutputStream.close();
        } catch (IOException e) {
            throw new ContextModelAccessException(e.toString(), e.getCause());
        }
    }

    public EACMInstance createContextInstance(String path) throws ContextModelAccessException {
        OWLModel owlModel = null;
        File file = new File(path);
        try {
            owlModel = ProtegeOWL.createJenaOWLModelFromURI(file.toURI().toString());
            return new OntologyEACMInstance(owlModel);

        } catch (OntologyLoadException ex) {
            throw new ContextModelAccessException(ex.toString(), ex.getCause());
        }
    }

    public void saveContextInstance(String path, EACMInstance instance) throws ContextModelAccessException {
        try {
            File file = new File(path);
            FileOutputStream fileOutputStream = new FileOutputStream(file);
            ((OntologyEACMInstance) instance).getOwlModel().getOntModel().write(fileOutputStream);
            fileOutputStream.flush();
            fileOutputStream.close();
        } catch (IOException e) {
            throw new ContextModelAccessException(e.toString(), e.getCause());
        }
    }

    public ActivityDto getService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public void addService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public void removeService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public ActivityDto getDeployedService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public void addDeployedService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }
 
    public void removeDeployedService(ActivityDto dto) throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public void updateContextModelWithMonitoredData() throws ContextModelAccessException {
        throw new UnsupportedOperationException("Not supported in OpenNebula scheduler mode");
    }

    public void refreshContextInstance(EACMInstance instance) throws ContextModelAccessException {
        createContextInstance();
        instance = currentContextInstanceOntology;
    }

    public Collection<ContextSituationDto> getHistoricalContextSituations() throws ContextModelAccessException {
        return ContextSituationAccess.getSituations();
    }

    public void persistHistoricalContextSituations(Collection<ContextSituationDto> contextSituationDtos) throws ContextModelAccessException {
        try {
            ContextSituationAccess.saveContextSituations(contextSituationDtos);
        } catch (IOException e) {
            throw new ContextModelAccessException(e.toString(), e.getCause());
        }

    }

    public void persistHistoricalContextSituation(ContextSituationDto contextSituationDto) throws ContextModelAccessException {
        try {
            ContextSituationAccess.saveContextSituation(contextSituationDto);
        } catch (IOException e) {
            throw new ContextModelAccessException(e.toString(), e.getCause());
        }

    }
}