/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;

import java.util.Collection;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public interface EACMInstance {

    String getIsServerUsefulProperty();

    RemoveActivity createRemoveActivity(String name);

    RemoveActivity getRemoveActivity(String name);

    Collection<RemoveActivity> getAllRemoveActivityInstances();


    ServerAdaptationAction createServerAdaptationAction(String name);

    ContextElement getContextElement(String name);

    Collection<ContextElement> getAllContextElementInstances();

    Core createCore(String name);

    Core getCore(String name);

    Collection<Core> getAllCoreInstances();

    SetServerStateActivity createSetServerStateActivity(String name);

    SetServerStateActivity getSetServerStateActivity(String name);

    Collection<SetServerStateActivity> getAllSetServerStateActivityInstances();

    FacilityDefaultAction createFacilityDefaultAction(java.lang.String name);

    DPMAction createDPMAction(java.lang.String name);

    DPMAction getDPMAction(java.lang.String name);

    Collection<DPMAction> getAllDPMActionInstances();

    ITComputingResourceAdaptationAction createITComputingResourceAdaptationAction(java.lang.String name);

    ITComputingResourceAdaptationAction getITComputingResourceAdaptationAction(java.lang.String name);

    Collection<ITComputingResourceAdaptationAction> getAllITComputingResourceAdaptationActionInstances();

    HDDIntensiveActivity createHDDIntensiveActivity(java.lang.String name);

    HDDIntensiveActivity getHDDIntensiveActivity(java.lang.String name);

    Collection<HDDIntensiveActivity> getAllHDDIntensiveActivityInstances();

    ApplicationActivity createApplicationActivity(java.lang.String name);

    ApplicationActivity getApplicationActivity(java.lang.String name);

    Collection<ApplicationActivity> getAllApplicationActivityInstances();

    GPI_KPI_Policy createGPI_KPI_Policy(java.lang.String name);

    GPI_KPI_Policy getGPI_KPI_Policy(java.lang.String name);

    Collection<GPI_KPI_Policy> getAllGPI_KPI_PolicyInstances();

    ContextPolicy createContextPolicy(java.lang.String name);

    ContextPolicy getContextPolicy(java.lang.String name);

    Collection<ContextPolicy> getAllContextPolicyInstances();

    ServiceCenterITComputingResource getServiceCenterITComputingResource(java.lang.String name);

    Collection<ServiceCenterITComputingResource> getAllServiceCenterITComputingResourceInstances();

    ContextResource getContextResource(java.lang.String name);

    Collection<ContextResource> getAllContextResourceInstances();

    ServiceCenterServer createServiceCenterServer(java.lang.String name);

    ServiceCenterServer getServiceCenterServer(java.lang.String name);

    Collection<ServiceCenterServer> getAllServiceCenterServerInstances();

    ComplexResource getComplexResource(java.lang.String name);

    Collection<ComplexResource> getAllComplexResourceInstances();

    ServiceCenterITFacilityResource createServiceCenterITFacilityResource(java.lang.String name);

    ServiceCenterITFacilityResource getServiceCenterITFacilityResource(java.lang.String name);

    Collection<ServiceCenterITFacilityResource> getAllServiceCenterITFacilityResourceInstances();

    BusinessContextResource createBusinessContextResource(java.lang.String name);

    BusinessContextResource getBusinessContextResource(java.lang.String name);

    Collection<BusinessContextResource> getAllBusinessContextResourceInstances();

    Application createApplication(java.lang.String name);

    Application getApplication(java.lang.String name);

    Collection<Application> getAllApplicationInstances();

    CPUIntensiveActivity createCPUIntensiveActivity(java.lang.String name);

    CPUIntensiveActivity getCPUIntensiveActivity(java.lang.String name);

    Collection<CPUIntensiveActivity> getAllCPUIntensiveActivityInstances();

    ApplicationAdaptationAction createApplicationAdaptationAction(java.lang.String name);

    ApplicationAdaptationAction getApplicationAdaptationAction(java.lang.String name);

    Collection<ApplicationAdaptationAction> getAllApplicationAdaptationActionInstances();

    ApplicationRedesign createApplicationRedesign(java.lang.String name);

    ApplicationRedesign getApplicationRedesign(java.lang.String name);

    Collection<ApplicationRedesign> getAllApplicationRedesignInstances();

    ITFacilityActiveResource getITFacilityActiveResource(java.lang.String name);

    Collection<ITFacilityActiveResource> getAllITFacilityActiveResourceInstances();

    HDD createHDD(java.lang.String name);

    HDD getHDD(java.lang.String name);

    Collection<HDD> getAllHDDInstances();

    SimpleResource getSimpleResource(java.lang.String name);

    Collection<SimpleResource> getAllSimpleResourceInstances();

    ITComputingContextPolicy createITComputingContextPolicy(java.lang.String name);

    ITComputingContextPolicy getITComputingContextPolicy(java.lang.String name);

    Collection<ITComputingContextPolicy> getAllITComputingContextPolicyInstances();

    ITFacilityPassiveResource getITFacilityPassiveResource(java.lang.String name);

    Collection<ITFacilityPassiveResource> getAllITFacilityPassiveResourceInstances();

    MigrateActivity createMigrateActivity(java.lang.String name);

    MigrateActivity getMigrateActivity(java.lang.String name);

    Collection<MigrateActivity> getAllMigrateActivityInstances();

    ConsolidationAction createConsolidationAction(java.lang.String name);

    ConsolidationAction getConsolidationAction(java.lang.String name);

    Collection<ConsolidationAction> getAllConsolidationActionInstances();

    Sensor createSensor(java.lang.String name);

    Sensor getSensor(java.lang.String name);

    Collection<Sensor> getAllSensorInstances();

    Actuator createActuator(java.lang.String name);

    Actuator getActuator(java.lang.String name);

    Collection<Actuator> getAllActuatorInstances();

    BusinessPolicy createBusinessPolicy(java.lang.String name);

    BusinessPolicy getBusinessPolicy(java.lang.String name);

    Collection<BusinessPolicy> getAllBusinessPolicyInstances();

    MEM createMEM(java.lang.String name);

    MEM getMEM(java.lang.String name);

    Collection<MEM> getAllMEMInstances();

    ExternalStorage createExternalStorage(java.lang.String name);

    ExternalStorage getExternalStorage(java.lang.String name);

    Collection<ExternalStorage> getAllExternalStorageInstances();

    CPU createCPU(java.lang.String name);

    CPU getCPU(java.lang.String name);

    Collection<CPU> getAllCPUInstances();

    SLAPolicy createSLAPolicy(java.lang.String name);

    SLAPolicy getSLAPolicy(java.lang.String name);

    Collection<SLAPolicy> getAllSLAPolicyInstances();

    MEMIntensiveActivity createMEMIntensiveActivity(java.lang.String name);

    MEMIntensiveActivity getMEMIntensiveActivity(java.lang.String name);

    Collection<MEMIntensiveActivity> getAllMEMIntensiveActivityInstances();

    Facility createFacility(java.lang.String name);

    Facility getFacility(java.lang.String name);

    Collection<Facility> getAllFacilityInstances();

    QoSPolicy createQoSPolicy(java.lang.String name);

    QoSPolicy getQoSPolicy(java.lang.String name);

    Collection<QoSPolicy> getAllQoSPolicyInstances();

    DeployActivity createDeployActivity(java.lang.String name);

    DeployActivity getDeployActivity(java.lang.String name);

    Collection<DeployActivity> getAllDeployActivityInstances();

    EnvironmentPolicy createEnvironmentPolicy(java.lang.String name);

    EnvironmentPolicy getEnvironmentPolicy(java.lang.String name);

    Collection<EnvironmentPolicy> getAllEnvironmentPolicyInstances();

    ITFacilityResourceAdaptationAction createITFacilityResourceAdaptationAction(java.lang.String name);

    ITFacilityResourceAdaptationAction getITFacilityResourceAdaptationAction(java.lang.String name);

    Collection<ITFacilityResourceAdaptationAction> getAllITFacilityResourceAdaptationActionInstances();

    ApplicationActivityNetworkInfo createApplicationActivityNetworkInfo(String name);

    ApplicationActivityNetworkInfo getApplicationActivityNetworkInfo(String name);

    Collection<ApplicationActivityNetworkInfo> getAllApplicationActivityNetworkInfoInstances();

    ApplicationActivityDiskInfo createApplicationActivityDiskInfo(String name);

    ApplicationActivityDiskInfo getApplicationActivityDiskInfo(String name);

    Collection<ApplicationActivityDiskInfo> getAllApplicationActivityDiskInfoInstances();
}
