/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts;

import edu.stanford.smi.protegex.owl.model.RDFProperty;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public interface ApplicationActivity extends BusinessContextResource {

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activityPolicies

    public String getName();

    Collection getActivityPolicies();

    boolean hasActivityPolicies();

    Iterator listActivityPolicies();

    void addActivityPolicies(BusinessPolicy newActivityPolicies);

    void removeActivityPolicies(BusinessPolicy oldActivityPolicies);

    void setActivityPolicies(Collection newActivityPolicies);

    boolean isRunning();

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf

    Collection getPartOf();

    RDFProperty getPartOfProperty();

    boolean hasPartOf();

    Iterator listPartOf();

    void addPartOf(ContextElement newPartOf);

    void removePartOf(ContextElement oldPartOf);

    void setPartOf(Collection newPartOf);

    void addReceivedCoreIndex(int i);

    void removeReceivedCoreIndex(int i);

    void removeAllReceivedCoreIndex();

    List<Integer> getReceivedCoreIndexes();


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedServer

    Collection getAssociatedServer();

    RDFProperty getAssociatedServerProperty();

    boolean hasAssociatedServer();

    Iterator listAssociatedServer();

    void addAssociatedServer(ServiceCenterServer newAssociatedServer);

    void removeAssociatedServer(ServiceCenterServer oldAssociatedServer);

    void setAssociatedServer(Collection newAssociatedServer);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#networkInfo

    ApplicationActivityNetworkInfo getNetworkInfo();

    RDFProperty getNetworkInfoProperty();

    void removeNetworkInfo(ApplicationActivityNetworkInfo activityNetworkInfo);

    boolean hasNetworkInfo();

    void setNetworkInfo(ApplicationActivityNetworkInfo newNetworkInfo);

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskInfo

    Collection getDiskInfo();

    RDFProperty getDiskInfoProperty();

    boolean hasDiskInfo();

    Iterator listDiskInfo();

    void addDiskInfo(ApplicationActivityDiskInfo newDiskInfo);

    void removeDiskInfo(ApplicationActivityDiskInfo oldDiskInfo);

    void setDiskInfo(Collection newDiskInfo);
}
