/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public interface ComplexResource extends ServiceCenterITComputingResource {

    List<String> getResourceWorkloadProperty();

    void setResourceWorkloadProperty(List<String> list);

    List<ServiceCenterITComputingResource> getResources();

    void setResources(List<ServiceCenterITComputingResource> list);

    void addResource(ServiceCenterITComputingResource resource);

    void removeResource(ServiceCenterITComputingResource resource);

    boolean isCluster();

//    RDFProperty getIsClusterProperty();

//    boolean hasIsCluster();

    void setIsCluster(boolean newIsCluster);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUWeight

    float getCPUWeight();


    boolean hasCPUWeight();

    void setCPUWeight(float newCPUWeight);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDDWeight

    float getHDDWeight();


    boolean hasHDDWeight();

    void setHDDWeight(float newHDDWeight);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEMWeight

    float getMEMWeight();


    boolean hasMEMWeight();

    void setMEMWeight(float newMEMWeight);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cpuResources

    Collection<CPU> getCpuResources();

    boolean hasCpuResources();

    void addCpuResource(CPU newCpuResources);

    void removeCpuResources(CPU oldCpuResources);

    void setCpuResources(Collection newCpuResources);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddResources

    Collection<HDD> getHddResources();


    boolean hasHddResources();

    Iterator listHddResources();

    void addHddResources(HDD newHddResources);

    void removeHddResources(HDD oldHddResources);

    void setHddResources(Collection newHddResources);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isActive

    boolean getIsActive();


    boolean hasIsActive();

    void setIsActive(boolean newIsActive);


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#memResources

    Collection<MEM> getMemResources();


    boolean hasMemResources();

    Iterator listMemResources();

    void addMemResource(MEM newMemResources);

    void removeMemResources(MEM oldMemResources);

    void setMemResources(Collection newMemResources);

    Iterator listResourceWorkloadProperty();

    void addResourceWorkloadProperty(String newResourceWorkloadProperty);

    void removeResourceWorkloadProperty(String oldResourceWorkloadProperty);

    void setResourceWorkloadProperty(Collection newResourceWorkloadProperty);

    boolean hasResourcesFor(ApplicationActivity task);

    boolean hasResourcesToBeNegotiatedFor(ApplicationActivity task);

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isRootCluster

    boolean isRootCluster();


    void setIsRootCluster(boolean newIsRootCluster);
}
