/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;

import java.util.Collection;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public interface ContextAction extends ContextElement {
    List<ContextResource> getResources();

    void setResources(List<ContextResource> resources);

    void addResource(ContextResource resource);

    void removeResource(ContextResource resource);

    /**
     * @param modelAccess
     **/
    void execute(EACMInstance modelAccess);

//    void executeOnServiceCenter(EACMInstance modelAccess) throws ApplicationException;

    void undo(EACMInstance modelAccess);

    void setCost(int cost);

    int getCost();

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#executedSubActions

    Collection<ContextAction> getExecutedSubActions();

    boolean hasExecutedSubActions();

    void addExecutedSubActions(ContextAction newExecutedSubActions);

    void removeExecutedSubActions(ContextAction oldExecutedSubActions);

    void setExecutedSubActions(Collection newExecutedSubActions);
}
