/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.Application;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;

import java.util.Collection;
import java.util.Iterator;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultApplication extends DefaultBusinessContextResource
        implements Application {

    public DefaultApplication(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultApplication() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BP_ActivityList

    public Collection getActivityList() {
        return getPropertyValuesAs(getActivityListProperty(), ApplicationActivity.class);
    }


    public RDFProperty getActivityListProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BP_ActivityList";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasActivityList() {
        return getPropertyValueCount(getActivityListProperty()) > 0;
    }


    public Iterator listActivityList() {
        return listPropertyValuesAs(getActivityListProperty(), ApplicationActivity.class);
    }


    public void addActivity(ApplicationActivity newActivityList) {
        addPropertyValue(getActivityListProperty(), newActivityList);
    }


    public void removeActivityList(ApplicationActivity oldActivityList) {
        removePropertyValue(getActivityListProperty(), oldActivityList);
    }


    public void setActivityList(Collection newActivityList) {
        setPropertyValues(getActivityListProperty(), newActivityList);
    }
}
