/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextElement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.BusinessPolicy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivityDiskInfo;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivityNetworkInfo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultApplicationActivity extends DefaultBusinessContextResource
        implements ApplicationActivity {

    private List<Integer> receivedCoresIndex;

    {
        receivedCoresIndex = new ArrayList<Integer>();
    }

    public DefaultApplicationActivity(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultApplicationActivity() {
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUWeight
    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activityPolicies
    public Collection getActivityPolicies() {
        return getPropertyValuesAs(getActivityPoliciesProperty(), BusinessPolicy.class);
    }

    public RDFProperty getActivityPoliciesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activityPolicies";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasActivityPolicies() {
        return getPropertyValueCount(getActivityPoliciesProperty()) > 0;
    }

    public Iterator listActivityPolicies() {
        return listPropertyValuesAs(getActivityPoliciesProperty(), BusinessPolicy.class);
    }

    public void addActivityPolicies(BusinessPolicy newActivityPolicies) {
        addPropertyValue(getActivityPoliciesProperty(), newActivityPolicies);
    }

    public void removeActivityPolicies(BusinessPolicy oldActivityPolicies) {
        removePropertyValue(getActivityPoliciesProperty(), oldActivityPolicies);
    }

    public void setActivityPolicies(Collection newActivityPolicies) {
        setPropertyValues(getActivityPoliciesProperty(), newActivityPolicies);
    }

    public boolean isRunning() {
        return getCpuAllocatedValue() > 0;
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf
    public Collection getPartOf() {
        return getPropertyValuesAs(getPartOfProperty(), ContextElement.class);
    }

    public RDFProperty getPartOfProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasPartOf() {
        return getPropertyValueCount(getPartOfProperty()) > 0;
    }

    public Iterator listPartOf() {
        return listPropertyValuesAs(getPartOfProperty(), ContextElement.class);
    }

    public void addPartOf(ContextElement newPartOf) {
        addPropertyValue(getPartOfProperty(), newPartOf);
    }

    public void removePartOf(ContextElement oldPartOf) {
        removePropertyValue(getPartOfProperty(), oldPartOf);
    }

    public void setPartOf(Collection newPartOf) {
        setPropertyValues(getPartOfProperty(), newPartOf);
    }

    public void addReceivedCoreIndex(int i) {
        receivedCoresIndex.add(i);
    }

    public void removeReceivedCoreIndex(int i) {
        receivedCoresIndex.remove(i);
    }

    public void removeAllReceivedCoreIndex() {
        receivedCoresIndex.clear();
        ;
    }

    public List<Integer> getReceivedCoreIndexes() {
        return receivedCoresIndex;
    }

// Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedServer

    public Collection getAssociatedServer() {
        return getPropertyValuesAs(getAssociatedServerProperty(), ServiceCenterServer.class);
    }


    public RDFProperty getAssociatedServerProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedServer";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasAssociatedServer() {
        return getPropertyValueCount(getAssociatedServerProperty()) > 0;
    }


    public Iterator listAssociatedServer() {
        return listPropertyValuesAs(getAssociatedServerProperty(), ServiceCenterServer.class);
    }


    public void addAssociatedServer(ServiceCenterServer newAssociatedServer) {
        addPropertyValue(getAssociatedServerProperty(), newAssociatedServer);
    }


    public void removeAssociatedServer(ServiceCenterServer oldAssociatedServer) {
        removePropertyValue(getAssociatedServerProperty(), oldAssociatedServer);
    }


    public void setAssociatedServer(Collection newAssociatedServer) {
        setPropertyValues(getAssociatedServerProperty(), newAssociatedServer);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#networkInfo
    public ApplicationActivityNetworkInfo getNetworkInfo() {
        return (ApplicationActivityNetworkInfo) getPropertyValueAs(getNetworkInfoProperty(), ApplicationActivityNetworkInfo.class);
    }

    public RDFProperty getNetworkInfoProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#networkInfo";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasNetworkInfo() {
        return getPropertyValueCount(getNetworkInfoProperty()) > 0;
    }

    public void setNetworkInfo(ApplicationActivityNetworkInfo newNetworkInfo) {
        setPropertyValue(getNetworkInfoProperty(), newNetworkInfo);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskInfo
    public Collection getDiskInfo() {
        return getPropertyValuesAs(getDiskInfoProperty(), ApplicationActivityDiskInfo.class);
    }

    public RDFProperty getDiskInfoProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskInfo";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasDiskInfo() {
        return getPropertyValueCount(getDiskInfoProperty()) > 0;
    }

    public Iterator listDiskInfo() {
        return listPropertyValuesAs(getDiskInfoProperty(), ApplicationActivityDiskInfo.class);
    }

    public void addDiskInfo(ApplicationActivityDiskInfo newDiskInfo) {
        addPropertyValue(getDiskInfoProperty(), newDiskInfo);
    }

    public void removeDiskInfo(ApplicationActivityDiskInfo oldDiskInfo) {
        removePropertyValue(getDiskInfoProperty(), oldDiskInfo);
    }

    public void setDiskInfo(Collection newDiskInfo) {
        setPropertyValues(getDiskInfoProperty(), newDiskInfo);
    }


//    public boolean equals(Object o) {
//        if (this == o) {
//            return true;
//        }
//        if (o == null || getClass() != o.getClass()) {
//            return false;
//        }
//
//        DefaultApplicationActivity that = (DefaultApplicationActivity) o;
//
//        if (this.getName().equals(that.getName())) {
//            return true;
//        }
//        //TODO: de vazut care a fost motivatia sa fac equals pe cerinte si la taskuri. si sa incep sa comentez codu
////        if (this.getCpuRequiredMaxValue() == that.getCpuRequiredMaxValue()
////                && this.getCpuRequiredMinValue() == that.getCpuRequiredMinValue()
////                && this.getNumberOfCoresRequiredValue() == that.getNumberOfCoresRequiredValue()
////                && this.getMemRequiredMaxValue() == that.getMemRequiredMaxValue()
////                && this.getMemRequiredMinValue() == that.getMemRequiredMinValue()) {
////            return true;
////        }
//        return false;
//    }

    public void removeNetworkInfo(ApplicationActivityNetworkInfo activityNetworkInfo) {
        removePropertyValue(getNetworkInfoProperty(), activityNetworkInfo);
    }
}
