/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivityDiskInfo;

import java.util.Collection;
import java.util.Iterator;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultApplicationActivityDiskInfo extends DefaultBusinessContextResource
        implements ApplicationActivityDiskInfo {

    public DefaultApplicationActivityDiskInfo(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultApplicationActivityDiskInfo() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#applicationActivityDiskInfoProperties

    public Collection getApplicationActivityDiskInfoProperties() {
        return getPropertyValues(getApplicationActivityDiskInfoPropertiesProperty());
    }


    public RDFProperty getApplicationActivityDiskInfoPropertiesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#applicationActivityDiskInfoProperties";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasApplicationActivityDiskInfoProperties() {
        return getPropertyValueCount(getApplicationActivityDiskInfoPropertiesProperty()) > 0;
    }


    public Iterator listApplicationActivityDiskInfoProperties() {
        return listPropertyValues(getApplicationActivityDiskInfoPropertiesProperty());
    }


    public void addApplicationActivityDiskInfoProperties(Object newApplicationActivityDiskInfoProperties) {
        addPropertyValue(getApplicationActivityDiskInfoPropertiesProperty(), newApplicationActivityDiskInfoProperties);
    }


    public void removeApplicationActivityDiskInfoProperties(Object oldApplicationActivityDiskInfoProperties) {
        removePropertyValue(getApplicationActivityDiskInfoPropertiesProperty(), oldApplicationActivityDiskInfoProperties);
    }


    public void setApplicationActivityDiskInfoProperties(Collection newApplicationActivityDiskInfoProperties) {
        setPropertyValues(getApplicationActivityDiskInfoPropertiesProperty(), newApplicationActivityDiskInfoProperties);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskFormat

    public String getDiskFormat() {
        return (String) getPropertyValue(getDiskFormatProperty());
    }


    public RDFProperty getDiskFormatProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskFormat";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasDiskFormat() {
        return getPropertyValueCount(getDiskFormatProperty()) > 0;
    }


    public void setDiskFormat(String newDiskFormat) {
        setPropertyValue(getDiskFormatProperty(), newDiskFormat);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskSize

    public int getDiskSize() {
        return getPropertyValueLiteral(getDiskSizeProperty()).getInt();
    }


    public RDFProperty getDiskSizeProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskSize";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasDiskSize() {
        return getPropertyValueCount(getDiskSizeProperty()) > 0;
    }


    public void setDiskSize(int newDiskSize) {
        setPropertyValue(getDiskSizeProperty(), new java.lang.Integer(newDiskSize));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskType

    public String getDiskType() {
        return (String) getPropertyValue(getDiskTypeProperty());
    }


    public RDFProperty getDiskTypeProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#diskType";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasDiskType() {
        return getPropertyValueCount(getDiskTypeProperty()) > 0;
    }

    public void setDiskType(String newDiskType) {
        setPropertyValue(getDiskTypeProperty(), newDiskType);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#physicalPath

    public String getPhysicalPath() {
        return (String) getPropertyValue(getPhysicalPathProperty());
    }


    public RDFProperty getPhysicalPathProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#physicalPath";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasPhysicalPath() {
        return getPropertyValueCount(getPhysicalPathProperty()) > 0;
    }


    public void setPhysicalPath(String newPhysicalPath) {
        setPropertyValue(getPhysicalPathProperty(), newPhysicalPath);
    }
}
