/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivityNetworkInfo;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultApplicationActivityNetworkInfo extends DefaultBusinessContextResource
        implements ApplicationActivityNetworkInfo {

    public DefaultApplicationActivityNetworkInfo(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultApplicationActivityNetworkInfo() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ipAddress

    public String getIpAddress() {
        return (String) getPropertyValue(getIpAddressProperty());
    }


    public RDFProperty getIpAddressProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ipAddress";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasIpAddress() {
        return getPropertyValueCount(getIpAddressProperty()) > 0;
    }


    public void setIpAddress(String newIpAddress) {
        setPropertyValue(getIpAddressProperty(), newIpAddress);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#vncPort

    public int getVncPort() {
        return getPropertyValueLiteral(getVncPortProperty()).getInt();
    }


    public RDFProperty getVncPortProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#vncPort";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasVncPort() {
        return getPropertyValueCount(getVncPortProperty()) > 0;
    }


    public void setVncPort(int newVncPort) {
        setPropertyValue(getVncPortProperty(), new java.lang.Integer(newVncPort));
    }
}
