/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationAdaptationAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultApplicationAdaptationAction extends DefaultContextAction
        implements ApplicationAdaptationAction {
    private int cost = 0;
    private int cpuMin;
    private int cpuMax;
    private int memMin;
    private int memMax;
    private double prevCpuMin;

    private double prevCpuMax;
    private double prevMemMin;
    private double prevMemMax;
    private ApplicationActivity task;

    public DefaultApplicationAdaptationAction() {

    }

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public DefaultApplicationAdaptationAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultApplicationAdaptationAction(ApplicationActivity task) {
        this.task = task;
    }

    public DefaultApplicationAdaptationAction(ApplicationActivity task, int cpuMin, int cpuMax, int memMin, int memMax) {
        this.task = task;
        this.setCpuMin(cpuMin);
        this.setCpuMax(cpuMax);
        this.setMemMin(memMin);
        this.setMemMax(memMax);
    }


    public void execute(EACMInstance modelAccess) {
        prevCpuMin = task.getCpuRequiredMinValue();
        prevCpuMax = task.getCpuRequiredMaxValue();
        prevMemMin = task.getMemRequiredMinValue();
        prevMemMax = task.getMemRequiredMaxValue();
        task.setCpuRequiredMinValue(cpuMin);
        task.setCpuRequiredMaxValue(cpuMax);
        task.setMemRequiredMinValue(memMin);
        task.setMemRequiredMaxValue(memMax);
    }

    public void executeOnServiceCenter(EACMInstance modelAccess) {
        throw new UnsupportedOperationException("not implemented yet");

    }


    public void undo(EACMInstance modelAccess) {
        task.setCpuRequiredMinValue((float) prevCpuMin);
        task.setCpuRequiredMaxValue((float) prevCpuMax);
        task.setMemRequiredMinValue((float) prevMemMin);
        task.setMemRequiredMaxValue((float) prevMemMax);
    }


//    public boolean equals(Object o) {
//        if (!(o instanceof DefaultApplicationAdaptationAction)) {
//            return false;
//        }
//
//        throw new UnsupportedOperationException("not implemented yet");
//    }


    public String toString() {
        return "Application Adaptation Action: name = " + this.getName() + " prevCpuMinValue =" + prevCpuMin + " prevCpuMax=" + prevCpuMax + " prevMemMin=" + prevMemMin + " prevMemMax=" + prevMemMax + "\n" +
                " cpuMin=" + cpuMin + " cpuMax=" + cpuMax + " memMin=" + memMin + " memMax=" + memMax + "\n" +
                "For Activity:" + task.toString();
    }

    public int getCpuMin() {
        return cpuMin;
    }

    public void setCpuMin(int cpuMin) {
        this.cpuMin = cpuMin;
    }

    public int getCpuMax() {
        return cpuMax;
    }

    public void setCpuMax(int cpuMax) {
        this.cpuMax = cpuMax;
    }

    public int getMemMin() {
        return memMin;
    }

    public void setMemMin(int memMin) {
        this.memMin = memMin;
    }

    public int getMemMax() {
        return memMax;
    }

    public void setMemMax(int memMax) {
        this.memMax = memMax;
    }

    public ApplicationActivity getActivity() {
        return task;
    }

    public void setActivity(ApplicationActivity task) {
        this.task = task;
    }

}
