/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;

import java.util.*;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultComplexResource extends DefaultServiceCenterITComputingResource
        implements ComplexResource {

    public DefaultComplexResource(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultComplexResource() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUWeight

    public float getCPUWeight() {
        return getPropertyValueLiteral(getCPUWeightProperty()).getFloat();
    }


    public RDFProperty getCPUWeightProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUWeight";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasCPUWeight() {
        return getPropertyValueCount(getCPUWeightProperty()) > 0;
    }


    public void setCPUWeight(float newCPUWeight) {
        setPropertyValue(getCPUWeightProperty(), new java.lang.Float(newCPUWeight));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDDWeight

    public float getHDDWeight() {
        return getPropertyValueLiteral(getHDDWeightProperty()).getFloat();
    }


    public RDFProperty getHDDWeightProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDDWeight";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasHDDWeight() {
        return getPropertyValueCount(getHDDWeightProperty()) > 0;
    }


    public void setHDDWeight(float newHDDWeight) {
        setPropertyValue(getHDDWeightProperty(), new java.lang.Float(newHDDWeight));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEMWeight

    public float getMEMWeight() {
        return getPropertyValueLiteral(getMEMWeightProperty()).getFloat();
    }


    public RDFProperty getMEMWeightProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEMWeight";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasMEMWeight() {
        return getPropertyValueCount(getMEMWeightProperty()) > 0;
    }


    public void setMEMWeight(float newMEMWeight) {
        setPropertyValue(getMEMWeightProperty(), new java.lang.Float(newMEMWeight));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cpuResources

    public Collection getCpuResources() {
        return getPropertyValuesAs(getCpuResourcesProperty(), CPU.class);
    }


    public RDFProperty getCpuResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cpuResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasCpuResources() {
        return getPropertyValueCount(getCpuResourcesProperty()) > 0;
    }


    public Iterator listCpuResources() {
        return listPropertyValuesAs(getCpuResourcesProperty(), CPU.class);
    }


    public void addCpuResource(CPU newCpuResources) {
        addPropertyValue(getCpuResourcesProperty(), newCpuResources);
    }


    public void removeCpuResources(CPU oldCpuResources) {
        removePropertyValue(getCpuResourcesProperty(), oldCpuResources);
    }


    public void setCpuResources(Collection newCpuResources) {
        setPropertyValues(getCpuResourcesProperty(), newCpuResources);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddResources

    public Collection getHddResources() {
        return getPropertyValuesAs(getHddResourcesProperty(), HDD.class);
    }


    public RDFProperty getHddResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasHddResources() {
        return getPropertyValueCount(getHddResourcesProperty()) > 0;
    }


    public Iterator listHddResources() {
        return listPropertyValuesAs(getHddResourcesProperty(), HDD.class);
    }


    public void addHddResources(HDD newHddResources) {
        addPropertyValue(getHddResourcesProperty(), newHddResources);
    }


    public void removeHddResources(HDD oldHddResources) {
        removePropertyValue(getHddResourcesProperty(), oldHddResources);
    }


    public void setHddResources(Collection newHddResources) {
        setPropertyValues(getHddResourcesProperty(), newHddResources);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isActive

    public boolean getIsActive() {
        if (hasIsActive()) {
            return getPropertyValueLiteral(getIsActiveProperty()).getBoolean();
        } else {
            return false;
        }
    }


    public RDFProperty getIsActiveProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isActive";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasIsActive() {
        return getPropertyValueCount(getIsActiveProperty()) > 0;
    }


    public void setIsActive(boolean newIsActive) {
        if(newIsActive){
            setCurrentEnergyState(1);
        }else{
            setCurrentEnergyState(0);
        }
        setPropertyValue(getIsActiveProperty(), new java.lang.Boolean(newIsActive));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#memResources

    public Collection getMemResources() {
        return getPropertyValuesAs(getMemResourcesProperty(), MEM.class);
    }


    public RDFProperty getMemResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#memResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasMemResources() {
        return getPropertyValueCount(getMemResourcesProperty()) > 0;
    }


    public Iterator listMemResources() {
        return listPropertyValuesAs(getMemResourcesProperty(), MEM.class);
    }


    public void addMemResource(MEM newMemResources) {
        addPropertyValue(getMemResourcesProperty(), newMemResources);
    }


    public void removeMemResources(MEM oldMemResources) {
        removePropertyValue(getMemResourcesProperty(), oldMemResources);
    }


    public void setMemResources(Collection newMemResources) {
        setPropertyValues(getMemResourcesProperty(), newMemResources);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceWorkloadProperty

    public List<String> getResourceWorkloadProperty() {
        return new ArrayList<String>(getPropertyValues(getResourceWorkloadPropertyProperty()));
    }

    public RDFProperty getResourceWorkloadPropertyProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceWorkloadProperty";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasResourceWorkloadProperty() {
        return getPropertyValueCount(getResourceWorkloadPropertyProperty()) > 0;
    }


    public Iterator listResourceWorkloadProperty() {
        return listPropertyValues(getResourceWorkloadPropertyProperty());
    }


    public void addResourceWorkloadProperty(String newResourceWorkloadProperty) {
        addPropertyValue(getResourceWorkloadPropertyProperty(), newResourceWorkloadProperty);
    }


    public void removeResourceWorkloadProperty(String oldResourceWorkloadProperty) {
        removePropertyValue(getResourceWorkloadPropertyProperty(), oldResourceWorkloadProperty);
    }


    public void setResourceWorkloadProperty(List<String> newResourceWorkloadProperty) {
        setResourceWorkloadProperty((Collection) newResourceWorkloadProperty);
    }

    public void setResourceWorkloadProperty(Collection newResourceWorkloadProperty) {
        setPropertyValues(getResourceWorkloadPropertyProperty(), newResourceWorkloadProperty);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#simpleResources

    public List<ServiceCenterITComputingResource> getResources() {
        if (!isCluster()) {
            ArrayList<ServiceCenterITComputingResource> resources = new ArrayList<ServiceCenterITComputingResource>();
            resources.addAll(getCpuResources());
            resources.addAll(getMemResources());
            resources.addAll(getHddResources());
            return resources;
        } else {
            return new ArrayList<ServiceCenterITComputingResource>(getPropertyValuesAs(getSimpleResourcesProperty(), ServiceCenterITComputingResource.class));
        }
    }

    public Collection getSimpleResources() {
        return getPropertyValuesAs(getSimpleResourcesProperty(), SimpleResource.class);
    }

    public RDFProperty getSimpleResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#simpleResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasSimpleResources() {
        return getPropertyValueCount(getSimpleResourcesProperty()) > 0;
    }


    public Iterator listSimpleResources() {
        return listPropertyValuesAs(getSimpleResourcesProperty(), SimpleResource.class);
    }


    public void addSimpleResources(SimpleResource newSimpleResources) {
        addPropertyValue(getSimpleResourcesProperty(), newSimpleResources);
    }


    public void removeSimpleResources(SimpleResource oldSimpleResources) {
        removePropertyValue(getSimpleResourcesProperty(), oldSimpleResources);
    }


    public void setResources(List<ServiceCenterITComputingResource> newSimpleResources) {
        setSimpleResources(newSimpleResources);
    }

    public void setSimpleResources(Collection newSimpleResources) {
        setPropertyValues(getSimpleResourcesProperty(), newSimpleResources);
    }

    public void addResource(ServiceCenterITComputingResource resource) {
        addPropertyValue(getSimpleResourcesProperty(), resource);
    }

    public void removeResource(ServiceCenterITComputingResource resource) {
        removePropertyValue(getSimpleResourcesProperty(), resource);
    }

    public boolean hasResourcesToBeNegotiatedFor(ApplicationActivity task) {
        //TODO: negociere pe clustere
        if (isCluster()) {
            return false;
        }
        CPU cpu = (CPU) this.getCpuResources().iterator().next();
        float requestedCPU = task.getNumberOfCoresRequiredValue() * task.getCpuRequiredMinValue();
        if (cpu.getCurrentWorkLoad() + requestedCPU > cpu.getMaximumWorkLoad()) {
            return false;
        }
//        int freeCores = cores.size();
//        for (Core core : cores) {
//            List<ApplicationActivity> runningActivities = core.getRunningActivities();
//            int usedCore = 0;
//            for (ApplicationActivity activity : runningActivities) {
//                usedCore += activity.getCpuAllocatedValue();
//            }
//            if (usedCore + task.getCpuRequiredMinValue() > core.getMaximumWorkLoad()) freeCores--;
//        }

//        if (freeCores < task.getNumberOfCoresRequiredValue()) return false;
//        for (Core core : cores) {
//            if (core.getCurrentWorkLoad() + task.getCpuRequiredMinValue() > core.getMaximumWorkLoad()) {
//                continue;
//            } else {
//                requestedCores--;
//            }
//        }
//        if (requestedCores > 0) {
//            return false;
//        }
        MEM memory = (MEM) this.getMemResources().iterator().next();
        if (memory.getCurrentWorkLoad() + task.getMemRequiredMinValue() > memory.getMaximumWorkLoad()) {
            return false;
        }

        return true;
    }

    public boolean hasResourcesFor(ApplicationActivity task) {
        if (this instanceof ServiceCenterServer) {
            ServiceCenterServer serviceCenterServer = (ServiceCenterServer) this;
            if (serviceCenterServer.isCluster()) {
                for (ServiceCenterITComputingResource resource : serviceCenterServer.getResources()) {
                    if (((ServiceCenterServer) resource).hasResourcesFor(task)) {
                        return true;
                    }
                }
                return false;
            } else {
                Collection<CPU> cpus = this.getCpuResources();
                for (CPU cpu : cpus) {

                    if (!cpu.hasResourcesFor(task)) {
                        return false;
                    }
                }
                Collection<MEM> memories = this.getMemResources();
                for (MEM mem : memories) {

                    if (!mem.hasResourcesFor(task)) {
                        return false;
                    }
                }
                Collection<HDD> hdds = this.getHddResources();
                for (HDD hdd : hdds) {

                    if (!hdd.hasResourcesFor(task)) {
                        return false;
                    }
                }

                return true;
            }
        } else {
            return false;
        }
    }
    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isCluster

    public boolean isCluster() {
        return hasIsCluster() && getPropertyValueLiteral(getIsClusterProperty()).getBoolean();
    }


    public RDFProperty getIsClusterProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isCluster";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasIsCluster() {
        return getPropertyValueCount(getIsClusterProperty()) > 0;
    }


    public void setIsCluster(boolean newIsCluster) {
        setPropertyValue(getIsClusterProperty(), new java.lang.Boolean(newIsCluster));
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isRootCluster

    public boolean isRootCluster() {
        return hasIsRootCluster() && getPropertyValueLiteral(getIsRootClusterProperty()).getBoolean();
    }


    public RDFProperty getIsRootClusterProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isRootCluster";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasIsRootCluster() {
        return getPropertyValueCount(getIsRootClusterProperty()) > 0;
    }


    public void setIsRootCluster(boolean newIsRootCluster) {
        setPropertyValue(getIsRootClusterProperty(), new java.lang.Boolean(newIsRootCluster));
    }

}
