/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextResource;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public abstract class DefaultContextAction extends DefaultContextElement
        implements ContextAction {

    public DefaultContextAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultContextAction() {
    }

    /*

   // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity

   public Collection getActivity() {
       return getPropertyValuesAs(getActivityProperty(), ApplicationActivity.class);
   }


   public RDFProperty getActivityProperty() {
       final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity";
       final String name = getOWLModel().getResourceNameForURI(uri);
       return getOWLModel().getRDFProperty(name);
   }


   public boolean hasActivity() {
       return getPropertyValueCount(getActivityProperty()) > 0;
   }


   public Iterator listActivity() {
       return listPropertyValuesAs(getActivityProperty(), ApplicationActivity.class);
   }


   public void addActivity(ApplicationActivity newActivity) {
       addPropertyValue(getActivityProperty(), newActivity);
   }


   public void removeActivity(ApplicationActivity oldActivity) {
       removePropertyValue(getActivityProperty(), oldActivity);
   }


   public void setActivities(List<ApplicationActivity> newActivities) {
      setActivity(newActivities);
   }
   public void setActivity(Collection newActivity) {
       setPropertyValues(getActivityProperty(), newActivity);
   }
    */
    /*
   // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedResources

   public List<ContextResource> getResources() {
       return new ArrayList<ContextResource>(getAssociatedResources());
   }
     public Collection getAssociatedResources() {
       return getPropertyValuesAs(getAssociatedResourcesProperty(), ContextResource.class);
   }

   public RDFProperty getAssociatedResourcesProperty() {
       final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedResources";
       final String name = getOWLModel().getResourceNameForURI(uri);
       return getOWLModel().getRDFProperty(name);
   }


   public boolean hasAssociatedResources() {
       return getPropertyValueCount(getAssociatedResourcesProperty()) > 0;
   }


   public Iterator listAssociatedResources() {
       return listPropertyValuesAs(getAssociatedResourcesProperty(), ContextResource.class);
   }


   public void addAssociatedResources(ContextResource newAssociatedResources) {
       addPropertyValue(getAssociatedResourcesProperty(), newAssociatedResources);
   }


   public void removeAssociatedResources(ContextResource oldAssociatedResources) {
       removePropertyValue(getAssociatedResourcesProperty(), oldAssociatedResources);
   }


   public void setAssociatedResources(List<ContextResource> newAssociatedResources) {
       setAssociatedResources(newAssociatedResources);
   }
  public void setAssociatedResources(Collection newAssociatedResources) {
       setPropertyValues(getAssociatedResourcesProperty(), newAssociatedResources);
   }

    */

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resources

    public List<ContextResource> getResources() {
        return new ArrayList<ContextResource>(getPropertyValuesAs(getResourcesProperty(), ContextResource.class));
    }


    public RDFProperty getResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasResources() {
        return getPropertyValueCount(getResourcesProperty()) > 0;
    }


    public Iterator listResources() {
        return listPropertyValuesAs(getResourcesProperty(), ContextResource.class);
    }

    public void addResource(ContextResource newResources) {
        if (getResourcesProperty() != null) {
            addPropertyValue(getResourcesProperty(), newResources);
        } else {
            setPropertyValue(getResourcesProperty(), newResources);
        }
    }


    public void removeResource(ContextResource oldResources) {
        removePropertyValue(getResourcesProperty(), oldResources);
    }


    public void setResources(List<ContextResource> newResources) {
        setPropertyValues(getResourcesProperty(), newResources);
    }

// Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#executedSubActions

    public Collection<ContextAction> getExecutedSubActions() {
        return getPropertyValuesAs(getExecutedSubActionsProperty(), ContextAction.class);
    }


    private RDFProperty getExecutedSubActionsProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#executedSubActions";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasExecutedSubActions() {
        return getPropertyValueCount(getExecutedSubActionsProperty()) > 0;
    }


    public void addExecutedSubActions(ContextAction newExecutedSubActions) {
        addPropertyValue(getExecutedSubActionsProperty(), newExecutedSubActions);
    }


    public void removeExecutedSubActions(ContextAction oldExecutedSubActions) {
        removePropertyValue(getExecutedSubActionsProperty(), oldExecutedSubActions);
    }


    public void setExecutedSubActions(Collection newExecutedSubActions) {
        setPropertyValues(getExecutedSubActionsProperty(), newExecutedSubActions);
    }


//    public abstract void execute(EACMInstance modelAccess);
//
//    public abstract void undo(EACMInstance modelAccess);


//    public boolean equals(Object o) {
//        if (o instanceof DefaultDeployActivity) {
//            return this.getName().equals(((DefaultDeployActivity) o).getName());
//        } else {
//            return false;
//        }
//    }

    public abstract String toString();
}
