/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.DefaultFrame;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.ModelUtilities;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.ValueType;
import edu.stanford.smi.protege.util.Assert;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLIndividual;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextElement;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class DefaultContextElement extends DefaultOWLIndividual
        implements ContextElement {
//    protected transient OWLModel owlModel;

    int id = 0;

    public DefaultContextElement(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
//        this.owlModel = owlModel;
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#id
    public int getId() {
        return (hasId()) ? getPropertyValueLiteral(getIdProperty()).getInt() : -1;
    }

    public RDFProperty getIdProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#id";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasId() {
        return getPropertyValueCount(getIdProperty()) > 0;
    }

    public void setId(int newId) {
        setPropertyValue(getIdProperty(), new java.lang.Integer(newId));
    }

    public DefaultContextElement() {
    }

    public int getOwnSlotValueCount(Slot slot) {
        return super.getOwnSlotValueCount(slot);
    }

    //    public void setKnowledgeBase(KnowledgeBase model) {
//         owlModel = (OWLModel) model;
//    }
    //
//    public void setPropertyValue(RDFProperty property, Object value) {
//        OntModel ontModel = getOWLModel().getOntModel();
//        this.setPropertyValue(property, value, ontModel);
//    }
//
//
//    public Object getPropertyValue(RDFProperty property) {
//        OntModel ontModel = getOWLModel().getOntModel();
//        return this.getPropertyValue(property, ontModel);   //To change body of overridden methods use File | Settings | File Templates.
//    }
//
//    /**
//     * Sets the property value both on the OWL model and on the ONT model
//     * to trigger SWRL rule evaluation
//     *
//     * @param rdfProperty the slot of the entity
//     * @param value       the value of the property to be inserted in slot rdfProperty
//     * @param ontModel    the ont model on which this property will also be set
//     */
//    public void setPropertyValue(RDFProperty rdfProperty, Object value, OntModel ontModel) {
//
//        super.setPropertyValue(rdfProperty, value);
//        Individual targetIndividual = ontModel.getIndividual(this.getName());
//        if (targetIndividual == null) {
//            System.out.println("It is null");
//            System.exit(1);
//        }
//        Property targetProperty = ontModel.getProperty(rdfProperty.getName());
//        if (targetIndividual.hasProperty(targetProperty)) {
//            targetIndividual.removeAll(targetProperty);
//        }
//        targetIndividual.setPropertyValue(targetProperty, ontModel.createLiteralStatement(
//                targetIndividual, targetProperty, value).getLiteral().as(RDFNode.class));
//    }
//
//    public Object getPropertyValue(RDFProperty rdfProperty, OntModel ontModel) {
//
//        Individual targetIndividual = ontModel.getIndividual(this.getName());
////        ExtendedIterator<Individual> iterator = ontModel.listIndividuals();
////        while (iterator.hasNext()) {
////            System.out.println(iterator.next().getURI());
////        }
//
//        Property targetProperty = ontModel.getProperty(rdfProperty.getName());
//        if (targetIndividual.hasProperty(targetProperty)) {
//            targetIndividual.removeAll(targetProperty);
//        }
//
//        System.out.print("ONT val: " + targetIndividual.getPropertyValue(targetProperty));
//        System.out.println("OWL val: " + super.getPropertyValue(rdfProperty));
//        return targetIndividual.getPropertyValue(targetProperty).asLiteral().getInt();
//
//    }
//
//    public final void deleteInstance(OntModel ontModel, SWRLFactory swrlFactory)
//            throws SWRLFactoryException {
//        super.delete();
//
//        //to be commented to avoid SWRL rule evaluation
//        //remove instance from underlying Ont model
////        Individual i = ontModel.getIndividual(getName());
////        i.remove();
////
////        //remove swrl rule associated to task
////        SWRLImp rule = swrlFactory
////                .getImp("http://www.owl-ontologies.com/Datacenter.owl#QoS_Policy_"
////                + this.getName().split("_")[1] + "_swrl_rule");
////        rule.delete();
//    }
    public void copyToAnotherKnowledgeBase(KnowledgeBase targetKB, Map valueMap, boolean isDeep) {
        // Log.enter(this, "deepCopy", targetKB);
        KnowledgeBase sourceKB = getKnowledgeBase();
        if (targetKB == null) {
            targetKB = sourceKB;
        }
        if (valueMap == null) {
            valueMap = ModelUtilities.createValueMap(sourceKB, targetKB);
        }
        Instance copy = (Instance) valueMap.get(this);
        if (copy == null) {
            String name = this.getName();
            Collection copyTypes = getCopyTypes(valueMap);
            copy = targetKB.createInstance(name, copyTypes);
            valueMap.put(this, copy);
        }
        copy(targetKB, valueMap, isDeep);
    }

    private static void deepCopyFrameBindingValues(Collection values, Frame copyFrame, Slot copySlot, Map valueMap) {

        KnowledgeBase copyKB = copyFrame.getKnowledgeBase();
        Collection copyValues = new ArrayList();
        Iterator i = values.iterator();
        while (i.hasNext()) {
            Frame origFrame = (Frame) i.next();
            Frame copyValue = (Frame) valueMap.get(origFrame);
            if (copyValue == null) {
                copyValue = origFrame.deepCopy(copyKB, valueMap);
            }
            copyValues.add(copyValue);
        }
        copyFrame.setOwnSlotValues(copySlot, copyValues);
    }

    private void copyOwnSlot(DefaultFrame copyFrame, Slot origSlot, Map valueMap, boolean isDeep) {
        Slot copySlot = (Slot) valueMap.get(origSlot);
        Assert.assertNotNull("copy slot", copySlot);
        copyOwnSlotValues(copyFrame, copySlot, origSlot, valueMap, isDeep);
    }

    private void copyOwnSlots(DefaultFrame copyFrame, Map valueMap, boolean isDeep) {
        Iterator i = getDefaultKnowledgeBase().getOwnSlots(this).iterator();
        while (i.hasNext()) {
            Slot origSlot = (Slot) i.next();
            if (!origSlot.isSystem()) {
                copyOwnSlot(copyFrame, origSlot, valueMap, isDeep);
            }
        }
    }

    private void copyOwnSlotValues(Frame copyFrame, Slot copySlot, Slot origSlot, Map valueMap, boolean isDeep) {
        ValueType type = getOwnSlotValueType(origSlot);
        Collection origValues = getDirectOwnSlotValues(origSlot);
        if (isDeep && (type == ValueType.INSTANCE || type == ValueType.CLS)) {
            deepCopyFrameBindingValues(origValues, copyFrame, copySlot, valueMap);
        } else {
            copyFrame.setOwnSlotValues(copySlot, origValues);
        }
    }

    public Frame copy(KnowledgeBase kb, Map valueMap, boolean isDeep) {
        Assert.assertNotNull("knowledge base", kb);
        Assert.assertNotNull("value map", valueMap);
        DefaultFrame copy = (DefaultFrame) valueMap.get(this);
        Assert.assertNotNull(null, copy);
        copyOwnSlots(copy, valueMap, isDeep);
        return copy;
    }

    private Collection getCopyTypes(Map valueMap) {
        Collection copyTypes = new ArrayList();
        Iterator i = getDirectTypes().iterator();
        while (i.hasNext()) {
            Cls type = (Cls) i.next();
            Cls copyType = (Cls) valueMap.get(type);
            copyTypes.add(copyType);
        }
        return copyTypes;
    }

    //http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#epkbID   
    public RDFProperty getEPKBIdProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#epkbID";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public String getEPKBId() {
        return hasEPKBId() ? getPropertyValueLiteral(getEPKBIdProperty()).getString() : "empty";
    }

    public boolean hasEPKBId() {
        return getPropertyValueCount(getEPKBIdProperty()) > 0;
    }

    public void setEPKBId(String newId) {
        setPropertyValue(getEPKBIdProperty(), newId);
    }

    @Override
    public String toString() {
        return this.getName().replace("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#","_");
    }

    @Override
    public void delete() {
        String thisName = this.getName();
        super.delete();
        KnowledgeBase knowledgeBase = this.getKnowledgeBase();
        Instance instance = knowledgeBase.getInstance(thisName);
        if (instance != null) {
            knowledgeBase.deleteInstance(instance);
        }
    }
}
