/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.DPMAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterITComputingResource;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultDPMAction extends DefaultITComputingResourceAdaptationAction
        implements DPMAction {
    private int cost = 0;

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public DefaultDPMAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultDPMAction() {
    }


    public void execute(EACMInstance modelAccess) {
        throw new UnsupportedOperationException("Not implemented yet");
    }

    public void executeOnServiceCenter(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }


    public void undo(EACMInstance modelAccess) {
        throw new UnsupportedOperationException("Not implemented yet");
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#noStatesJump

    public int getNoStatesJump() {
        return getPropertyValueLiteral(getNoStatesJumpProperty()).getInt();
    }


    public RDFProperty getNoStatesJumpProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#noStatesJump";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasNoStatesJump() {
        return getPropertyValueCount(getNoStatesJumpProperty()) > 0;
    }


    public void setNoStatesJump(int newNoStatesJump) {
        setPropertyValue(getNoStatesJumpProperty(), new java.lang.Integer(newNoStatesJump));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceOn

    public ServiceCenterITComputingResource getResourceOn() {
        return (ServiceCenterITComputingResource) getPropertyValueAs(getResourceOnProperty(), ServiceCenterITComputingResource.class);
    }


    public RDFProperty getResourceOnProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceOn";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasResourceOn() {
        return getPropertyValueCount(getResourceOnProperty()) > 0;
    }


    public void setResourceOn(ServiceCenterITComputingResource newResourceOn) {
        setPropertyValue(getResourceOnProperty(), newResourceOn);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateAfterAction

    public int getStateAfterAction() {
        return getPropertyValueLiteral(getStateAfterActionProperty()).getInt();
    }


    public RDFProperty getStateAfterActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateAfterAction";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasStateAfterAction() {
        return getPropertyValueCount(getStateAfterActionProperty()) > 0;
    }


    public void setStateAfterAction(int newStateAfterAction) {
        setPropertyValue(getStateAfterActionProperty(), new java.lang.Integer(newStateAfterAction));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateBeforeAction

    public int getStateBeforeAction() {
        return getPropertyValueLiteral(getStateBeforeActionProperty()).getInt();
    }


    public RDFProperty getStateBeforeActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateBeforeAction";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasStateBeforeAction() {
        return getPropertyValueCount(getStateBeforeActionProperty()) > 0;
    }


    public void setStateBeforeAction(int newStateBeforeAction) {
        setPropertyValue(getStateBeforeActionProperty(), new java.lang.Integer(newStateBeforeAction));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#toLowOrHighState

    public boolean getToLowOrHighState() {
        return getPropertyValueLiteral(getToLowOrHighStateProperty()).getBoolean();
    }


    public RDFProperty getToLowOrHighStateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#toLowOrHighState";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasToLowOrHighState() {
        return getPropertyValueCount(getToLowOrHighStateProperty()) > 0;
    }


    public void setToLowOrHighState(boolean newToLowOrHighState) {
        setPropertyValue(getToLowOrHighStateProperty(), new java.lang.Boolean(newToLowOrHighState));
    }


//    public boolean equals(Object o) {
//        throw new UnsupportedOperationException("Not implemented yet");
//    }


    public String toString() {
        return this.getName();
    }
}
