/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Queue;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultDeployActivity extends DefaultConsolidationAction
        implements DeployActivity {

    private int cost = 200;

    public DefaultDeployActivity(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultDeployActivity() {
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity
    public ApplicationActivity getActivity() {
        return (ApplicationActivity) getPropertyValueAs(getActivityProperty(), ApplicationActivity.class);
    }

    public RDFProperty getActivityProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasActivity() {
        return getPropertyValueCount(getActivityProperty()) > 0;
    }

    public void setActivity(ApplicationActivity newActivity) {
        setPropertyValue(getActivityProperty(), newActivity);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceTo
    public ServiceCenterITComputingResource getResourceTo() {
        return (ServiceCenterITComputingResource) getResources().toArray()[0];
    }

    public void setResourceTo(ServiceCenterITComputingResource newResourceTo) {
        if (hasResources()) {
            ContextResource resourceTo = getResourceTo();
            removePropertyValue(getResourcesProperty(), resourceTo);
        }

        addPropertyValue(getResourcesProperty(), newResourceTo);
    }


    public void execute(EACMInstance modelAccess) {
        ServiceCenterServer server = modelAccess.getServiceCenterServer(getResourceTo().getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());


        server.addRunningActivity(activity);

    }


    public void undo(EACMInstance modelAccess) {
        ServiceCenterServer server = modelAccess.getServiceCenterServer(getResourceTo().getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());

        int subActionsCount = getExecutedSubActions().size();
        List<ContextAction> executedSubActions = new ArrayList<ContextAction>(getExecutedSubActions());
        for (int i = subActionsCount - 1; i >= 0; i--) {
            ContextAction contextAction = executedSubActions.get(i);
            contextAction.undo(modelAccess);
//            if (contextAction instanceof DeployActivity) {
//                server.removeRunningActivity(((DeployActivity) contextAction).getActivity());
//            }
        }
        server.removeRunningActivity(activity);
    }


    public String toString() {
        Collection<ContextAction> contextActions = this.getExecutedSubActions();
        String value = "";
        if (contextActions.size() == 0) {

            try {
                String serviceName = this.getActivity().getName();
                serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
                value = "\t Deploy Activity" + " " + serviceName + " to " + this.getResourceTo().getName().split("#")[1];
            } catch (Exception e) {
                GlobalLoopLogger.error(e);
                e.printStackTrace();
            }
        } else {
            String serviceName = this.getActivity().getName();
            serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
            value = "\t Deploy Activity" + " " + serviceName + " to " + this.getResourceTo().getName().split("#")[1];
            value += "\n \t [";
            for (ContextAction contextAction : contextActions) {
                value += "\n \t \t" + contextAction.toString();
            }
            value += "\n \t ]";
        }
        return value;
    }


//    public boolean equals(Object o) {
//        if (o instanceof DefaultDeployActivity) {
//            DefaultDeployActivity activityAction = (DefaultDeployActivity) o;
//            return activityAction.getName().equals(this.getName())
//                    || activityAction.getResources().equals(this.getResources())
//                    && activityAction.getActivity().equals(this.getActivity());
//        } else if (o instanceof DefaultMigrateActivity) {
//            DefaultMigrateActivity activityAction = (DefaultMigrateActivity) o;
//            return activityAction.getActivity().equals(this.getActivity());
//        } else {
//            return super.equals(o);
//        }
//    }

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }
}
