/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ActionEffect;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.FacilityDefaultAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextResource;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityPassiveResource;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultFacilityDefaultAction extends DefaultContextAction
        implements FacilityDefaultAction {
    private int cost = 0;

    private ActionEffect effect;

    public DefaultFacilityDefaultAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultFacilityDefaultAction() {
    }

    public DefaultFacilityDefaultAction(ActionEffect effect) {
    }


    public void execute(EACMInstance modelAccess ) {
        if (effect == null) {
            throw new UnsupportedOperationException("First assign an effect");
        }
        for (ContextResource contextResource : getResources()) {
            effect.execute((ITFacilityPassiveResource) contextResource);
        }
    }

    public void executeOnServiceCenter(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void executeOnWebService(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }


    public void undo(EACMInstance modelAccess) {
        if (effect == null) {
            throw new UnsupportedOperationException("First assign an effect");
        }
        for (ContextResource contextResource : getResources()) {
            effect.undo((ITFacilityPassiveResource) contextResource);
        }
    }


    public ActionEffect getEffect() {
        return effect;
    }

    public void setEffect(ActionEffect effect) {
        this.effect = effect;
    }


//    public boolean equals(Object o) {
//        if (!(o instanceof DefaultFacilityDefaultAction)) {
//            return super.equals(o);
//        }
//        DefaultFacilityDefaultAction action = (DefaultFacilityDefaultAction) o;
//        return action.getName().equals(this.getName()) || action.getResources().equals(this.getResources()) && action.getEffect().equals(this.getEffect());
//    }


    public String toString() {
        return this.getName();
    }

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }
}
