/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.Goal;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.GPI_KPI_Policy;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public abstract class DefaultGPI_KPI_Policy extends DefaultContextPolicy
        implements GPI_KPI_Policy {

    private Goal policyGoal;

    public DefaultGPI_KPI_Policy(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultGPI_KPI_Policy() {
    }


    public abstract boolean isRespected();
//    {
//       throw new UnsupportedOperationException("Not implemented yet");
//    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Threshold_Max

    public float getGPI_KPI_Threshold_Max() {
        if (hasGPI_KPI_Threshold_Max()) {
            return getPropertyValueLiteral(getGPI_KPI_Threshold_MaxProperty()).getFloat();
        } else {
            return 0.0f;
        }
    }


    public RDFProperty getGPI_KPI_Threshold_MaxProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Threshold_Max";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasGPI_KPI_Threshold_Max() {
        return getPropertyValueCount(getGPI_KPI_Threshold_MaxProperty()) > 0;
    }


    public void setGPI_KPI_Threshold_Max(float newGPI_KPI_Threshold_Max) {
        setPropertyValue(getGPI_KPI_Threshold_MaxProperty(), new java.lang.Float(newGPI_KPI_Threshold_Max));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Threshold_Min

    public float getGPI_KPI_Threshold_Min() {
        if (hasGPI_KPI_Threshold_Min()) {
            return getPropertyValueLiteral(getGPI_KPI_Threshold_MinProperty()).getFloat();
        } else {
            return 0.0f;
        }
    }


    public RDFProperty getGPI_KPI_Threshold_MinProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Threshold_Min";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasGPI_KPI_Threshold_Min() {
        return getPropertyValueCount(getGPI_KPI_Threshold_MinProperty()) > 0;
    }


    public void setGPI_KPI_Threshold_Min(float newGPI_KPI_Threshold_Min) {
        setPropertyValue(getGPI_KPI_Threshold_MinProperty(), new java.lang.Float(newGPI_KPI_Threshold_Min));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_value

    public float getGPI_KPI_value() {
        if (hasGPI_KPI_value()) {
            return getPropertyValueLiteral(getGPI_KPI_valueProperty()).getFloat();
        } else {
            return 0.0f;
        }
    }


    public RDFProperty getGPI_KPI_valueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_value";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasGPI_KPI_value() {
        return getPropertyValueCount(getGPI_KPI_valueProperty()) > 0;
    }


    public void setGPI_KPI_value(float newGPI_KPI_value) {
        setPropertyValue(getGPI_KPI_valueProperty(), new java.lang.Float(newGPI_KPI_value));
    }

    public Goal getPolicyGoal() {
        return policyGoal;
    }

    public void setPolicyGoal(Goal policyGoal) {
        this.policyGoal = policyGoal;
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policyAction

    public ContextAction getPolicyAction() {
        return (ContextAction) getPropertyValueAs(getPolicyActionProperty(), ContextAction.class);
    }


    public RDFProperty getPolicyActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policyAction";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasPolicyAction() {
        return getPropertyValueCount(getPolicyActionProperty()) > 0;
    }


    public void setPolicyAction(ContextAction newPolicyAction) {
        setPropertyValue(getPolicyActionProperty(), newPolicyAction);
    }

//
//    public boolean isRespected() {
//        Goal goal = getPolicyGoal();
//        if (goal == null) {
//            System.err.println("No goal specified for policy:" + this.getLocalName() + ". Returning isRespected = false");
//            return false;
//        }
//        for (ContextResource resource : getPolicySubject()) {
//            if (!goal.isAchieved((ITFacilityPassiveResource) resource)) {
//                return false;
//            }
//        }
//        return true;
//    }
}