/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextElement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.HDD;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;

import java.util.Collection;
import java.util.Iterator;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultHDD extends DefaultSimpleResource
        implements HDD {

    public DefaultHDD(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultHDD() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf

    public Collection getPartOf() {
        return getPropertyValuesAs(getPartOfProperty(), ContextElement.class);
    }


    public RDFProperty getPartOfProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasPartOf() {
        return getPropertyValueCount(getPartOfProperty()) > 0;
    }


    public Iterator listPartOf() {
        return listPropertyValuesAs(getPartOfProperty(), ContextElement.class);
    }


    public void addPartOf(ContextElement newPartOf) {
        addPropertyValue(getPartOfProperty(), newPartOf);
    }


    public void removePartOf(ContextElement oldPartOf) {
        removePropertyValue(getPartOfProperty(), oldPartOf);
    }


    public void setPartOf(Collection newPartOf) {
        setPropertyValues(getPartOfProperty(), newPartOf);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#rotationSpeed

    public Double getRotationSpeed() {
        return getPropertyValueLiteral(getRotationSpeedProperty()).getDouble();
    }


    public RDFProperty getRotationSpeedProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#rotationSpeed";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#physicalPath

    public String getPhysicalPath() {
        return (String) getPropertyValue(getPhysicalPathProperty());
    }


    public RDFProperty getPhysicalPathProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#physicalPath";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasPhysicalPath() {
        return getPropertyValueCount(getPhysicalPathProperty()) > 0;
    }


    public void setPhysicalPath(String newPhysicalPath) {
        setPropertyValue(getPhysicalPathProperty(), newPhysicalPath);
    }

    public RDFProperty getHddFormatProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddFormat";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public String getHddFormat() {
        return (String) getPropertyValue(getHddFormatProperty());
    }

    public void setHddFormat(String format) {
        setPropertyValue(getHddFormatProperty(), format);
    }

    public boolean hasRotationSpeed() {
        return getPropertyValueCount(getRotationSpeedProperty()) > 0;
    }


    public void setRotationSpeed(Double newRotationSpeed) {
        setPropertyValue(getRotationSpeedProperty(), new java.lang.Float(newRotationSpeed));
    }


    public boolean hasResourcesFor(ApplicationActivity activity) {
        return (2 * this.getMaximumWorkLoad() + this.getOptimalWorkLoad()) / 3.0 >= getCurrentWorkLoad() + activity.getHddRequiredMaxValue();
    }


    public void addRunningActivity(ApplicationActivity activity) {
        setCurrentWorkLoad(getCurrentWorkLoad() + activity.getHddRequiredMaxValue());
        super.addRunningActivity(activity);
    }


    public void removeRunningActivity(ApplicationActivity activity) {
        setCurrentWorkLoad(getCurrentWorkLoad() - activity.getHddRequiredMaxValue());
        super.removeRunningActivity(activity);
    }
}
