/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityResourceAdaptationAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityActiveResource;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultITFacilityActiveResource extends DefaultServiceCenterITFacilityResource
        implements ITFacilityActiveResource {

    public DefaultITFacilityActiveResource(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultITFacilityActiveResource() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedActions

    public Collection getAssociatedActions() {
        return getPropertyValuesAs(getAssociatedActionsProperty(), ITFacilityResourceAdaptationAction.class);
    }


    public RDFProperty getAssociatedActionsProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedActions";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasAssociatedActions() {
        return getPropertyValueCount(getAssociatedActionsProperty()) > 0;
    }


    public Iterator listAssociatedActions() {
        return listPropertyValuesAs(getAssociatedActionsProperty(), ITFacilityResourceAdaptationAction.class);
    }


    public void addAssociatedAction(ITFacilityResourceAdaptationAction newAssociatedActions) {
        addPropertyValue(getAssociatedActionsProperty(), newAssociatedActions);
    }


    public void removeAssociatedAction(ITFacilityResourceAdaptationAction oldAssociatedActions) {
        removePropertyValue(getAssociatedActionsProperty(), oldAssociatedActions);
    }


    public void setAssociatedActions(List<ITFacilityResourceAdaptationAction> actions) {
        setPropertyValues(getAssociatedActionsProperty(), actions);
    }

    public List<ITFacilityResourceAdaptationAction> getActions() {

        return new ArrayList<ITFacilityResourceAdaptationAction>(getAssociatedActions());

    }


}
