/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityActiveResource;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityPassiveResource;

import java.util.Collection;
import java.util.Iterator;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultITFacilityPassiveResource extends DefaultServiceCenterITFacilityResource
        implements ITFacilityPassiveResource {

    public DefaultITFacilityPassiveResource(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultITFacilityPassiveResource() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedActiveResources

    public Collection getAssociatedActiveResources() {
        return getPropertyValuesAs(getAssociatedActiveResourcesProperty(), ITFacilityActiveResource.class);
    }


    public RDFProperty getAssociatedActiveResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedActiveResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasAssociatedActiveResources() {
        return getPropertyValueCount(getAssociatedActiveResourcesProperty()) > 0;
    }


    public Iterator listAssociatedActiveResources() {
        return listPropertyValuesAs(getAssociatedActiveResourcesProperty(), ITFacilityActiveResource.class);
    }


    public void addAssociatedActiveResources(ITFacilityActiveResource newAssociatedActiveResources) {
        addPropertyValue(getAssociatedActiveResourcesProperty(), newAssociatedActiveResources);
    }


    public void removeAssociatedActiveResources(ITFacilityActiveResource oldAssociatedActiveResources) {
        removePropertyValue(getAssociatedActiveResourcesProperty(), oldAssociatedActiveResources);
    }


    public void setAssociatedActiveResources(Collection newAssociatedActiveResources) {
        setPropertyValues(getAssociatedActiveResourcesProperty(), newAssociatedActiveResources);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#maximumValue

    public Double getMaximumValue() {
        return getPropertyValueLiteral(getMaximumValueProperty()).getDouble();
    }


    public RDFProperty getMaximumValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#maximumValue";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasMaximumValue() {
        return getPropertyValueCount(getMaximumValueProperty()) > 0;
    }


    public void setMaximumValue(Double newMaximumValue) {
        setPropertyValue(getMaximumValueProperty(), new java.lang.Float(newMaximumValue));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#minimumValue

    public Double getMinimumValue() {
        return getPropertyValueLiteral(getMinimumValueProperty()).getDouble();
    }


    public RDFProperty getMinimumValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#minimumValue";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasMinimumValue() {
        return getPropertyValueCount(getMinimumValueProperty()) > 0;
    }


    public void setMinimumValue(Double newMinimumValue) {
        setPropertyValue(getMinimumValueProperty(), new java.lang.Float(newMinimumValue));
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#recordedValue

    public Double getRecordedValue() {
        if (hasRecordedValue()) {
            return new Double(((Float) (getPropertyValue(getRecordedValueProperty()))));
        } else {
            return 0.0;
        }

    }


    public RDFProperty getRecordedValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#recordedValue";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasRecordedValue() {
        return getPropertyValueCount(getRecordedValueProperty()) > 0;
    }


    public void setRecordedValue(Double newRecordedValue) {
        setPropertyValue(getRecordedValueProperty(), new java.lang.Float(newRecordedValue));
    }
}
