/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.FacilityDefaultAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITFacilityResourceAdaptationAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextResource;

import java.util.Collection;
import java.util.Iterator;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultITFacilityResourceAdaptationAction extends DefaultContextAction
        implements ITFacilityResourceAdaptationAction {
    private int cost = 0;

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public DefaultITFacilityResourceAdaptationAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultITFacilityResourceAdaptationAction() {
    }


    public void execute(EACMInstance modelAccess ) {
        getFacilityAction().execute(modelAccess );
    }

    public void executeOnServiceCenter(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }


    public void undo(EACMInstance modelAccess) {
        getFacilityAction().undo(modelAccess);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedResources

    public Collection getAssociatedResources() {
        return getPropertyValuesAs(getAssociatedResourcesProperty(), ContextResource.class);
    }


    public RDFProperty getAssociatedResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedResources";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasAssociatedResources() {
        return getPropertyValueCount(getAssociatedResourcesProperty()) > 0;
    }


    public Iterator listAssociatedResources() {
        return listPropertyValuesAs(getAssociatedResourcesProperty(), ContextResource.class);
    }


    public void addAssociatedResources(ContextResource newAssociatedResources) {
        addPropertyValue(getAssociatedResourcesProperty(), newAssociatedResources);
    }


    public void removeAssociatedResources(ContextResource oldAssociatedResources) {
        removePropertyValue(getAssociatedResourcesProperty(), oldAssociatedResources);
    }


    public void setAssociatedResources(Collection newAssociatedResources) {
        setPropertyValues(getAssociatedResourcesProperty(), newAssociatedResources);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#facilityAction

    public FacilityDefaultAction getFacilityAction() {
        return (FacilityDefaultAction) getPropertyValueAs(getFacilityActionProperty(), FacilityDefaultAction.class);
    }


    public RDFProperty getFacilityActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#facilityAction";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasFacilityAction() {
        return getPropertyValueCount(getFacilityActionProperty()) > 0;
    }


    public void setFacilityAction(FacilityDefaultAction newFacilityAction) {
        setPropertyValue(getFacilityActionProperty(), newFacilityAction);
    }


//    public boolean equals(Object o) {
//        if (!(o instanceof DefaultITFacilityResourceAdaptationAction)) {
//            return super.equals(o);
//        }
//        DefaultITFacilityResourceAdaptationAction activity = (DefaultITFacilityResourceAdaptationAction) o;
//        return activity.getFacilityAction().equals(this.getFacilityAction());
//    }


    public String toString() {
        return this.getName();
    }
}
