/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultMigrateActivity extends DefaultConsolidationAction
        implements MigrateActivity {

    private int cost = 400;

    public DefaultMigrateActivity(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultMigrateActivity() {
    }


    public void execute(EACMInstance modelAccess) {
        ServiceCenterServer from = modelAccess.getServiceCenterServer(getResourceFrom().getName());
        ServiceCenterServer to = modelAccess.getServiceCenterServer(getResourceTo().getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());

        //if activity has been deleted in meantime
        if (activity == null) {
            return;
        }



        //from.removeRunningActivity(activity);
        // to.addRunningActivity(activity);
    }


    public void undo(EACMInstance modelAccess) {
        ServiceCenterServer from = modelAccess.getServiceCenterServer(getResourceFrom().getName());
        ServiceCenterServer to = modelAccess.getServiceCenterServer(getResourceTo().getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());
        int subActionsCount = getExecutedSubActions().size();
        List<ContextAction> executedSubActions = new ArrayList<ContextAction>(getExecutedSubActions());
        for (int i = subActionsCount - 1; i >= 0; i--) {
            ContextAction contextAction = executedSubActions.get(i);
            contextAction.undo(modelAccess);
            //pentru ca poate face deploy la mai multe taskuri.( ii trimit politicile pentru toate taskurile undeployed sa incerce deploy)
//            if (contextAction instanceof DeployActivity) {
//                to.removeRunningActivity(((DeployActivity) contextAction).getActivity());
//            }
        }
//        to.removeRunningActivity(activity);
        //from.addRunningActivity(activity);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity
    public ApplicationActivity getActivity() {
        return (ApplicationActivity) getPropertyValueAs(getActivityProperty(), ApplicationActivity.class);
    }

    public RDFProperty getActivityProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasActivity() {
        return getPropertyValueCount(getActivityProperty()) > 0;
    }

    public void setActivity(ApplicationActivity newActivity) {
        setPropertyValue(getActivityProperty(), newActivity);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceFrom
    public ServiceCenterITComputingResource getResourceFrom() {
        return (ServiceCenterITComputingResource) ((Collection<ContextResource>) getPropertyValuesAs(getResourcesProperty(), ContextResource.class)).toArray()[0];
    }

    public boolean hasResourceFrom() {
        return getPropertyValueCount(getResourcesProperty()) > 1;
    }

    public void setResourceFrom(ServiceCenterITComputingResource newResourceFrom) {
//        ContextResource resourceTo = getResourceTo();
//        removePropertyValue(getResourcesProperty(), getResourceFrom());
//        removePropertyValue(getResourcesProperty(), resourceTo);
        addPropertyValue(getResourcesProperty(), newResourceFrom);
//        setPropertyValue(getResourcesProperty(), resourceTo);
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceTo
    public ServiceCenterITComputingResource getResourceTo() {
        return (ServiceCenterITComputingResource) ((Collection<ContextResource>) getPropertyValuesAs(getResourcesProperty(), ContextResource.class)).toArray()[1];
    }

    public boolean hasResourceTo() {
        return getPropertyValueCount(getResourcesProperty()) > 2;
    }

    public void setResourceTo(ServiceCenterITComputingResource newResourceTo) {
//        ContextResource resourceFrom = getResourceFrom();
//        removePropertyValue(getResourcesProperty(), resourceFrom);
//        removePropertyValue(getResourcesProperty(), getResourceTo());
//        addPropertyValue(getResourcesProperty(), resourceFrom);
        addPropertyValue(getResourcesProperty(), newResourceTo);
    }


    public String toString() {
        Collection<ContextAction> contextActions = this.getExecutedSubActions();
        String value = "";
        if (contextActions.size() == 0) {

            String serviceName = this.getActivity().getName();
            serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
            value = "\t Migrate Activity" + " " + serviceName + " from "
                    + this.getResourceFrom().getName().split("#")[1] + " to " + this.getResourceTo().getName().split("#")[1];

        } else {
            String serviceName = this.getActivity().getName();
            serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
            value = "\t Migrate Activity" + " " + serviceName + " from "
                    + this.getResourceFrom().getName().split("#")[1] + " to " + this.getResourceTo().getName().split("#")[1];
            value += "[\n ";
            for (ContextAction contextAction : contextActions) {
                value += "\n \t \t" + contextAction.toString() + "\n";
            }
            value += "\n \t]";
        }
        return value;

    }


//    public boolean equals(Object o) {
//        if (o instanceof DefaultMigrateActivity) {
//
//            DefaultMigrateActivity activityAction = (DefaultMigrateActivity) o;
//            return activityAction.getName().equals(this.getName())
//                    || (activityAction.getActivity().equals(this.getActivity())
////                    && activityAction.getResourceTo().equals(this.getResourceTo())
////                    && activityAction.getActivity().equals(this.getActivity()))
////                    || (activityAction.getResourceFrom().equals(this.getResourceTo())
////                    && activityAction.getResourceTo().equals(this.getResourceFrom())
////                    && activityAction.getActivity().equals(this.getActivity())
//            );
//        } else if (o instanceof DefaultDeployActivity) {
//            DefaultDeployActivity activityAction = (DefaultDeployActivity) o;
//            return activityAction.getActivity().equals(this.getActivity());
//        } else {
//            return super.equals(o);
//        }
//    }

    //    public void executeOnServiceCenter(EACMInstance modelAccess) throws ApplicationException {
//        ServiceCenterServer newServer = (ServiceCenterServer) getResourceTo();
//        ApplicationActivity task = getActivity();
//        WorldInterfaceDelegate proxy = WorldInterfaceDelegate.getInstance();
//        if (proxy != null) {
//            VirtualTaskInfo taskInfo = new VirtualTaskInfo(task.getName());
//            taskInfo.setId(task.getId());
//            PhysicalHost host = new PhysicalHost();
//            host.setId(newServer.getId());
//
//            proxy.migrateVirtualMachine(taskInfo, host);
//        } else {
//            throw new ApplicationException("Proxy is null");
//        }
//    }
    public int getCost() {
        ApplicationActivity applicationActivity = getActivity();
        double respectance = applicationActivity.getCPUWeight() * applicationActivity.getNumberOfCoresRequiredValue()
                * applicationActivity.getCpuRequiredMaxValue()
                + applicationActivity.getMEMWeight() * applicationActivity.getMemRequiredMaxValue()
                + applicationActivity.getHDDWeight() * applicationActivity.getHddRequiredMaxValue();
        return (int) respectance;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }
}
