/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Queue;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultRemoveActivity extends DefaultConsolidationAction
        implements RemoveActivity {

    public DefaultRemoveActivity(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

//    @Override
//    public boolean equals(Object o) {
//        if (o instanceof DefaultRemoveActivity) {
//
//            DefaultRemoveActivity activityAction = (DefaultRemoveActivity) o;
//            return activityAction.getName().equals(this.getName())
//                    || activityAction.getResources().equals(this.getResources())
//                    && activityAction.getActivity().equals(this.getActivity());
//        } else {
//            return super.equals(o);
//        }
//    }

    @Override
    public String toString() {
        String value = "";
        Collection<ContextAction> contextActions = this.getExecutedSubActions();

        if (contextActions.size() == 0) {

            String serviceName = this.getActivity().getName();
            serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
            value = "\t Remove Activity" + " " + serviceName + " from " + ((ServiceCenterITComputingResource) getResources().toArray()[0]).getName().split("#")[1];

        } else {
            String serviceName = this.getActivity().getName();
            serviceName = serviceName.substring(serviceName.lastIndexOf("#") + 1);
            value = "\t Remove Activity" + " " + serviceName + " from " + ((ServiceCenterITComputingResource) getResources().toArray()[0]).getName().split("#")[1];
            value += "[ \n \t";
            for (ContextAction contextAction : contextActions) {
                value += "\n \t \t" + contextAction.toString() + "\n";
            }
            value += "\n \t ]";
        }
        return value;
    }

    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity
    public ApplicationActivity getActivity() {
        return (ApplicationActivity) getPropertyValueAs(getActivityProperty(), ApplicationActivity.class);
    }

    public RDFProperty getActivityProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }

    public boolean hasActivity() {
        return getPropertyValueCount(getActivityProperty()) > 0;
    }

    public void setActivity(ApplicationActivity newActivity) {
        setPropertyValue(getActivityProperty(), newActivity);
    }

    public void execute(EACMInstance modelAccess ) {
        ServiceCenterServer server = modelAccess.getServiceCenterServer(((ServiceCenterITComputingResource) getResources().toArray()[0]).getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());
        server.removeRunningActivity(activity);
    }

    public void undo(EACMInstance modelAccess) {
        ServiceCenterServer server = modelAccess.getServiceCenterServer(((ServiceCenterITComputingResource) getResources().toArray()[0]).getName());
        ApplicationActivity activity = modelAccess.getApplicationActivity(getActivity().getName());

        if (server.isCluster()) {
            int subActionsCount = getExecutedSubActions().size();
            List<ContextAction> executedSubActions = new ArrayList<ContextAction>(getExecutedSubActions());
            for (int i = subActionsCount - 1; i >= 0; i--) {
                executedSubActions.get(i).undo(modelAccess);
            }
        }
        server.addRunningActivity(activity);


    }

    public void setCost(int cost) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public int getCost() {
        return 0;  //To change body of implemented methods use File | Settings | File Templates.
    }
}
