/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServerAdaptationAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultServerAdaptationAction extends DefaultITComputingResourceAdaptationAction implements ServerAdaptationAction {
    private int cost = 10;
    private ServiceCenterServer server;
    private int newOptimalValueForCpu;
    private int newOptimalValueForMem;
    private Double oldOptimalValueForCpu;
    private Double oldOptimalValueForMem;

    public DefaultServerAdaptationAction(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }

    public DefaultServerAdaptationAction(ServiceCenterServer server, int newOptimalValueForCpu, int newOptimalValueForMem) {
        this.server = server;
        this.setNewOptimalValueForCpu(newOptimalValueForCpu);
        this.setNewOptimalValueForMem(newOptimalValueForMem);
    }


    public void execute(EACMInstance modelAccess) {
        CPU cpu = server.getCpuResources().iterator().next();
        MEM mem = server.getMemResources().iterator().next();
        oldOptimalValueForCpu = cpu.getOptimalWorkLoad();
        oldOptimalValueForMem = mem.getOptimalWorkLoad();
        cpu.setOptimalWorkLoad((double) getNewOptimalValueForCpu());
        mem.setOptimalWorkLoad((double) getNewOptimalValueForMem());
    }

    public void executeOnServiceCenter(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }


    public void undo(EACMInstance modelAccess) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public int getCost() {
        return cost;
    }

//
//    public boolean equals(Object o) {
//        return super.equals(o);  //To change body of implemented methods use File | Settings | File Templates.
//    }


    public String toString() {
        return "Server Adaptation Action for :" + server.toString() + "\n" + "With old opt values : cpu= " + newOptimalValueForCpu + " mem=" + newOptimalValueForMem + "\n" +
                "\n With new values: cpu=" + newOptimalValueForCpu + " mem=" + newOptimalValueForMem;
    }

    public int getNewOptimalValueForCpu() {
        return newOptimalValueForCpu;
    }

    public void setNewOptimalValueForCpu(int newOptimalValueForCpu) {
        this.newOptimalValueForCpu = newOptimalValueForCpu;
    }

    public int getNewOptimalValueForMem() {
        return newOptimalValueForMem;
    }

    public void setNewOptimalValueForMem(int newOptimalValueForMem) {
        this.newOptimalValueForMem = newOptimalValueForMem;
    }

    public void setServer(ServiceCenterServer server) {
        this.server = server;
    }

    public ServiceCenterServer getServer() {
        return server;
    }
}
