/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterITFacilityResource;

import java.util.Collection;
import java.util.Iterator;


/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultServiceCenterITFacilityResource extends DefaultContextResource
        implements ServiceCenterITFacilityResource {

    public DefaultServiceCenterITFacilityResource(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultServiceCenterITFacilityResource() {
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hasAssociatedActions

    public Collection getHasAssociatedActions() {
        return getPropertyValues(getHasAssociatedActionsProperty());
    }


    public RDFProperty getHasAssociatedActionsProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hasAssociatedActions";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasHasAssociatedActions() {
        return getPropertyValueCount(getHasAssociatedActionsProperty()) > 0;
    }


    public Iterator listHasAssociatedActions() {
        return listPropertyValues(getHasAssociatedActionsProperty());
    }


    public void addHasAssociatedActions(boolean newHasAssociatedActions) {
        addPropertyValue(getHasAssociatedActionsProperty(), newHasAssociatedActions);
    }


    public void removeHasAssociatedActions(boolean oldHasAssociatedActions) {
        removePropertyValue(getHasAssociatedActionsProperty(), oldHasAssociatedActions);
    }


    public void setHasAssociatedActions(Collection newHasAssociatedActions) {
        setPropertyValues(getHasAssociatedActionsProperty(), newHasAssociatedActions);
    }


    // Property http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceProperty

    public String getResourceProperty() {
        return (String) getPropertyValue(getResourcePropertyProperty());
    }


    public RDFProperty getResourcePropertyProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceProperty";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasResourceProperty() {
        return getPropertyValueCount(getResourcePropertyProperty()) > 0;
    }


    public void setResourceProperty(String newResourceProperty) {
        setPropertyValue(getResourcePropertyProperty(), newResourceProperty);
    }
}
