/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl;

//import commonDtos.PhysicalHost;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.SetServerStateActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextResource;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class DefaultSetServerStateActivity extends DefaultConsolidationAction
        implements SetServerStateActivity {


    private Integer oldServerState = 0;

    public DefaultSetServerStateActivity(OWLModel owlModel, FrameID id) {
        super(owlModel, id);
    }


    public DefaultSetServerStateActivity() {
    }


    public int getTargetServerState() {
        if (hasTargetServerState()) {
            return getPropertyValueLiteral(getTargetServerStateProperty()).getInt();
        } else {
            setTargetServerState(0);
            return 0;
        }
    }


    public RDFProperty getTargetServerStateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#targetServerState";
        final String name = getOWLModel().getResourceNameForURI(uri);
        return getOWLModel().getRDFProperty(name);
    }


    public boolean hasTargetServerState() {
        return getPropertyValueCount(getTargetServerStateProperty()) > 0;
    }


    public void setTargetServerState(int newTargetServerState) {
        setPropertyValue(getTargetServerStateProperty(), new java.lang.Integer(newTargetServerState));
    }


    public void execute(EACMInstance modelAccess) {
        for (ContextResource resource : getResources()) {
            ServiceCenterServer server = modelAccess.getServiceCenterServer(resource.getName());
//            Collection<ServiceCenterITComputingResource> serverResources = server.getResources();
//            Collection<ServiceCenterServer> attachedComplexResources = new ArrayList<ServiceCenterServer>(serverResources.size());
//            for (ContextResource contextResource : serverResources) {
//                if (contextResource instanceof ServiceCenterServer) {
//                    attachedComplexResources.add((ServiceCenterServer) contextResource);
//                }
//            }

//            final CountDownLatch countDownLatch = new CountDownLatch(attachedComplexResources.size());

            oldServerState = server.getCurrentEnergyState();
            int newState = getTargetServerState();
            server.setCurrentEnergyState(newState);
            if (newState == 0) {
                server.setIsActive(false);
            } else {
                server.setIsActive(true);
            }


        }
    }

//    public void executeOnServiceCenter(EACMInstance modelAccess) throws ApplicationException {
//        for (ContextResource resource : getResources()) {
//            ServiceCenterServer server = (ServiceCenterServer) resource;
//            WorldInterfaceDelegate delegate = WorldInterfaceDelegate.getInstance();
//            PhysicalHost host = new PhysicalHost();
//            host.setHostname(server.getIpAddress());
//            host.setMac(server.getMacAddress());
//            host.setId(server.getId());
//            if (getTargetServerState() == 0) {
//                delegate.sendServerToSleep(host);
//                delegate.disableHost(host);
//            } else {
//                delegate.wakeUpServer(host);
//                delegate.enableHost(host);
//            }
//
//        }
//    }


    public void undo(EACMInstance modelAccess) {
        for (ContextResource resource : getResources()) {
            ServiceCenterServer server = modelAccess.getServiceCenterServer(resource.getName());
            server.setCurrentEnergyState(oldServerState);
            if (oldServerState == 0) {
                server.setIsActive(false);
            } else {
                server.setIsActive(true);
            }
        }
    }

    public void setCost(int cost) {

    }

    public int getCost() {
        if (oldServerState == 0) return 600;
        else return 0;
    }


    public String toString() {
        String serversList = "";
        for (ContextResource resource : getResources()) {
            ServiceCenterServer server = (ServiceCenterServer) resource;
            serversList += server.getName().split("#")[1] + ", ";
        }
        return "\t Set Server State" + " " + serversList + " to " + ((this.getTargetServerState() == 0) ? "OFF" : "ON");
    }


//    public boolean equals(Object o) {
//        if (!(o instanceof DefaultSetServerStateActivity)) {
//            return super.equals(o);
//        }
//        DefaultSetServerStateActivity action = (DefaultSetServerStateActivity) o;
//        return (action.getName().equals(this.getName())) || (action.getResources().equals(this.getResources()));// && ((action.getTargetServerState() == this.getTargetServerState()) || (action.getTargetServerState() == this.oldServerState)));
//    }
}
