/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.modelAPI;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.*;
import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.server.RemoteProjectManager;
import edu.stanford.smi.protegex.owl.javacode.ProtegeJavaMapping;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSNamedClass;
import edu.stanford.smi.protegex.owl.model.impl.OWLUtil;

import edu.stanford.smi.protegex.owl.swrl.model.SWRLFactory;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.ContextModelAccessException;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class OntologyEACMInstance implements EACMInstance {

    public static final String ontologyFile = "./eacmOntology/EnergyAwareContextModel/EACM.owl";
    private OWLModel owlModel;
    private RemoteProjectManager rpm;
    private Project p;

    static {
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextPolicy", ContextPolicy.class, DefaultContextPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextElement", ContextElement.class, DefaultContextElement.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDDIntensiveActivity", HDDIntensiveActivity.class, DefaultHDDIntensiveActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivity", ApplicationActivity.class, DefaultApplicationActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServerAdaptationAction", ServerAdaptationAction.class, DefaultServerAdaptationAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITComputingResourceAdaptationAction", ITComputingResourceAdaptationAction.class, DefaultITComputingResourceAdaptationAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#QoSPolicy", QoSPolicy.class, DefaultQoSPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BusinessPolicy", BusinessPolicy.class, DefaultBusinessPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ExternalStorage", ExternalStorage.class, DefaultExternalStorage.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ComplexResource", ComplexResource.class, DefaultComplexResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BusinessContextResource", BusinessContextResource.class, DefaultBusinessContextResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Application", Application.class, DefaultApplication.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#EnvironmentPolicy", EnvironmentPolicy.class, DefaultEnvironmentPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Policy", GPI_KPI_Policy.class, DefaultGPI_KPI_Policy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#DeployActivity", DeployActivity.class, DefaultDeployActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ConsolidationAction", ConsolidationAction.class, DefaultConsolidationAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationRedesign", ApplicationRedesign.class, DefaultApplicationRedesignAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationAdaptationAction", ApplicationAdaptationAction.class, DefaultApplicationAdaptationAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SLAPolicy", SLAPolicy.class, DefaultSLAPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MigrateActivity", MigrateActivity.class, DefaultMigrateActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterITComputingResource", ServiceCenterITComputingResource.class, DefaultServiceCenterITComputingResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextResource", ContextResource.class, DefaultContextResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITComputingContextPolicy", ITComputingContextPolicy.class, DefaultITComputingContextPolicy.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextAction", ContextAction.class, DefaultContextAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Facility", Facility.class, DefaultFacility.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityActiveResource", ITFacilityActiveResource.class, DefaultITFacilityActiveResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivityDiskInfo", ApplicationActivityDiskInfo.class, DefaultApplicationActivityDiskInfo.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Actuator", Actuator.class, DefaultActuator.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityPassiveResource", ITFacilityPassiveResource.class, DefaultITFacilityPassiveResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterITFacilityResource", ServiceCenterITFacilityResource.class, DefaultServiceCenterITFacilityResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPU", CPU.class, DefaultCPU.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterServer", ServiceCenterServer.class, DefaultServiceCenterServer.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Core", Core.class, DefaultCore.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SimpleResource", SimpleResource.class, DefaultSimpleResource.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Sensor", Sensor.class, DefaultSensor.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEM", MEM.class, DefaultMEM.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDD", HDD.class, DefaultHDD.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#DPMAction", DPMAction.class, DefaultDPMAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivityNetworkInfo", ApplicationActivityNetworkInfo.class, DefaultApplicationActivityNetworkInfo.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityResourceAdaptationAction", ITFacilityResourceAdaptationAction.class, DefaultITFacilityResourceAdaptationAction.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SetServerStateActivity", SetServerStateActivity.class, DefaultSetServerStateActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEMIntensiveActivity", MEMIntensiveActivity.class, DefaultMEMIntensiveActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUIntensiveActivity", CPUIntensiveActivity.class, DefaultCPUIntensiveActivity.class);
        ProtegeJavaMapping.add("http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#FacilityDefaultAction", FacilityDefaultAction.class, DefaultFacilityDefaultAction.class);
    }

    public OntologyEACMInstance(OWLModel owlModel) {
        this.owlModel = owlModel;
    }


    public OntologyEACMInstance() throws ContextModelAccessException {
        try {
            rpm = RemoteProjectManager.getInstance();
            p = rpm.getProject("localhost", "GAMES", "EACM", "EACMDatabase", true);
            owlModel = (OWLModel) p.getKnowledgeBase();
            SWRLFactory factory = new SWRLFactory(owlModel);
        } catch (Exception e) {
            ContextModelAccessException accessException = new ContextModelAccessException(e.getMessage(), e.getCause());
            accessException.setStackTrace(e.getStackTrace());
            throw accessException;
        }
    }


    public OntologyEACMInstance(String ontologyFileServerIP) throws ContextModelAccessException {

        try {
            rpm = RemoteProjectManager.getInstance();
            p = rpm.getProject(ontologyFileServerIP, "GAMES", "EACM", "EACMDatabase", true);
            owlModel = (OWLModel) p.getKnowledgeBase();
            SWRLFactory factory = new SWRLFactory(owlModel);
        } catch (Exception e) {
            throw new ContextModelAccessException(e.getMessage(), e.getCause());
        }
    }

    public OntologyEACMInstance(String ontologyFileServerIP, String ontologyPath) throws ContextModelAccessException {

        try {
            // SWRLFactory factory = new SWRLFactory(owlModel);
            rpm = RemoteProjectManager.getInstance();

            p = rpm.getProject(ontologyFileServerIP, "GAMES", "EACM", "EACMDatabase", true);
            Collection c = new ArrayList();
            p.includeProject(ontologyPath, c);
            owlModel = (OWLModel) p.getKnowledgeBase();
        } catch (Exception e) {
            //e.printStackTrace();
            throw new ContextModelAccessException(e.getMessage(), e.getCause());
        }
    }

    public OWLModel getOwlModel() {
        return owlModel;
    }

    public RDFSNamedClass getCoreClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Core";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public Core createCore(String name) {
        final RDFSNamedClass cls = getCoreClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultCore newInstance = new DefaultCore(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);

        return newInstance;


    }

    public Core getCore(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<Core> items = getAllCoreInstances();
            for (Core item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof Core) {
            return (Core) res;
        } else if (res.hasProtegeType(getCoreClass())) {
            return new DefaultCore(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<Core> getAllCoreInstances() {
        return getAllCoreInstances(false);
    }

    private RDFSNamedClass getRemoveActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#RemoveActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }


    public RemoveActivity createRemoveActivity(String name) {

        final RDFSNamedClass cls = getRemoveActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        return new DefaultRemoveActivity(owlModel, cls.createInstance(name).getFrameID());
    }

    public RemoveActivity getRemoveActivity(String name) {
        RDFResource res = owlModel.getRDFResource(OWLUtil.getInternalFullName(owlModel, name));
        if (res == null) {
            return null;
        }
        if (res instanceof RemoveActivity) {
            return (RemoveActivity) res;
        } else if (res.hasProtegeType(getRemoveActivityClass())) {
            return new DefaultRemoveActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<RemoveActivity> getAllRemoveActivityInstances() {
        return getAllRemoveActivityInstances(false);
    }

    public Collection<RemoveActivity> getAllRemoveActivityInstances(boolean transitive) {
        Collection<RemoveActivity> result = new ArrayList<RemoveActivity>();
        final RDFSNamedClass cls = getRemoveActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultRemoveActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public Collection<Core> getAllCoreInstances(boolean transitive) {
        Collection<Core> result = new ArrayList<Core>();
        final RDFSNamedClass cls = getCoreClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultCore(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFProperty getServerIsUsefullProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#serverIsUsefull";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFSNamedClass getSetServerStateActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SetServerStateActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public SetServerStateActivity createSetServerStateActivity(String name) {
        final RDFSNamedClass cls = getSetServerStateActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultSetServerStateActivity newInstance = new DefaultSetServerStateActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public SetServerStateActivity getSetServerStateActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<SetServerStateActivity> items = getAllSetServerStateActivityInstances();
            for (SetServerStateActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof SetServerStateActivity) {
            return (SetServerStateActivity) res;
        } else if (res.hasProtegeType(getSetServerStateActivityClass())) {
            return new DefaultSetServerStateActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<SetServerStateActivity> getAllSetServerStateActivityInstances() {
        return getAllSetServerStateActivityInstances(true);
    }

    public Collection<SetServerStateActivity> getAllSetServerStateActivityInstances(boolean transitive) {
        Collection<SetServerStateActivity> result = new ArrayList<SetServerStateActivity>();
        final RDFSNamedClass cls = getSetServerStateActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultSetServerStateActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getFacilityDefaultActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#FacilityDefaultAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public FacilityDefaultAction createFacilityDefaultAction(String name) {
        final RDFSNamedClass cls = getFacilityDefaultActionClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultFacilityDefaultAction newInstance = new DefaultFacilityDefaultAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public FacilityDefaultAction getFacilityDefaultAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<FacilityDefaultAction> items = getAllFacilityDefaultActionInstances();
            for (FacilityDefaultAction item : items) {
                if (item.getName().equals(fullName)) {

                    return item;
                }
            }
            return null;
        }
        if (res instanceof FacilityDefaultAction) {
            return (FacilityDefaultAction) res;
        } else if (res.hasProtegeType(getFacilityDefaultActionClass())) {
            return new DefaultFacilityDefaultAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<FacilityDefaultAction> getAllFacilityDefaultActionInstances() {
        return getAllFacilityDefaultActionInstances(true);
    }

    public Collection<FacilityDefaultAction> getAllFacilityDefaultActionInstances(boolean transitive) {
        Collection<FacilityDefaultAction> result = new ArrayList<FacilityDefaultAction>();
        final RDFSNamedClass cls = getFacilityDefaultActionClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultFacilityDefaultAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getDPMActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#DPMAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public DPMAction createDPMAction(String name) {
        final RDFSNamedClass cls = getDPMActionClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultDPMAction newInstance = new DefaultDPMAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public DPMAction getDPMAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<DPMAction> items = getAllDPMActionInstances();
            for (DPMAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof DPMAction) {
            return (DPMAction) res;
        } else if (res.hasProtegeType(getDPMActionClass())) {
            return new DefaultDPMAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<DPMAction> getAllDPMActionInstances() {
        return getAllDPMActionInstances(true);
    }

    public Collection<DPMAction> getAllDPMActionInstances(boolean transitive) {
        Collection<DPMAction> result = new ArrayList<DPMAction>();
        final RDFSNamedClass cls = getDPMActionClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultDPMAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getITComputingResourceAdaptationActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITComputingResourceAdaptationAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ITComputingResourceAdaptationAction createITComputingResourceAdaptationAction(String name) {
//        final RDFSNamedClass cls = getITComputingResourceAdaptationActionClass();
//        if (name == null) {
//            name = owlModel.getNextAnonymousResourceName();
//        }
//        return new DefaultITComputingResourceAdaptationAction(owlModel, instance.getFrameID());
        return null;
    }

    public ITComputingResourceAdaptationAction getITComputingResourceAdaptationAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ITComputingResourceAdaptationAction> items = getAllITComputingResourceAdaptationActionInstances();
            for (ITComputingResourceAdaptationAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ITComputingResourceAdaptationAction) {
            return (ITComputingResourceAdaptationAction) res;
        } else {
            try {
                Class individualClass = null;
                if (res.hasRDFType(getDPMActionClass())) {
                    individualClass = DefaultDPMAction.class;
                } else if (res.hasRDFType(getDeployActivityClass())) {
                    individualClass = DefaultDeployActivity.class;
                } else if (res.hasRDFType(getMigrateActivityClass())) {
                    individualClass = DefaultMigrateActivity.class;
                } else if (res.hasRDFType(getSetServerStateActivityClass())) {
                    individualClass = DefaultSetServerStateActivity.class;
                }else{
                    return null;
                }
                return (ITComputingResourceAdaptationAction) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID()));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return null;
    }

    public Collection<ITComputingResourceAdaptationAction> getAllITComputingResourceAdaptationActionInstances() {
        return getAllITComputingResourceAdaptationActionInstances(true);
    }

    public Collection<ITComputingResourceAdaptationAction> getAllITComputingResourceAdaptationActionInstances(boolean transitive) {
        Collection<ITComputingResourceAdaptationAction> result = new ArrayList<ITComputingResourceAdaptationAction>();
        final RDFSNamedClass cls = getITComputingResourceAdaptationActionClass();
        RDFResource res;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            res = (RDFResource) it.next();
            try {
                Class individualClass = null;
                if (res.hasRDFType(getDPMActionClass())) {
                    individualClass = DefaultDPMAction.class;
                } else if (res.hasRDFType(getDeployActivityClass())) {
                    individualClass = DefaultDeployActivity.class;
                } else if (res.hasRDFType(getMigrateActivityClass())) {
                    individualClass = DefaultMigrateActivity.class;
                } else if (res.hasRDFType(getSetServerStateActivityClass())) {
                    individualClass = DefaultSetServerStateActivity.class;
                }
                result.add(
                        (ITComputingResourceAdaptationAction) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID())));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return result;
    }

    public RDFSNamedClass getContextActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    //    public ContextAction createContextAction(String name) {
//        final RDFSNamedClass cls = getContextActionClass();
//        if (name == null) {
//            name = owlModel.getNextAnonymousResourceName();
//        }
//        return new DefaultContextAction(owlModel, instance.getFrameID());
//    }
//
//    public ContextAction getContextAction(String name) {
//         String fullName = OWLUtil.getInternalFullName(owlModel, name);         RDFResource res = owlModel.getRDFIndividual(fullName);
//        if (res==null)   1 {  {
//            return null;
//        }
//        if (res instanceof ContextAction) {
//            return (ContextAction) res;
//        } else if (res.hasProtegeType(getContextActionClass())) {
//            return new DefaultContextAction(owlModel, res.getFrameID());
//        }
//        return null;
//    }
//
//    public Collection<ContextAction> getAllContextActionInstances() {
//        return getAllContextActionInstances(true);
//    }
//
//    public Collection<ContextAction> getAllContextActionInstances(boolean transitive) {
//        Collection<ContextAction> result = new ArrayList<ContextAction>();
//        final RDFSNamedClass cls = getContextActionClass();
//        RDFResource owlIndividual;
//        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext();) {
//            owlIndividual = (RDFResource) it.next();
//            result.add(new DefaultContextAction(owlModel, owlIndividual.getFrameID()));
//        }
//        return result;
//    }
//
    public RDFSNamedClass getHDDIntensiveActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDDIntensiveActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public HDDIntensiveActivity createHDDIntensiveActivity(String name) {
        final RDFSNamedClass cls = getHDDIntensiveActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultHDDIntensiveActivity newInstance = new DefaultHDDIntensiveActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public HDDIntensiveActivity getHDDIntensiveActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<HDDIntensiveActivity> items = getAllHDDIntensiveActivityInstances();
            for (HDDIntensiveActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof HDDIntensiveActivity) {
            return (HDDIntensiveActivity) res;
        } else if (res.hasProtegeType(getHDDIntensiveActivityClass())) {
            return new DefaultHDDIntensiveActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<HDDIntensiveActivity> getAllHDDIntensiveActivityInstances() {
        return getAllHDDIntensiveActivityInstances(true);
    }

    public Collection<HDDIntensiveActivity> getAllHDDIntensiveActivityInstances(boolean transitive) {
        Collection<HDDIntensiveActivity> result = new ArrayList<HDDIntensiveActivity>();
        final RDFSNamedClass cls = getHDDIntensiveActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultHDDIntensiveActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getApplicationActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ApplicationActivity createApplicationActivity(String name) {
        final RDFSNamedClass cls = getApplicationActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }

//        instance.setRDFType(getApplicationActivityClass());
//        owlModel.setDirectType(instance, cls);         instance.setRDFType(cls);         // instance.setDirectType(cls);;
//        p.getKnowledgeBase().commitTransaction();
//        instance.setDirectType(getApplicationActivityClass());
//          owlModel.setDirectType(instance, cls);         instance.setRDFType(cls);         // instance.setDirectType(cls);;
//        p.getKnowledgeBase().addDirectType(instance, cls);
        Instance instance = cls.createInstance(name);
        DefaultApplicationActivity newInstance = new DefaultApplicationActivity(owlModel, instance.getFrameID());
//        owlModel.setDirectType(instance, cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ApplicationActivity getApplicationActivity(String name) {

        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);

        if (res == null) {
//            Collection<ApplicationActivity> items = getAllApplicationActivityInstances();
//            for (ApplicationActivity item : items) {
//                if (item.getName().equals(fullName)) {
//                    return item;
//                }
//            }
//            System.err.println();
            return null;
        }
        if (res instanceof ApplicationActivity) {
            return (ApplicationActivity) res;
        } else if (res.hasProtegeType(getApplicationActivityClass()) || res.hasDirectType(getApplicationActivityClass()) || res.hasRDFType(getApplicationActivityClass())) {
            return new DefaultApplicationActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ApplicationActivity> getAllApplicationActivityInstances() {
        return getAllApplicationActivityInstances(true);
    }

    public Collection<ApplicationActivity> getAllApplicationActivityInstances(boolean transitive) {
        Collection<ApplicationActivity> result = new ArrayList<ApplicationActivity>();
        final RDFSNamedClass cls = getApplicationActivityClass();
        RDFResource owlIndividual;

        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            if (owlModel.getFrame(owlIndividual.getFrameID()) == null) {
                continue;
            }
            result.add(new DefaultApplicationActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getGPI_KPI_PolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#GPI_KPI_Policy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public GPI_KPI_Policy createGPI_KPI_Policy(String name) {
//        final RDFSNamedClass cls = getGPI_KPI_PolicyClass();
//        if (name == null) {
//            name = owlModel.getNextAnonymousResourceName();
//        }
        throw new UnsupportedOperationException("Please create a more specific policy from:"
                + " QoSPolicy, SLAPolicy, ITComputingContextPolicy,EnvironmentPolicyClass ");
    }

    public GPI_KPI_Policy getGPI_KPI_Policy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
//            Collection<GPI_KPI_Policy> items = getAllGPI_KPI_PolicyInstances();
//            for (GPI_KPI_Policy item : items) {
//                if (item.getName().equals(fullName)) {
//                    return item;
//                }
//            }
            return null;
        }
        if (res instanceof GPI_KPI_Policy) {
            return (GPI_KPI_Policy) res;
        } else {
            try {
                Class individualClass = null;
                if (res.hasRDFType(getEnvironmentPolicyClass())) {
                    individualClass = DefaultEnvironmentPolicy.class;
                } else if (res.hasRDFType(getITComputingContextPolicyClass())) {
                    individualClass = DefaultITComputingContextPolicy.class;
                } else if (res.hasRDFType(getQoSPolicyClass())) {
                    individualClass = DefaultQoSPolicy.class;
                } else if (res.hasRDFType(getSLAPolicyClass())) {
                    individualClass = DefaultSLAPolicy.class;
                }
                if (individualClass == null) {
                    return null;
                }
                return (GPI_KPI_Policy) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID()));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return null;
    }

    public Collection<GPI_KPI_Policy> getAllGPI_KPI_PolicyInstances() {
        return getAllGPI_KPI_PolicyInstances(true);
    }

    public Collection<GPI_KPI_Policy> getAllGPI_KPI_PolicyInstances(boolean transitive) {
        Collection<GPI_KPI_Policy> result = new ArrayList<GPI_KPI_Policy>();
        final RDFSNamedClass cls = getGPI_KPI_PolicyClass();
        RDFResource res;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            res = (RDFResource) it.next();
            try {
                Class individualClass = null;
                if (res.hasRDFType(getEnvironmentPolicyClass())) {
                    individualClass = DefaultEnvironmentPolicy.class;
                } else if (res.hasRDFType(getITComputingContextPolicyClass())) {
                    individualClass = DefaultITComputingContextPolicy.class;
                } else if (res.hasRDFType(getQoSPolicyClass())) {
                    individualClass = DefaultQoSPolicy.class;
                } else if (res.hasRDFType(getSLAPolicyClass())) {
                    individualClass = DefaultSLAPolicy.class;
                }
                if (individualClass == null) {
                    continue;
                }
                result.add((GPI_KPI_Policy) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID())));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return result;
    }

    public RDFSNamedClass getContextPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ContextPolicy createContextPolicy(String name) {
        throw new UnsupportedOperationException("Please create a more specific policy: QoSPolicy, SLAPolicy, ITComputingContextPolicy or EnvironmentPolicy ");
    }

    public ContextPolicy getContextPolicy(String name) {
        return getGPI_KPI_Policy(name);
    }

    public Collection<ContextPolicy> getAllContextPolicyInstances() {
        return getAllContextPolicyInstances(true);
    }

    public Collection<ContextPolicy> getAllContextPolicyInstances(boolean transitive) {
        Collection<ContextPolicy> result = new ArrayList<ContextPolicy>();
        final RDFSNamedClass cls = getGPI_KPI_PolicyClass();
        RDFResource res;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            res = (RDFResource) it.next();
            try {
                Class individualClass = null;
                if (res.hasRDFType(getEnvironmentPolicyClass())) {
                    individualClass = DefaultEnvironmentPolicy.class;
                } else if (res.hasRDFType(getITComputingContextPolicyClass())) {
                    individualClass = DefaultITComputingContextPolicy.class;
                } else if (res.hasRDFType(getQoSPolicyClass())) {
                    individualClass = DefaultQoSPolicy.class;
                } else if (res.hasRDFType(getSLAPolicyClass())) {
                    individualClass = DefaultSLAPolicy.class;
                }
                result.add((ContextPolicy) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID())));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return result;
    }

    public RDFSNamedClass getServiceCenterITComputingResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterITComputingResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ServiceCenterITComputingResource createServiceCenterITComputingResource(String name) {
        final RDFSNamedClass cls = getServiceCenterITComputingResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultServiceCenterITComputingResource newInstance = new DefaultServiceCenterITComputingResource(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ServiceCenterITComputingResource getServiceCenterITComputingResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ServiceCenterITComputingResource> items = getAllServiceCenterITComputingResourceInstances();
            for (ServiceCenterITComputingResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ServiceCenterITComputingResource) {
            return (ServiceCenterITComputingResource) res;
        } else if (res.hasProtegeType(getServiceCenterITComputingResourceClass())) {
            return new DefaultServiceCenterITComputingResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ServiceCenterITComputingResource> getAllServiceCenterITComputingResourceInstances() {
        return getAllServiceCenterITComputingResourceInstances(true);
    }

    public Collection<ServiceCenterITComputingResource> getAllServiceCenterITComputingResourceInstances(boolean transitive) {
        Collection<ServiceCenterITComputingResource> result = new ArrayList<ServiceCenterITComputingResource>();
        final RDFSNamedClass cls = getServiceCenterITComputingResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            if (owlModel.getFrame(owlIndividual.getFrameID()) == null) {
                continue;
            }
            result.add(new DefaultServiceCenterITComputingResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getContextResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ContextResource createContextResource(String name) {
        final RDFSNamedClass cls = getContextResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultContextResource newInstance = new DefaultContextResource(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ContextResource getContextResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
//            Collection<ContextResource> items = getAllContextResourceInstances();
//            for (ContextResource item : items) {
//                if (item.getName().equals(fullName)) {
//
//                    return item;
//                }
//            }
            return null;
        }

        return new DefaultContextResource(owlModel, res.getFrameID());

    }

    public Collection<ContextResource> getAllContextResourceInstances() {
        return getAllContextResourceInstances(true);
    }

    public Collection<ContextResource> getAllContextResourceInstances(boolean transitive) {
        Collection<ContextResource> result = new ArrayList<ContextResource>();
        final RDFSNamedClass cls = getContextResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultContextResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getServiceCenterServerClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterServer";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ServiceCenterServer createServiceCenterServer(String name) {
        final RDFSNamedClass cls = getServiceCenterServerClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultServiceCenterServer newInstance = new DefaultServiceCenterServer(owlModel, instance.getFrameID());

//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
//       // newInstance.setRDFType(cls);
//        // instance.setDirectType(cls);;
//        //newInstance.addRDFType(cls);
//       // instance.addDirectType(cls);
//       // newInstance.setProtegeType(cls);
//        //newInstance.addProtegeType(cls);
//       // new// instance.setDirectType(cls);;
//       //  newInstance.addDirectType(cls);
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ServiceCenterServer getServiceCenterServer(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ServiceCenterServer> items = getAllServiceCenterServerInstances();
            for (ServiceCenterServer item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }

            return null;
        }
        if (res instanceof ServiceCenterServer) {
            return (ServiceCenterServer) res;
        } else if (res.hasProtegeType(getServiceCenterServerClass())) {
            return new DefaultServiceCenterServer(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ServiceCenterServer> getAllServiceCenterServerInstances() {
        return getAllServiceCenterServerInstances(true);
    }

    public Collection<ServiceCenterServer> getAllServiceCenterServerInstances(boolean transitive) {
        Collection<ServiceCenterServer> result = new ArrayList<ServiceCenterServer>();
        final RDFSNamedClass cls = getServiceCenterServerClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            if (owlModel.getFrame(owlIndividual.getFrameID()) == null) {
                continue;
            }
            result.add(new DefaultServiceCenterServer(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getComplexResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ComplexResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ComplexResource createComplexResource(String name) {
        final RDFSNamedClass cls = getComplexResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultComplexResource newInstance = new DefaultComplexResource(owlModel, instance.getFrameID());

//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
//       // newInstance.setRDFType(cls);
//        // instance.setDirectType(cls);;
//        //newInstance.addRDFType(cls);
//       // instance.addDirectType(cls);
//       // newInstance.setProtegeType(cls);
//        //newInstance.addProtegeType(cls);
//       // new// instance.setDirectType(cls);;
//       //  newInstance.addDirectType(cls);
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ComplexResource getComplexResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ComplexResource> items = getAllComplexResourceInstances();
            for (ComplexResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }

            return null;
        }
        if (res instanceof ComplexResource) {
            return (ComplexResource) res;
        } else if (res.hasProtegeType(getComplexResourceClass())) {
            return new DefaultComplexResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ComplexResource> getAllComplexResourceInstances() {
        return getAllComplexResourceInstances(true);
    }

    public Collection<ComplexResource> getAllComplexResourceInstances(boolean transitive) {
        Collection<ComplexResource> result = new ArrayList<ComplexResource>();
        final RDFSNamedClass cls = getComplexResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultComplexResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getContextElementClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ContextElement";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ContextElement createContextElement(String name) {
        final RDFSNamedClass cls = getContextElementClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultContextElement newInstance = new DefaultContextElement(owlModel, instance.getFrameID());

//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
//       // newInstance.setRDFType(cls);
//        // instance.setDirectType(cls);;
//        //newInstance.addRDFType(cls);
//       // instance.addDirectType(cls);
//       // newInstance.setProtegeType(cls);
//        //newInstance.addProtegeType(cls);
//       // new// instance.setDirectType(cls);;
//       //  newInstance.addDirectType(cls);
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ContextElement getContextElement(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ContextElement> items = getAllContextElementInstances();
            for (ContextElement item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }

            return null;
        }
        if (res instanceof ContextElement) {
            return (ContextElement) res;
        } else if (res.hasProtegeType(getContextElementClass())) {
            return new DefaultContextElement(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ContextElement> getAllContextElementInstances() {
        return getAllContextElementInstances(true);
    }

    public Collection<ContextElement> getAllContextElementInstances(boolean transitive) {
        Collection<ContextElement> result = new ArrayList<ContextElement>();
        final RDFSNamedClass cls = getContextElementClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultContextElement(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getServiceCenterITFacilityResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServiceCenterITFacilityResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ServiceCenterITFacilityResource createServiceCenterITFacilityResource(String name) {
        final RDFSNamedClass cls = getServiceCenterITFacilityResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultServiceCenterITFacilityResource newInstance = new DefaultServiceCenterITFacilityResource(owlModel, instance.getFrameID());

//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
//       // newInstance.setRDFType(cls);
//        // instance.setDirectType(cls);;
//        //newInstance.addRDFType(cls);
//       // instance.addDirectType(cls);
//       // newInstance.setProtegeType(cls);
//        //newInstance.addProtegeType(cls);
//       // new// instance.setDirectType(cls);;
//       //  newInstance.addDirectType(cls);
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ServiceCenterITFacilityResource getServiceCenterITFacilityResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ServiceCenterITFacilityResource> items = getAllServiceCenterITFacilityResourceInstances();
            for (ServiceCenterITFacilityResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ServiceCenterITFacilityResource) {
            return (ServiceCenterITFacilityResource) res;
        } else if (res.hasProtegeType(getServiceCenterITFacilityResourceClass())) {
            return new DefaultServiceCenterITFacilityResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ServiceCenterITFacilityResource> getAllServiceCenterITFacilityResourceInstances() {
        return getAllServiceCenterITFacilityResourceInstances(true);
    }

    public Collection<ServiceCenterITFacilityResource> getAllServiceCenterITFacilityResourceInstances(boolean transitive) {
        Collection<ServiceCenterITFacilityResource> result = new ArrayList<ServiceCenterITFacilityResource>();
        final RDFSNamedClass cls = getServiceCenterITFacilityResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultServiceCenterITFacilityResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getBusinessContextResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BusinessContextResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public BusinessContextResource createBusinessContextResource(String name) {
        final RDFSNamedClass cls = getBusinessContextResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultBusinessContextResource newInstance = new DefaultBusinessContextResource(owlModel, instance.getFrameID());
//
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
//       // newInstance.setRDFType(cls);
//        // instance.setDirectType(cls);;
//        //newInstance.addRDFType(cls);
//       // instance.addDirectType(cls);
//       // newInstance.setProtegeType(cls);
//        //newInstance.addProtegeType(cls);
//       // new// instance.setDirectType(cls);;
//       //  newInstance.addDirectType(cls);
//        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
//        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public BusinessContextResource getBusinessContextResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<BusinessContextResource> items = getAllBusinessContextResourceInstances();
            for (BusinessContextResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof BusinessContextResource) {
            return (BusinessContextResource) res;
        } else if (res.hasProtegeType(getBusinessContextResourceClass())) {
            return new DefaultBusinessContextResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<BusinessContextResource> getAllBusinessContextResourceInstances() {
        return getAllBusinessContextResourceInstances(true);
    }

    public Collection<BusinessContextResource> getAllBusinessContextResourceInstances(boolean transitive) {
        Collection<BusinessContextResource> result = new ArrayList<BusinessContextResource>();
        final RDFSNamedClass cls = getBusinessContextResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultBusinessContextResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getApplicationClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Application";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public Application createApplication(String name) {
        final RDFSNamedClass cls = getApplicationClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultApplication newInstance = new DefaultApplication(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public Application getApplication(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<Application> items = getAllApplicationInstances();
            for (Application item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof Application) {
            return (Application) res;
        } else if (res.hasProtegeType(getApplicationClass())) {
            return new DefaultApplication(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<Application> getAllApplicationInstances() {
        return getAllApplicationInstances(true);
    }

    public Collection<Application> getAllApplicationInstances(boolean transitive) {
        Collection<Application> result = new ArrayList<Application>();
        final RDFSNamedClass cls = getApplicationClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultApplication(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getCPUIntensiveActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPUIntensiveActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public CPUIntensiveActivity createCPUIntensiveActivity(String name) {
        final RDFSNamedClass cls = getCPUIntensiveActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultCPUIntensiveActivity newInstance = new DefaultCPUIntensiveActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public CPUIntensiveActivity getCPUIntensiveActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<CPUIntensiveActivity> items = getAllCPUIntensiveActivityInstances();
            for (CPUIntensiveActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof CPUIntensiveActivity) {
            return (CPUIntensiveActivity) res;
        } else if (res.hasProtegeType(getCPUIntensiveActivityClass())) {
            return new DefaultCPUIntensiveActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<CPUIntensiveActivity> getAllCPUIntensiveActivityInstances() {
        return getAllCPUIntensiveActivityInstances(true);
    }

    public Collection<CPUIntensiveActivity> getAllCPUIntensiveActivityInstances(boolean transitive) {
        Collection<CPUIntensiveActivity> result = new ArrayList<CPUIntensiveActivity>();
        final RDFSNamedClass cls = getCPUIntensiveActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultCPUIntensiveActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getApplicationAdaptationActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationAdaptationAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ApplicationAdaptationAction createApplicationAdaptationAction(String name) {
        final RDFSNamedClass cls = getApplicationAdaptationActionClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultApplicationAdaptationAction newInstance = new DefaultApplicationAdaptationAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;
    }

    public ApplicationAdaptationAction getApplicationAdaptationAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ApplicationAdaptationAction> items = getAllApplicationAdaptationActionInstances();
            for (ApplicationAdaptationAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ApplicationAdaptationAction) {
            return (ApplicationAdaptationAction) res;
        } else if (res.hasProtegeType(getApplicationAdaptationActionClass())) {
            return new DefaultApplicationAdaptationAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ServerAdaptationAction> getAllServerAdaptationActionInstances() {
        return getAllServerAdaptationActionInstances(true);
    }

    public Collection<ServerAdaptationAction> getAllServerAdaptationActionInstances(boolean transitive) {
        Collection<ServerAdaptationAction> result = new ArrayList<ServerAdaptationAction>();
        final RDFSNamedClass cls = getServerAdaptationActionClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultServerAdaptationAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getServerAdaptationActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ServerAdaptationAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public boolean isSimulation() {
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void setSimulation(boolean simulation) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public ServerAdaptationAction createServerAdaptationAction(String name) {
        final RDFSNamedClass cls = getServerAdaptationActionClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultServerAdaptationAction newInstance = new DefaultServerAdaptationAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ServerAdaptationAction getServerAdaptationAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ServerAdaptationAction> items = getAllServerAdaptationActionInstances();
            for (ServerAdaptationAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ApplicationAdaptationAction) {
            return (ServerAdaptationAction) res;
        } else if (res.hasProtegeType(getApplicationAdaptationActionClass())) {
            return new DefaultServerAdaptationAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ApplicationAdaptationAction> getAllApplicationAdaptationActionInstances() {
        return getAllApplicationAdaptationActionInstances(true);
    }

    public Collection<ApplicationAdaptationAction> getAllApplicationAdaptationActionInstances(boolean transitive) {
        Collection<ApplicationAdaptationAction> result = new ArrayList<ApplicationAdaptationAction>();
        final RDFSNamedClass cls = getApplicationAdaptationActionClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultApplicationAdaptationAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getApplicationRedesignClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationRedesign";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ApplicationRedesign createApplicationRedesign(String name) {
        final RDFSNamedClass cls = getApplicationRedesignClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultApplicationRedesignAction newInstance = new DefaultApplicationRedesignAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ApplicationRedesign getApplicationRedesign(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ApplicationRedesign> items = getAllApplicationRedesignInstances();
            for (ApplicationRedesign item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ApplicationRedesign) {
            return (ApplicationRedesign) res;
        } else if (res.hasProtegeType(getApplicationRedesignClass())) {
            return new DefaultApplicationRedesignAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ApplicationRedesign> getAllApplicationRedesignInstances() {
        return getAllApplicationRedesignInstances(true);
    }

    public Collection<ApplicationRedesign> getAllApplicationRedesignInstances(boolean transitive) {
        Collection<ApplicationRedesign> result = new ArrayList<ApplicationRedesign>();
        final RDFSNamedClass cls = getApplicationRedesignClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultApplicationRedesignAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getITFacilityActiveResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityActiveResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ITFacilityActiveResource createITFacilityActiveResource(String name) {
        final RDFSNamedClass cls = getITFacilityActiveResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultITFacilityActiveResource newInstance = new DefaultITFacilityActiveResource(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ITFacilityActiveResource getITFacilityActiveResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ITFacilityActiveResource> items = getAllITFacilityActiveResourceInstances();
            for (ITFacilityActiveResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ITFacilityActiveResource) {
            return (ITFacilityActiveResource) res;
        } else if (res.hasProtegeType(getITFacilityActiveResourceClass())) {
            return new DefaultITFacilityActiveResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ITFacilityActiveResource> getAllITFacilityActiveResourceInstances() {
        return getAllITFacilityActiveResourceInstances(true);
    }

    public Collection<ITFacilityActiveResource> getAllITFacilityActiveResourceInstances(boolean transitive) {
        Collection<ITFacilityActiveResource> result = new ArrayList<ITFacilityActiveResource>();
        final RDFSNamedClass cls = getITFacilityActiveResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultITFacilityActiveResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getHDDClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#HDD";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public HDD createHDD(String name) {
        final RDFSNamedClass cls = getHDDClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultHDD newInstance = new DefaultHDD(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public HDD getHDD(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<HDD> items = getAllHDDInstances();
            for (HDD item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof HDD) {
            return (HDD) res;
        } else if (res.hasProtegeType(getHDDClass())) {
            return new DefaultHDD(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<HDD> getAllHDDInstances() {
        return getAllHDDInstances(true);
    }

    public Collection<HDD> getAllHDDInstances(boolean transitive) {
        Collection<HDD> result = new ArrayList<HDD>();
        final RDFSNamedClass cls = getHDDClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultHDD(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getSimpleResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SimpleResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public SimpleResource createSimpleResource(String name) {
        final RDFSNamedClass cls = getSimpleResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultSimpleResource newInstance = new DefaultSimpleResource(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public SimpleResource getSimpleResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<SimpleResource> items = getAllSimpleResourceInstances();
            for (SimpleResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof SimpleResource) {
            return (SimpleResource) res;
        } else if (res.hasProtegeType(getSimpleResourceClass())) {
            return new DefaultSimpleResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<SimpleResource> getAllSimpleResourceInstances() {
        return getAllSimpleResourceInstances(true);
    }

    public Collection<SimpleResource> getAllSimpleResourceInstances(boolean transitive) {
        Collection<SimpleResource> result = new ArrayList<SimpleResource>();
        final RDFSNamedClass cls = getSimpleResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultSimpleResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getITComputingContextPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITComputingContextPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ITComputingContextPolicy createITComputingContextPolicy(String name) {
        final RDFSNamedClass cls = getITComputingContextPolicyClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultITComputingContextPolicy newInstance = new DefaultITComputingContextPolicy(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ITComputingContextPolicy getITComputingContextPolicy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ITComputingContextPolicy> items = getAllITComputingContextPolicyInstances();
            for (ITComputingContextPolicy item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ITComputingContextPolicy) {
            return (ITComputingContextPolicy) res;
        } else if (res.hasProtegeType(getITComputingContextPolicyClass())) {
            return new DefaultITComputingContextPolicy(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ITComputingContextPolicy> getAllITComputingContextPolicyInstances() {
        return getAllITComputingContextPolicyInstances(true);
    }

    public Collection<ITComputingContextPolicy> getAllITComputingContextPolicyInstances(boolean transitive) {
        Collection<ITComputingContextPolicy> result = new ArrayList<ITComputingContextPolicy>();
        final RDFSNamedClass cls = getITComputingContextPolicyClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultITComputingContextPolicy(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getITFacilityPassiveResourceClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityPassiveResource";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ITFacilityPassiveResource createITFacilityPassiveResource(String name) {
        final RDFSNamedClass cls = getITFacilityPassiveResourceClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultITFacilityPassiveResource newInstance = new DefaultITFacilityPassiveResource(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ITFacilityPassiveResource getITFacilityPassiveResource(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ITFacilityPassiveResource> items = getAllITFacilityPassiveResourceInstances();
            for (ITFacilityPassiveResource item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ITFacilityPassiveResource) {
            return (ITFacilityPassiveResource) res;
        } else if (res.hasProtegeType(getITFacilityPassiveResourceClass())) {
            return new DefaultITFacilityPassiveResource(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ITFacilityPassiveResource> getAllITFacilityPassiveResourceInstances() {
        return getAllITFacilityPassiveResourceInstances(true);
    }

    public Collection<ITFacilityPassiveResource> getAllITFacilityPassiveResourceInstances(boolean transitive) {
        Collection<ITFacilityPassiveResource> result = new ArrayList<ITFacilityPassiveResource>();
        final RDFSNamedClass cls = getITFacilityPassiveResourceClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultITFacilityPassiveResource(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getMigrateActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MigrateActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public MigrateActivity createMigrateActivity(String name) {
        final RDFSNamedClass cls = getMigrateActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultMigrateActivity newInstance = new DefaultMigrateActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public MigrateActivity getMigrateActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<MigrateActivity> items = getAllMigrateActivityInstances();
            for (MigrateActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof MigrateActivity) {
            return (MigrateActivity) res;
        } else if (res.hasProtegeType(getMigrateActivityClass())) {
            return new DefaultMigrateActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<MigrateActivity> getAllMigrateActivityInstances() {
        return getAllMigrateActivityInstances(true);
    }

    public Collection<MigrateActivity> getAllMigrateActivityInstances(boolean transitive) {
        Collection<MigrateActivity> result = new ArrayList<MigrateActivity>();
        final RDFSNamedClass cls = getMigrateActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultMigrateActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getConsolidationActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ConsolidationAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ConsolidationAction createConsolidationAction(String name) {
        throw new UnsupportedOperationException("Please create a more specific action from:"
                + "DeployActivityAction, MigrateActivityAction, SetServerStateAction");
    }

    public ConsolidationAction getConsolidationAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ConsolidationAction> items = getAllConsolidationActionInstances();
            for (ConsolidationAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ConsolidationAction) {
            return (ConsolidationAction) res;
        } else {
            try {
                Class individualClass = null;
                if (res.hasRDFType(getDeployActivityClass())) {
                    individualClass = DefaultDeployActivity.class;
                } else if (res.hasRDFType(getConsolidationActionClass())) {
                    individualClass = DefaultConsolidationAction.class;
                } else if (res.hasRDFType(getMigrateActivityClass())) {
                    individualClass = DefaultMigrateActivity.class;
                } else if (res.hasRDFType(getSetServerStateActivityClass())) {
                    individualClass = DefaultSetServerStateActivity.class;
                }else{
                    return null;
                }
                return (DefaultConsolidationAction) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID()));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return null;
    }

    public Collection<ConsolidationAction> getAllConsolidationActionInstances() {
        return getAllConsolidationActionInstances(true);
    }

    public Collection<ConsolidationAction> getAllConsolidationActionInstances(boolean transitive) {
        Collection<ConsolidationAction> result = new ArrayList<ConsolidationAction>();
        final RDFSNamedClass cls = getConsolidationActionClass();
        RDFResource res;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            res = (RDFResource) it.next();
            try {
                Class individualClass = null;
                if (res.hasRDFType(getDeployActivityClass())) {
                    individualClass = DefaultDeployActivity.class;
                } else if (res.hasRDFType(getConsolidationActionClass())) {
                    individualClass = DefaultConsolidationAction.class;
                } else if (res.hasRDFType(getSetServerStateActivityClass())) {
                    individualClass = DefaultSetServerStateActivity.class;
                } else if (res.hasRDFType(getMigrateActivityClass())) {
                    individualClass = DefaultMigrateActivity.class;
                } else if (res.hasRDFType(getRemoveActivityClass())) {
                    individualClass = DefaultRemoveActivity.class;
                }
                result.add((DefaultConsolidationAction) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID())));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return result;
    }

    public RDFSNamedClass getSensorClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Sensor";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public Sensor createSensor(String name) {
        final RDFSNamedClass cls = getSensorClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultSensor newInstance = new DefaultSensor(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public Sensor getSensor(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<Sensor> items = getAllSensorInstances();
            for (Sensor item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof Sensor) {
            return (Sensor) res;
        } else if (res.hasProtegeType(getSensorClass())) {
            return new DefaultSensor(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<Sensor> getAllSensorInstances() {
        return getAllSensorInstances(true);
    }

    public Collection<Sensor> getAllSensorInstances(boolean transitive) {
        Collection<Sensor> result = new ArrayList<Sensor>();
        final RDFSNamedClass cls = getSensorClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultSensor(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getActuatorClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Actuator";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public Actuator createActuator(String name) {
        final RDFSNamedClass cls = getActuatorClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultActuator newInstance = new DefaultActuator(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public Actuator getActuator(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<Actuator> items = getAllActuatorInstances();
            for (Actuator item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof Actuator) {
            return (Actuator) res;
        } else if (res.hasProtegeType(getActuatorClass())) {
            return new DefaultActuator(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<Actuator> getAllActuatorInstances() {
        return getAllActuatorInstances(true);
    }

    public Collection<Actuator> getAllActuatorInstances(boolean transitive) {
        Collection<Actuator> result = new ArrayList<Actuator>();
        final RDFSNamedClass cls = getActuatorClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultActuator(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getBusinessPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BusinessPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public BusinessPolicy createBusinessPolicy(String name) {
        throw new UnsupportedOperationException("Please create a more specific policy from:"
                + " QoSPolicy, SLAPolicy, ITComputingContextPolicy,EnvironmentPolicyClass ");
    }

    public BusinessPolicy getBusinessPolicy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {

            Collection<BusinessPolicy> items = getAllBusinessPolicyInstances();
            for (BusinessPolicy item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof BusinessPolicy) {
            return (BusinessPolicy) res;
        } else  {
            try {
                Class individualClass = null;
                if (res.hasRDFType(getQoSPolicyClass())) {
                    individualClass = DefaultQoSPolicy.class;
                } else if (res.hasRDFType(getSLAPolicyClass())) {
                    individualClass = DefaultSLAPolicy.class;
                }else{
                    return null;
                }
                return (BusinessPolicy) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID()));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return null;
    }

    public Collection<BusinessPolicy> getAllBusinessPolicyInstances() {
        return getAllBusinessPolicyInstances(true);
    }

    public Collection<BusinessPolicy> getAllBusinessPolicyInstances(boolean transitive) {
        Collection<BusinessPolicy> result = new ArrayList<BusinessPolicy>();
        final RDFSNamedClass cls = getBusinessPolicyClass();
        RDFResource res;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            res = (RDFResource) it.next();
            try {
                Class individualClass = null;
                if (res.hasRDFType(getQoSPolicyClass())) {
                    individualClass = DefaultQoSPolicy.class;
                } else if (res.hasRDFType(getSLAPolicyClass())) {
                    individualClass = DefaultSLAPolicy.class;
                }
                result.add((DefaultBusinessPolicy) (Class.forName(individualClass.getName()).getConstructor(OWLModel.class, FrameID.class).newInstance(owlModel, res.getFrameID())));
            } catch (InstantiationException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (ClassNotFoundException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchMethodException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (InvocationTargetException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return result;
    }

    public RDFSNamedClass getMEMClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEM";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public MEM createMEM(String name) {
        final RDFSNamedClass cls = getMEMClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultMEM newInstance = new DefaultMEM(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public MEM getMEM(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<MEM> items = getAllMEMInstances();
            for (MEM item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }

            return null;
        }
        if (res instanceof MEM) {
            return (MEM) res;
        } else if (res.hasProtegeType(getMEMClass())) {
            return new DefaultMEM(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<MEM> getAllMEMInstances() {
        return getAllMEMInstances(true);
    }

    public Collection<MEM> getAllMEMInstances(boolean transitive) {
        Collection<MEM> result = new ArrayList<MEM>();
        final RDFSNamedClass cls = getMEMClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultMEM(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getExternalStorageClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ExternalStorage";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ExternalStorage createExternalStorage(String name) {
        final RDFSNamedClass cls = getExternalStorageClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultExternalStorage newInstance = new DefaultExternalStorage(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ExternalStorage getExternalStorage(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ExternalStorage> items = getAllExternalStorageInstances();
            for (ExternalStorage item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ExternalStorage) {
            return (ExternalStorage) res;
        } else if (res.hasProtegeType(getExternalStorageClass())) {
            return new DefaultExternalStorage(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ExternalStorage> getAllExternalStorageInstances() {
        return getAllExternalStorageInstances(true);
    }

    public Collection<ExternalStorage> getAllExternalStorageInstances(boolean transitive) {
        Collection<ExternalStorage> result = new ArrayList<ExternalStorage>();
        final RDFSNamedClass cls = getExternalStorageClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultExternalStorage(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getCPUClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#CPU";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public CPU createCPU(String name) {
        final RDFSNamedClass cls = getCPUClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultCPU newInstance = new DefaultCPU(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public CPU getCPU(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            return null;
        }
        if (res instanceof CPU) {
            return (CPU) res;
        } else if (res.hasProtegeType(getCPUClass())) {
            return new DefaultCPU(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<CPU> getAllCPUInstances() {
        return getAllCPUInstances(true);
    }

    public Collection<CPU> getAllCPUInstances(boolean transitive) {
        Collection<CPU> result = new ArrayList<CPU>();
        final RDFSNamedClass cls = getCPUClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultCPU(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getSLAPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#SLAPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public SLAPolicy createSLAPolicy(String name) {
        final RDFSNamedClass cls = getSLAPolicyClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultSLAPolicy newInstance = new DefaultSLAPolicy(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public SLAPolicy getSLAPolicy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<SLAPolicy> items = getAllSLAPolicyInstances();
            for (SLAPolicy item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof SLAPolicy) {
            return (SLAPolicy) res;
        } else if (res.hasProtegeType(getSLAPolicyClass())) {
            return new DefaultSLAPolicy(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<SLAPolicy> getAllSLAPolicyInstances() {
        return getAllSLAPolicyInstances(true);
    }

    public Collection<SLAPolicy> getAllSLAPolicyInstances(boolean transitive) {
        Collection<SLAPolicy> result = new ArrayList<SLAPolicy>();
        final RDFSNamedClass cls = getSLAPolicyClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultSLAPolicy(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getMEMIntensiveActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#MEMIntensiveActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public MEMIntensiveActivity createMEMIntensiveActivity(String name) {
        final RDFSNamedClass cls = getMEMIntensiveActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultMEMIntensiveActivity newInstance = new DefaultMEMIntensiveActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public MEMIntensiveActivity getMEMIntensiveActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<MEMIntensiveActivity> items = getAllMEMIntensiveActivityInstances();
            for (MEMIntensiveActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof MEMIntensiveActivity) {
            return (MEMIntensiveActivity) res;
        } else if (res.hasProtegeType(getMEMIntensiveActivityClass())) {
            return new DefaultMEMIntensiveActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<MEMIntensiveActivity> getAllMEMIntensiveActivityInstances() {
        return getAllMEMIntensiveActivityInstances(true);
    }

    public Collection<MEMIntensiveActivity> getAllMEMIntensiveActivityInstances(boolean transitive) {
        Collection<MEMIntensiveActivity> result = new ArrayList<MEMIntensiveActivity>();
        final RDFSNamedClass cls = getMEMIntensiveActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultMEMIntensiveActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getFacilityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#Facility";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public Facility createFacility(String name) {
        final RDFSNamedClass cls = getFacilityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultFacility newInstance = new DefaultFacility(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public Facility getFacility(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<Facility> items = getAllFacilityInstances();
            for (Facility item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof Facility) {
            return (Facility) res;
        } else if (res.hasProtegeType(getFacilityClass())) {
            return new DefaultFacility(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<Facility> getAllFacilityInstances() {
        return getAllFacilityInstances(true);
    }

    public Collection<Facility> getAllFacilityInstances(boolean transitive) {
        Collection<Facility> result = new ArrayList<Facility>();
        final RDFSNamedClass cls = getFacilityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultFacility(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getQoSPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#QoSPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public QoSPolicy createQoSPolicy(String name) {
        final RDFSNamedClass cls = getQoSPolicyClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultQoSPolicy newInstance = new DefaultQoSPolicy(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);

        return newInstance;


    }

    public QoSPolicy getQoSPolicy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<QoSPolicy> items = getAllQoSPolicyInstances();
            for (QoSPolicy item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof QoSPolicy) {
            return (QoSPolicy) res;
        } else if (res.hasProtegeType(getQoSPolicyClass())) {
            return new DefaultQoSPolicy(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<QoSPolicy> getAllQoSPolicyInstances() {
        return getAllQoSPolicyInstances(true);
    }

    public Collection<QoSPolicy> getAllQoSPolicyInstances(boolean transitive) {
        Collection<QoSPolicy> result = new ArrayList<QoSPolicy>();
        final RDFSNamedClass cls = getQoSPolicyClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultQoSPolicy(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getDeployActivityClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#DeployActivity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public DeployActivity createDeployActivity(String name) {
        final RDFSNamedClass cls = getDeployActivityClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultDeployActivity newInstance = new DefaultDeployActivity(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public DeployActivity getDeployActivity(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<DeployActivity> items = getAllDeployActivityInstances();
            for (DeployActivity item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof DeployActivity) {
            return (DeployActivity) res;
        } else if (res.hasProtegeType(getDeployActivityClass())) {
            return new DefaultDeployActivity(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<DeployActivity> getAllDeployActivityInstances() {
        return getAllDeployActivityInstances(true);
    }

    public Collection<DeployActivity> getAllDeployActivityInstances(boolean transitive) {
        Collection<DeployActivity> result = new ArrayList<DeployActivity>();
        final RDFSNamedClass cls = getDeployActivityClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultDeployActivity(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getEnvironmentPolicyClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#EnvironmentPolicy";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public EnvironmentPolicy createEnvironmentPolicy(String name) {
        final RDFSNamedClass cls = getEnvironmentPolicyClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultEnvironmentPolicy newInstance = new DefaultEnvironmentPolicy(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public EnvironmentPolicy getEnvironmentPolicy(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<EnvironmentPolicy> items = getAllEnvironmentPolicyInstances();
            for (EnvironmentPolicy item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof EnvironmentPolicy) {
            return (EnvironmentPolicy) res;
        } else if (res.hasProtegeType(getEnvironmentPolicyClass())) {
            return new DefaultEnvironmentPolicy(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<EnvironmentPolicy> getAllEnvironmentPolicyInstances() {
        return getAllEnvironmentPolicyInstances(true);
    }

    public Collection<EnvironmentPolicy> getAllEnvironmentPolicyInstances(boolean transitive) {
        Collection<EnvironmentPolicy> result = new ArrayList<EnvironmentPolicy>();
        final RDFSNamedClass cls = getEnvironmentPolicyClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultEnvironmentPolicy(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getITFacilityResourceAdaptationActionClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ITFacilityResourceAdaptationAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ITFacilityResourceAdaptationAction createITFacilityResourceAdaptationAction(String name) {
        final RDFSNamedClass cls = getITFacilityResourceAdaptationActionClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultITFacilityResourceAdaptationAction newInstance = new DefaultITFacilityResourceAdaptationAction(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ITFacilityResourceAdaptationAction getITFacilityResourceAdaptationAction(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ITFacilityResourceAdaptationAction> items = getAllITFacilityResourceAdaptationActionInstances();
            for (ITFacilityResourceAdaptationAction item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ITFacilityResourceAdaptationAction) {
            return (ITFacilityResourceAdaptationAction) res;
        } else if (res.hasProtegeType(getITFacilityResourceAdaptationActionClass())) {
            return new DefaultITFacilityResourceAdaptationAction(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ITFacilityResourceAdaptationAction> getAllITFacilityResourceAdaptationActionInstances() {
        return getAllITFacilityResourceAdaptationActionInstances(true);
    }

    public Collection<ITFacilityResourceAdaptationAction> getAllITFacilityResourceAdaptationActionInstances(boolean transitive) {
        Collection<ITFacilityResourceAdaptationAction> result = new ArrayList<ITFacilityResourceAdaptationAction>();
        final RDFSNamedClass cls = getITFacilityResourceAdaptationActionClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultITFacilityResourceAdaptationAction(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFProperty getToLowOrHighStateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#toLowOrHighState";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getStateAfterActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateAfterAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getStateBeforeActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#stateBeforeAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getEnergyStatesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#energyStates";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourcePropertyProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceProperty";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getCpuRequiredValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cpuRequiredValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getMemRequiredValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#memRequiredValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getHddAllocatedValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddAllocatedValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPerformanceEstimationProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#performanceEstimation";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getCpuAllocatedValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cpuAllocatedValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getMemAllocatedValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#memAllocatedValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPerformanceDegradationProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#performanceDegradation";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getHddRequiredValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hddRequiredValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPolicyNameProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policyName";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getEvaluationConditionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#evaluationCondition";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getIsRespectedProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#isRespected";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourceWorkloadPropertyProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceWorkloadProperty";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getRotationSpeedProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#rotationSpeed";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getClockRateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#clockRate";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getNoStatesJumpProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#noStatesJump";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getCacheRateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#cacheRate";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getCurrentWorkloadProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#currentWorkload";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getMaximumWorkloadProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#maximumWorkload";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getRecordedValueProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#recordedValue";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourceIDProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceID";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getCurrentEnergyStateProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#currentEnergyState";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getOptimalWorkloadProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#optimalWorkload";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourceOnProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceOn";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getHasAttachedProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#hasAttached";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPolicyActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policyAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getRunningActivitiesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#runningActivities";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resources";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getActivityPoliciesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activityPolicies";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getAssociatedActionsProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedActions";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPolicySubjectProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policySubject";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPolicyTargetProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#policyTarget";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getActivityProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#activity";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourceFromProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceFrom";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getResourceToProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#resourceTo";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getSimpleResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#simpleResources";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getBP_ActivityListProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#BP_ActivityList";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getAssociatedResourcesProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#associatedResources";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getInterractsWithProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#interractsWith";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getFacilityActionProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#facilityAction";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFProperty getPartOfProperty() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#partOf";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFProperty(name);
    }

    public RDFSNamedClass getApplicationActivityNetworkInfoClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivityNetworkInfo";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ApplicationActivityNetworkInfo createApplicationActivityNetworkInfo(String name) {
        final RDFSNamedClass cls = getApplicationActivityNetworkInfoClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultApplicationActivityNetworkInfo newInstance = new DefaultApplicationActivityNetworkInfo(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        return newInstance;

    }

    public ApplicationActivityNetworkInfo getApplicationActivityNetworkInfo(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {

            Collection<ApplicationActivityNetworkInfo> items = getAllApplicationActivityNetworkInfoInstances();
            for (ApplicationActivityNetworkInfo item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ApplicationActivityNetworkInfo) {
            return (ApplicationActivityNetworkInfo) res;
        } else if (res.hasProtegeType(getApplicationActivityNetworkInfoClass())) {
            return new DefaultApplicationActivityNetworkInfo(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ApplicationActivityNetworkInfo> getAllApplicationActivityNetworkInfoInstances() {
        return getAllApplicationActivityNetworkInfoInstances(true);
    }

    public Collection<ApplicationActivityNetworkInfo> getAllApplicationActivityNetworkInfoInstances(boolean transitive) {
        Collection<ApplicationActivityNetworkInfo> result = new ArrayList<ApplicationActivityNetworkInfo>();
        final RDFSNamedClass cls = getApplicationActivityNetworkInfoClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultApplicationActivityNetworkInfo(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public RDFSNamedClass getApplicationActivityDiskInfoClass() {
        final String uri = "http://www.semanticweb.org/ontologies/2010/6/ContextModel.owl#ApplicationActivityDiskInfo";
        final String name = owlModel.getResourceNameForURI(uri);
        return owlModel.getRDFSNamedClass(name);
    }

    public ApplicationActivityDiskInfo createApplicationActivityDiskInfo(String name) {
        final RDFSNamedClass cls = getApplicationActivityDiskInfoClass();
        if (name == null) {
            name = owlModel.getNextAnonymousResourceName();
        }
        Instance instance = cls.createInstance(name);
        DefaultApplicationActivityDiskInfo newInstance = new DefaultApplicationActivityDiskInfo(owlModel, instance.getFrameID());

        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);
        // newInstance.setRDFType(cls);
        // instance.setDirectType(cls);;
        //newInstance.addRDFType(cls);
        // instance.addDirectType(cls);
        // newInstance.setProtegeType(cls);
        //newInstance.addProtegeType(cls);
        // new// instance.setDirectType(cls);;
        //  newInstance.addDirectType(cls);
        //((KnowledgeBase) owlModel).addDirectType(instance, cls);
        //((KnowledgeBase) owlModel).setDirectType(instance, cls);

        return newInstance;

    }

    public ApplicationActivityDiskInfo getApplicationActivityDiskInfo(String name) {
        String fullName = OWLUtil.getInternalFullName(owlModel, name);
        RDFResource res = owlModel.getRDFIndividual(fullName);
        if (res == null) {
            Collection<ApplicationActivityDiskInfo> items = getAllApplicationActivityDiskInfoInstances();
            for (ApplicationActivityDiskInfo item : items) {
                if (item.getName().equals(fullName)) {
                    return item;
                }
            }
            return null;
        }
        if (res instanceof ApplicationActivityDiskInfo) {
            return (ApplicationActivityDiskInfo) res;
        } else if (res.hasProtegeType(getApplicationActivityDiskInfoClass())) {
            return new DefaultApplicationActivityDiskInfo(owlModel, res.getFrameID());
        }
        return null;
    }

    public Collection<ApplicationActivityDiskInfo> getAllApplicationActivityDiskInfoInstances() {
        return getAllApplicationActivityDiskInfoInstances(true);
    }

    public Collection<ApplicationActivityDiskInfo> getAllApplicationActivityDiskInfoInstances(boolean transitive) {
        Collection<ApplicationActivityDiskInfo> result = new ArrayList<ApplicationActivityDiskInfo>();
        final RDFSNamedClass cls = getApplicationActivityDiskInfoClass();
        RDFResource owlIndividual;
        for (Iterator it = cls.getInstances(transitive).iterator(); it.hasNext(); ) {
            owlIndividual = (RDFResource) it.next();
            result.add(new DefaultApplicationActivityDiskInfo(owlModel, owlIndividual.getFrameID()));
        }
        return result;
    }

    public String getIsServerUsefulProperty() {
        return getServerIsUsefullProperty().getName();
    }
}
