/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.propertyManager;

import ro.utcn.dsrl.utils.exceptions.ApplicationException;

import java.io.FileInputStream;
import java.util.Properties;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public final class PropertyFileManager implements IPropertyFileManager {

    private final static String energyAwareContextPropertiesFile = "./configurationFiles/energyAwareContext.properties";
    private final static String energyAwareContextModelLocationKey = "energyAwareContextModelLocation";
    private final static String energyAwareContextDatabaseLocationKey = "energyAwareContextDatabaseLocation";
    private final static String defaultOntologyFileLocation = "localhost";
    private final static String defaultOntologyDatabaseLocation = "./ontology/EnergyAwareContextDatabase";
    private Properties properties;
    private static PropertyFileManager fileManager;

    private PropertyFileManager() throws ApplicationException {
        try {
            properties = new Properties();
            properties.load(new FileInputStream(energyAwareContextPropertiesFile));
        } catch (Exception ex) {
            ApplicationException ae = new ApplicationException(ex.getMessage(), ex.getCause());
            ae.setStackTrace(ex.getStackTrace());
            throw ae;
        }
    }

    public static synchronized IPropertyFileManager getInstance() throws ApplicationException {
        IPropertyFileManager iFileManager = fileManager;
        if (iFileManager == null) {
            fileManager = new PropertyFileManager();
            iFileManager = fileManager;
        }
        return iFileManager;
    }

    public synchronized String getEnergyAwareContextDatabaseLocation() {
        return defaultOntologyDatabaseLocation;//getProperty(energyAwareContextDatabaseLocationKey);
    }

    public synchronized String getEnergyAwareContextModelLocation() {
        return defaultOntologyFileLocation;//getProperty(energyAwareContextModelLocationKey);
    }

    private String getProperty(String propertyKey) {
        return properties.getProperty(propertyKey);
    }


}
