/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAccessingAndProcessingAPI.utils.knowledgeBaseInterraction.impl;

import ro.utcn.dsrl.utils.similarContextIdentification.Cluster;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.misc.Pair;
import ro.utcn.dsrl.utils.similarContextIdentification.PowerCluster;

import java.io.*;
import java.util.*;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ContextSituationAccess {

    private static final File situationsFile = new File("./situations.xml");
    private static final File structureFile = new File("./structure.xml");
    private static Collection<ContextSituationDto> situations;
    private static Collection<Cluster> energyClusters;

    static {
        try {
            situations = refreshSituations();
            energyClusters = refreshPowerClusters();
        } catch (Exception e) {
            situations = new ArrayList<ContextSituationDto>();
            energyClusters = new ArrayList<Cluster>();
        }
    }

    private ContextSituationAccess() {
    }

    public static Collection<ContextSituationDto> refreshSituations() throws IOException, ClassNotFoundException {
        if (!situationsFile.exists()) {
            return new ArrayList<ContextSituationDto>();
        }
        ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream(situationsFile));
        Object content = inputStream.readObject();
        return (content != null && content instanceof Collection) ? (Collection<ContextSituationDto>) content : new ArrayList<ContextSituationDto>();
    }

    public static List<Cluster> refreshPowerClusters() throws IOException, ClassNotFoundException {
        if (!structureFile.exists()) {
            return new ArrayList<Cluster>();
        }
        HashMap<UUID, Cluster> clusters = new HashMap<UUID, Cluster>();
        HashMap<UUID, ContextSituationDto> situations = new HashMap<UUID, ContextSituationDto>();
        for (ContextSituationDto contextSituationDto : getSituations()) {
            situations.put(contextSituationDto.getId(), contextSituationDto);
        }
        ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream(structureFile));
        Object content = inputStream.readObject();
        if (content instanceof Collection) {

            for (Pair<UUID, UUID> p : ((Collection<Pair<UUID, UUID>>) content)) {
                if (!(clusters.containsKey(p.getFirst()))) {
                    PowerCluster powerCluster = new PowerCluster();
                    powerCluster.addToCluster(situations.get(p.getSecond()));
                    powerCluster.setId(p.getFirst());
                    clusters.put(p.getFirst(), powerCluster);
                } else {
                    clusters.get(p.getFirst()).addToCluster(situations.get(p.getSecond()));
                }
            }
        }
        if (content instanceof Pair) {
            Pair<UUID, UUID> p = (Pair<UUID, UUID>) content;
            if (!(clusters.containsKey(p.getFirst()))) {
                PowerCluster powerCluster = new PowerCluster();
                powerCluster.addToCluster(situations.get(p.getSecond()));
                powerCluster.setId(p.getFirst());
                clusters.put(p.getFirst(), powerCluster);
            } else {

                clusters.get(p.getFirst()).addToCluster(situations.get(p.getSecond()));
            }

        }
        List<Cluster> list = new ArrayList<Cluster>();
        for (Cluster c : clusters.values()) {
            list.add(c);
        }
        return list;
    }

    public static void savePowerClusters(Collection<Cluster> clusters) throws IOException {
        boolean hasThings = false;
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(new FileOutputStream(structureFile));
        Collection<Pair<UUID, UUID>> toWrite = new ArrayList<Pair<UUID, UUID>>();
        for (Cluster cluster : clusters) {
            PowerCluster ec = (PowerCluster) cluster;
            for (Object obj : ec.getAllElements()) {
                ContextSituationDto situation = (ContextSituationDto) obj;
                hasThings = true;
                toWrite.add(new Pair<UUID, UUID>(ec.getId(), situation.getId()));
            }
        }
        if (hasThings) {
            objectOutputStream.writeObject(toWrite);
            objectOutputStream.flush();
        }
        objectOutputStream.close();
    }

    public static void saveContextSituations(Collection<ContextSituationDto> commonDtos) throws IOException {
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(new FileOutputStream(situationsFile));
        objectOutputStream.writeObject(commonDtos);
        objectOutputStream.flush();
        objectOutputStream.close();
    }

    public static void saveContextSituation(ContextSituationDto situationDto) throws IOException {
        Collection<ContextSituationDto> situationcommonDtos = getSituations();
        for (ContextSituationDto s : situationcommonDtos) {
            if (s.equals(situationDto)) {
                return;
            }
        }
        situationcommonDtos.add(situationDto);
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(new FileOutputStream(situationsFile));
        objectOutputStream.writeObject(situationcommonDtos);
        objectOutputStream.flush();
        objectOutputStream.close();
    }


    public static Collection<ContextSituationDto> getSituations() {
        return situations;
    }

    public static void setSituations(Collection<ContextSituationDto> situations) {
        ContextSituationAccess.situations = situations;
    }

    public static Collection<Cluster> getEnergyClusters() {
        return energyClusters;
    }

    public static void setPowerClusters(Collection<Cluster> energyClusters) {
        ContextSituationAccess.energyClusters = energyClusters;
    }
}
