/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAnalysisModule.agents;

import jade.core.Agent;
import ro.utcn.dsrl.contextAnalysisModule.agents.behaviors.CAAContextAnalysysBehavior;
import ro.utcn.dsrl.contextAnalysisModule.agents.behaviors.CAAReceiveMessageBehavior;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.ContextEntropyEvaluator;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class ContextAnalysisAgent extends Agent implements Interruptable {
    private EACMInstance eacmInstance;
    private ContextEntropyEvaluator entropyEvaluator;
    private Thread currentlyRunningThread;
    private CAAContextAnalysysBehavior analysysBehavior;
    private ContextAnalysisAgent thisAgent;


    protected void setup() {
        Object[] args = getArguments();
        if (args != null && args.length > 1) {
            eacmInstance = (EACMInstance) args[0];
            entropyEvaluator = (ContextEntropyEvaluator) args[1];
        } else {
            GlobalLoopLogger.getLogger().info("CAA not enough parameters received.Terminating");
            this.takeDown();
        }
        addBehaviour(new CAAReceiveMessageBehavior(this));
        thisAgent = this;
        GlobalLoopLogger.getLogger().info("CAA started");
    }

    public void analyzeContext() {
        if(currentlyRunningThread != null && currentlyRunningThread.isAlive()){
            GlobalLoopLogger.getLogger().info("Refusing to double analysis");
            return;
        }
        currentlyRunningThread = new Thread("AnalysisBehaviorThread") {


            public void run() {
                analysysBehavior = new CAAContextAnalysysBehavior(thisAgent, eacmInstance, entropyEvaluator);
                analysysBehavior.action();
            }

        };
        currentlyRunningThread.start();

    }


    public void terminate() {
        if (currentlyRunningThread != null && currentlyRunningThread.isAlive()) {
            currentlyRunningThread.interrupt();
            if(analysysBehavior!= null){
                analysysBehavior.terminate();
            }
            try {
                currentlyRunningThread.join();
            } catch (InterruptedException e) {
                GlobalLoopLogger.error(e);
            }
        }


    }
}