/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAnalysisModule.agents.behaviors;

import jade.core.behaviours.OneShotBehaviour;
import java.rmi.RemoteException;
import ro.utcn.dsrl.actionPlanningModule.agents.ActionPlanningAgent;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.ContextEntropyEvaluator;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import ro.utcn.dsrl.utils.misc.Pair;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import ro.utcn.dsrl.actionEnforcementModule.agents.ActionPlanEnforcementAgent;
import ro.utcn.dsrl.actionPlanningModule.reasoning.impl.ReinforcementLearningPlanningEngine;
import ro.utcn.dsrl.actionPlanningModule.utils.actionExecutionSimulation.IActionSimulationUtil;
import ro.utcn.dsrl.actionPlanningModule.utils.actionExecutionSimulation.impl.DefaultActionSimulationUtil;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultDeployActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultMigrateActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.impl.DefaultSetServerStateActivity;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.impl.PowerConsumptionEstimator;
import ro.utcn.dsrl.contextMonitoringModule.utils.objectToDTOMappers.ContextSituationToDtoMapper;
import ro.utcn.dsrl.utils.actionEnforcement.ActionPlanDto;
import ro.utcn.dsrl.utils.commonDtos.ActionDto;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.commonInterfaces.PowerConsumption;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;
import ro.utcn.dsrl.utils.similarContextIdentification.SimilarContextSituationIdentification;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class CAAContextAnalysysBehavior extends OneShotBehaviour implements Interruptable {

    private EACMInstance eacmInstance;
    private Double entropy;
    private ContextEntropyEvaluator entropyEvaluator;
    private ContextAnalysisAgent agent;
    private Thread currentlyRunningThread;

    public CAAContextAnalysysBehavior(ContextAnalysisAgent agent, EACMInstance eacmInstance,
            ContextEntropyEvaluator entropyEvaluator) {
        super(agent);
        this.eacmInstance = eacmInstance;
        this.entropyEvaluator = entropyEvaluator;
        this.agent = agent;
    }

//    private boolean checkIfContextBroken(List<GPI_KPI_Policy> policies) {
//        for (GPI_KPI_Policy policy : policies) {
//            double percentage = 0;
//            if (policy.getPolicySubject().get(0) instanceof ServiceCenterServer) {
//                percentage = 100 * energyViolationDegree((ServiceCenterServer) policy.getPolicySubject().get(0)) / computeWorstCase(policy);
//            } else {
//                if (policy.getPolicySubject().get(0) instanceof ApplicationActivity) {
//                    percentage = 100 * taskViolationDegree((ApplicationActivity) policy.getPolicySubject().get(0)) / computeWorstCase(policy);
//                }
//            }
//
//            if (percentage > GeneralConfigurationManager.getEntropyThreshold()) {
//                return true;
//            }
//        }
//        return false;
//    }
//    private double taskViolationDegree(ApplicationActivity applicationActivity) {
//        double respectance =
//                applicationActivity.getCPUWeight() * (
//                        +applicationActivity.getCpuRequiredMaxValue()
//                                - applicationActivity.getCpuAllocatedValue())
//                        + applicationActivity.getMEMWeight()
//                        * (applicationActivity.getMemRequiredMaxValue() - applicationActivity.getMemAllocatedValue());
//        return respectance;
//    }
//
//    public double computeWorstCase(GPI_KPI_Policy policy) {
//        if (policy.getPolicySubject() instanceof ServiceCenterServer) {
//            double result = 0;
//            ServiceCenterServer server = (ServiceCenterServer) policy.getPolicySubject().get(0);
//            CPU cpu = server.getCpuResources().iterator().next();
//            double optimalCpuValue = cpu.getOptimalWorkLoad();
//            double totalCpuValue = cpu.getMaximumWorkLoad();
//            double minCpuValue = optimalCpuValue - GeneralConfigurationManager.getResourceLoadVariation() * totalCpuValue / 100;
//            double maxCpuValue = optimalCpuValue + GeneralConfigurationManager.getResourceLoadVariation() * totalCpuValue / 100;
//            if (minCpuValue > totalCpuValue - maxCpuValue) {
//                result += minCpuValue;
//            } else {
//                result += totalCpuValue - maxCpuValue;
//            }
//            MEM mem = server.getMemResources().iterator().next();
//            double optimalMemValue = mem.getOptimalWorkLoad();
//            double totalMemValue = mem.getMaximumWorkLoad();
//            double minMemValue = optimalMemValue - GeneralConfigurationManager.getResourceLoadVariation() * totalMemValue / 100;
//            double maxMemValue = optimalMemValue + GeneralConfigurationManager.getResourceLoadVariation() * totalMemValue / 100;
//
//            if (minMemValue > totalMemValue - maxMemValue) {
//                result += minMemValue;
//            } else {
//                result += totalMemValue - maxMemValue;
//            }
//            return result;
//        } else {
//            double result = 0;
//
//            ApplicationActivity applicationActivity = (ApplicationActivity) policy.getPolicySubject().get(0);
//            result = applicationActivity.getCpuRequiredMaxValue() + applicationActivity.getMemRequiredMaxValue();
//
//            return result;
//
//        }
//    }
//
//    private double energyViolationDegree(ServiceCenterServer server) {
//        double respectance = 0.0;
//        Collection<ServiceCenterITComputingResource> resources = server.getResources();
//        double diff = 0;
//        if (server.getIsActive()) {
//            for (CPU cpu : server.getCpuResources()) {
//                double optimalCpuValue = cpu.getOptimalWorkLoad();
//                double totalCpuValue = cpu.getMaximumWorkLoad();
//                double cpuMinAcceptableValue = optimalCpuValue - GeneralConfigurationManager.getResourceLoadVariation() * totalCpuValue / 100;
//                double cpuMaxAcceptableValue = optimalCpuValue + GeneralConfigurationManager.getResourceLoadVariation() * totalCpuValue / 100;
//                double usedCPU = cpu.getCurrentWorkLoad();
//                if (usedCPU < cpuMinAcceptableValue) {
//                    diff += cpuMinAcceptableValue - usedCPU;
//                }
//                if (usedCPU > cpuMaxAcceptableValue) {
//                    diff += usedCPU - cpuMaxAcceptableValue;
//                }
//                if (server.hasCPUWeight()) {
//                    respectance += server.getCPUWeight() * diff;
//                } else {
//                    respectance += diff;
//                }
//            }
//            diff = 0.0;
//            double usedMemory = 0.0;
//            double total = 0.0;
//            double optimal = 0.0;
//
//            Collection<MEM> memories = server.getMemResources();
//            for (MEM mem : memories) {
//                ;
//                double optimalMemValue = mem.getOptimalWorkLoad();
//                double totalMemValue = mem.getMaximumWorkLoad();
//                double memoryMinAcceptableValue = optimalMemValue - GeneralConfigurationManager.getResourceLoadVariation() * totalMemValue / 100;
//                double memoryMaxAcceptableValue = optimalMemValue + GeneralConfigurationManager.getResourceLoadVariation() * totalMemValue / 100;
//
//                if (usedMemory > memoryMaxAcceptableValue) {
//                    diff = usedMemory - memoryMaxAcceptableValue;
//                } else if (usedMemory < memoryMinAcceptableValue) {
//                    diff = memoryMinAcceptableValue - usedMemory;
//                }
//                if (server.hasMEMWeight()) {
//                    respectance += server.getMEMWeight() * diff;
//                } else {
//                    respectance += diff;
//                }
//            }
//            diff = 0.0;
//
//            Double usedStorage = 0.0;
//            Double storageMaxAcceptableValue = 0.0;
//            double storageMinAcceptableValue = 0.0;
//
//            for (HDD hdd : server.getHddResources()) {
//                total = hdd.getMaximumWorkLoad();
//                optimal = hdd.getOptimalWorkLoad();
//                usedStorage = hdd.getCurrentWorkLoad();
//                storageMaxAcceptableValue = optimal - GeneralConfigurationManager.getResourceLoadVariation() * total / 100;
//                storageMinAcceptableValue = optimal - GeneralConfigurationManager.getResourceLoadVariation() * total / 100;
//                if (usedStorage > storageMaxAcceptableValue) {
//                    diff = usedStorage - storageMaxAcceptableValue;
//                } else if (usedStorage < storageMinAcceptableValue) {
//                    diff = storageMinAcceptableValue - usedStorage;
//                }
//                if (server.hasHDDWeight()) {
//                    respectance += server.getHDDWeight() * diff;
//                } else {
//                    respectance += diff;
//                }
//            }
//        }
//        return respectance; 
//    }
    public void action() {
        currentlyRunningThread = new Thread("CAAContextAnalysisBehaviorThread") {

            public void run() {
                GlobalLoopLogger.getLogger().info("Analysing context");
                Collection<GPI_KPI_Policy> policies = new ArrayList<GPI_KPI_Policy>();
                policies.addAll(eacmInstance.getAllQoSPolicyInstances());
                policies.addAll(eacmInstance.getAllITComputingContextPolicyInstances());

                Pair<Double, List<GPI_KPI_Policy>> contextAnalysisResult = entropyEvaluator.computeEntropy(policies);

                if (contextAnalysisResult.getFirst() > 0) { // if (checkIfContextBroken(contextAnalysisResult.getSecond())) {
                    if (GeneralConfigurationManager.getMemoryState().equalsIgnoreCase("enabled")){
                    Queue<ContextAction> actionPlanActions = findSimilarActions(eacmInstance, new LinkedList(), policies);
                    if (actionPlanActions!=null && actionPlanActions.size() > 0) {
                        int toBeDeployed = 0;
                         GlobalLoopLogger.getLogger().debug("Entropy " + contextAnalysisResult.getFirst());
                         GlobalLoopLogger.getLogger().debug("Found actions, now sending enforce message");
                        for (ApplicationActivity applicationActivity : eacmInstance.getAllApplicationActivityInstances()) {
                            if (!applicationActivity.isRunning()) {
                                toBeDeployed++;
                            }
                        }
                        List<String> generatedActionsNames = new ArrayList<String>(
                                actionPlanActions.size());
                        for (ContextAction contextAction : actionPlanActions) {
                            generatedActionsNames.add(contextAction.getName());
                        }

                        ActionPlanDto actionPlanDto = new ActionPlanDto(generatedActionsNames);

                        GlobalLoopLogger.getLogger().info("");
                        InterAgentMessageBuilder.sendMessage(agent, ActionPlanEnforcementAgent.class,
                                AgentMessageType.START_ACTION_PLAN_EXECUTION, actionPlanDto);
                    }else{
                        GlobalLoopLogger.getLogger().debug("Entropy " + contextAnalysisResult.getFirst());
                        InterAgentMessageBuilder.sendMessage(agent, ActionPlanningAgent.class, AgentMessageType.START_ACTION_PLAN_GENERATION);
                    }
                    
                    } else {
                        GlobalLoopLogger.getLogger().debug("Entropy " + contextAnalysisResult.getFirst());
                        InterAgentMessageBuilder.sendMessage(agent, ActionPlanningAgent.class, AgentMessageType.START_ACTION_PLAN_GENERATION);
                    }
                } else {
                    GlobalLoopLogger.getLogger().info("System in accepted state");
                    if (GeneralConfigurationManager.usesWorkloadGenerator()) {

                        InterAgentMessageBuilder.sendMessage(agent, ro.utcn.dsrl.workloadScheduler.WorkloadSchedulerAgent.class, AgentMessageType.ACTION_PLAN_EXECUTION_COMPLETED);
                    }
                    InterAgentMessageBuilder.sendMessage(agent, ContextMonitoringAgent.class, AgentMessageType.ACTION_PLAN_EXECUTION_COMPLETED);
                }
            }
        };
        currentlyRunningThread.start();


    }

    public void terminate() {
        if (currentlyRunningThread != null && currentlyRunningThread.isAlive()) {
//            currentlyRunningThread.interrupt();
            currentlyRunningThread.stop();
            try {
                currentlyRunningThread.join();
            } catch (InterruptedException e) {
                GlobalLoopLogger.error(e);
            }
        }
    }

    public Queue<ContextAction> findSimilarActions(final EACMInstance eacmInstance, List<ContextAction> currentContextActions, final Collection<GPI_KPI_Policy> policies) {
        ContextSituationDto contextSituationDto = ContextSituationToDtoMapper.map(eacmInstance);
        SimilarContextSituationIdentification similarContextSituationIdentification = SimilarContextSituationIdentification.getIdentification();
        List<ContextAction> actionsList = new ArrayList<ContextAction>();

        similarContextSituationIdentification.refresh();


        PowerConsumption powerConsumption = null;
        try {
            powerConsumption = new PowerConsumptionEstimator(eacmInstance);
        } catch (RemoteException e) {
            GlobalLoopLogger.error(e);
        }
        Double powerConsumptionWithoutAlgorithm = null;
        try {
            powerConsumptionWithoutAlgorithm = powerConsumption.getValueWithRunningAlgorithm().doubleValue();
        } catch (ApplicationException e) {
            GlobalLoopLogger.error(e);
        } catch (RemoteException e) {
            GlobalLoopLogger.error(e);
        }
        List<Pair<ContextSituationDto, Double>> orderedPairs =
                similarContextSituationIdentification.getOrderedListOfContextFromTheNearestEnergyCluster(
                contextSituationDto, powerConsumptionWithoutAlgorithm);
        double similarityThreshold = ContextSituationDto.MAXIMUM_DISTANCE;
        int currentIndex = 0;
        int size = orderedPairs.size();
        if (size > 0) {
            while (currentIndex < size && orderedPairs.get(currentIndex).getSecond() != ContextSituationDto.MAXIMUM_DISTANCE && orderedPairs.get(currentIndex).getSecond() <= similarityThreshold) {
                ContextSituationDto situationDto = orderedPairs.get(currentIndex).getFirst();
                currentIndex++;
                List<ContextAction> actions = new ArrayList<ContextAction>();
                if (contextSituationDto.equals(situationDto)) {
                    List<ActionDto> actioncommonDtos = situationDto.getActions();
                    Map<String, String> serversMatch = similarContextSituationIdentification.matchServers(situationDto, contextSituationDto.getServers());
                    Map<String, String> tasksMatch = similarContextSituationIdentification.matchTasks(situationDto, contextSituationDto.getTasks());
                    for (ActionDto actionDto : actioncommonDtos) {
                        String className = actionDto.getActionClassName();
                        List<String> resources = actionDto.getContextResources();
                        ContextAction action = null;
                        if (className.equals(DefaultDeployActivity.class.getName())) {
                            ApplicationActivity activity = null;
                            ServiceCenterITComputingResource server = null;
                            for (String string : resources) {

                                if (tasksMatch.containsKey(string)) {
                                    activity = eacmInstance.getApplicationActivity(tasksMatch.get(string));
                                } else if (serversMatch.containsKey(string)) {
                                    server = eacmInstance.getServiceCenterITComputingResource(serversMatch.get(string));
                                }
                            }
                            action = eacmInstance.createDeployActivity("Deploy_"
                                    + activity.getName() + "_to_" + server.getName());
                            action.addResource(server);
                            ((DefaultDeployActivity) action).setActivity(activity);
                            actions.add(action);

                        } else if (className.equals(DefaultMigrateActivity.class.getName())) {
                            ApplicationActivity activity = null;
                            ServiceCenterITComputingResource server1 = null;
                            ServiceCenterITComputingResource server2 = null;
                            for (String string : resources) {

                                if (tasksMatch.containsKey(string)) {
                                    activity = eacmInstance.getApplicationActivity(tasksMatch.get(string));
                                } else if (serversMatch.containsKey(string)) {
                                    if (server1 == null) {
                                        server1 = eacmInstance.getServiceCenterITComputingResource(serversMatch.get(string));
                                    } else {
                                        server2 = eacmInstance.getServiceCenterITComputingResource(serversMatch.get(string));
                                    }
                                }
                            }
                            action = eacmInstance.createMigrateActivity("Migrate_from_"
                                    + server1.getName()
                                    + "_to_" + server2.getName() + "_Activity");

                            action.addResource(server1);
                            action.addResource(server2);
                            ((DefaultMigrateActivity) action).setActivity(activity);
                            actions.add(action);

                        } else if (className.equals(DefaultSetServerStateActivity.class.getName())) {
                            ApplicationActivity activity = null;
                            ServiceCenterITComputingResource server = null;
                            for (String s : resources) {
                                String string = s.split("\n")[0];
                                if (tasksMatch.containsKey(string)) {
                                    activity = eacmInstance.getApplicationActivity(tasksMatch.get(string));
                                } else if (serversMatch.containsKey(string)) {
                                    server = eacmInstance.getServiceCenterITComputingResource(serversMatch.get(string));
                                }
                            }


                            if (!((ServiceCenterServer) server).getIsActive()) {

                                action = eacmInstance.createSetServerStateActivity("Set_state_for_" + server.getName()
                                        + "_to_" + 1);
                                ((SetServerStateActivity) action).setTargetServerState(1);
                            } else {

                                action = eacmInstance.createSetServerStateActivity("Set_state_for_" + server.getName()
                                        + "_to_" + 0);
                                ((SetServerStateActivity) action).setTargetServerState(0);
                            }
                            action.addResource(server);
                            actions.add(action);

                            if (action != null) {

                                actionsList.add(action);
                            }
                        }
                    }
                    Queue<ContextAction> cs = new LinkedList(currentContextActions);
                    IActionSimulationUtil simulationUtil = null;
                    if (GeneralConfigurationManager.getPlanningEngine().equals("ffd")) {
                        simulationUtil = new DefaultActionSimulationUtil(new ReinforcementLearningPlanningEngine(entropyEvaluator, eacmInstance));
                        for (ContextAction contextAction : actions) {
                            cs.add(contextAction);
                            simulationUtil.simulateActionExecution(eacmInstance, contextAction);
                        }
                    } else if (GeneralConfigurationManager.getPlanningEngine().equals("rl")) {
                        simulationUtil = new DefaultActionSimulationUtil(new ReinforcementLearningPlanningEngine(entropyEvaluator, eacmInstance));
                        for (ContextAction contextAction : actions) {
                            cs.add(contextAction);
                            simulationUtil.simulateActionExecution(eacmInstance, contextAction);
                        }

                    }


                    Double afterExecuteEntropy = entropyEvaluator.computeEntropy(policies).getFirst();
                    Object[] commands = cs.toArray();

                    for (int i = commands.length - 1; i >= 0; i--) {
                        ContextAction command = (ContextAction) commands[i];
                        command.undo(eacmInstance);
                    }

                    //TODO: de inlocuit cu thresholdu sau verificarea daca ii ok contextu
                    if (afterExecuteEntropy == 0) {
//                            if (cs.getActions().size() == actioncommonDtos.size()) {
//                                remembered = true;
//                            }
                        return cs;
                    }

                }
            }
        }

        return null;
    }
}
