/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextAnalysisModule.agents.behaviors;

import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import ro.utcn.dsrl.mainModule.agents.MainAgent;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class CAAReceiveMessageBehavior extends CyclicBehaviour {

    private ContextAnalysisAgent agent;
//	private EACMInstance eacmInstance;
//	private ContextModelAnalyser contextModelAnalyser;

    public CAAReceiveMessageBehavior(ContextAnalysisAgent agent)//, EACMInstance eacmInstance,
//			ContextModelAnalyser analyser) 
    {
        super(agent);
        this.agent = agent;
//		this.eacmInstance = eacmInstance;
//		this.contextModelAnalyser = analyser;
    }


    public void action() {
        ACLMessage message = agent.blockingReceive();
        if (message == null) {
            return;
        }

        Object[] content = null;
        try {
            content = (Object[]) message.getContentObject();
        } catch (UnreadableException e) {
            GlobalLoopLogger.error(e);
        }

        AgentMessageType command = (AgentMessageType) content[0];

        switch (command) {
            case START_CONTEXT_ANALYSIS:
                System.out.println("Starting analysis behavior");
                agent.analyzeContext();
                break;
            case TERMINATE_EXECUTION:
                agent.terminate();
                InterAgentMessageBuilder.sendMessage(agent, MainAgent.class, AgentMessageType.EXECUTION_TERMINATED);
                InterAgentMessageBuilder.sendMessage(agent, ContextMonitoringAgent.class, AgentMessageType.EXECUTION_TERMINATED);
                break;
        }

    }
}