/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents;

import jade.core.Agent;
import jade.wrapper.AgentController;
import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.mainModule.agents.MainAgent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.contextMonitoringModule.agents.behaviors.CMAReceiveMessageBehavior;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import java.util.Collection;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.utils.knowledgeBaseInterraction.impl.ContextSituationAccess;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.similarContextIdentification.SimilarContextSituationIdentification;
/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ContextMonitoringAgent extends Agent implements Interruptable {

    private AgentController contextAnalysisAgentContainer;

    private EACMInstance localEacmInstance;

    private CAPInteractions CAPInteractions;
    private ServiceCenterManagement serviceCenterManagement;

    private PlanningEngine planningEngine;

    private Thread contextUpdatingThread;
    
    //TODO: update from OpenNebula
        public  void loadMemory() {
        Collection<ContextSituationDto> situations = ContextSituationAccess.getSituations();

        SimilarContextSituationIdentification similarContextSituationIdentification = SimilarContextSituationIdentification.getIdentification();

        similarContextSituationIdentification.addSituationsToEnergyClusters(situations);
        similarContextSituationIdentification.initializeEnergyClusters(similarContextSituationIdentification.getEnergyClusters(), situations);
    }
    protected void setup() {
        Object[] args = getArguments();
        if (args != null && args.length > 4) {
            localEacmInstance = (EACMInstance) args[0];
            serviceCenterManagement = (ServiceCenterManagement) args[2];
            CAPInteractions = (CAPInteractions) args[3];
            planningEngine = (PlanningEngine) args[4];
        } else {
            GlobalLoopLogger.getLogger().info("CMA not enough parameters received.Terminating");
            this.takeDown();
        }
        addBehaviour(new CMAReceiveMessageBehavior(this, localEacmInstance, 
                 serviceCenterManagement, CAPInteractions, planningEngine));
        GlobalLoopLogger.getLogger().info("CMA started");
        loadMemory();
        GlobalLoopLogger.getLogger().info("Memory loaded");

    }

    public void updateContextModel() {
        contextUpdatingThread = new Thread() {


            public void run() {
            }

        };
        contextUpdatingThread.start();
    }

    public void terminate() {
        if (contextUpdatingThread != null && contextUpdatingThread.isAlive()) {
            contextUpdatingThread.interrupt();
            try {
                contextUpdatingThread.join();
            } catch (InterruptedException e) {
                GlobalLoopLogger.error(e);
            }
        }
        InterAgentMessageBuilder.sendMessage(this, MainAgent.class, AgentMessageType.EXECUTION_TERMINATED);

    }

}
