/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.commonDtos.ActivityDiskDto;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ApplicationDto;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class CMAAddActivitiesBehavior extends OneShotBehaviour {

    private EACMInstance eacmInstance;
    private List<ApplicationDto> applicationsToBeAdded;

    public CMAAddActivitiesBehavior(Agent a, EACMInstance eacmInstance, List<ApplicationDto> applicationsToBeAdded) {
        super(a);
        this.eacmInstance = eacmInstance;
        this.applicationsToBeAdded = applicationsToBeAdded;
    }

    public void action() {

        for (ApplicationDto applicationDto : applicationsToBeAdded) {
            Application application = eacmInstance.createApplication(applicationDto.getApplicationName());
            application.setEPKBId(applicationDto.getEpkbID());

            for (ActivityDto activityDto : applicationDto.getActivities()) {
                if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
                    if (eacmInstance.getApplicationActivity(activityDto.getTaskName() + "_" + activityDto.getIp()) != null) {
                        continue;
                    }
                }
                ApplicationActivity applicationActivity = eacmInstance.createApplicationActivity(activityDto.getTaskName() + "_" + activityDto.getIp());

                activityDto.setTaskName(activityDto.getTaskName() + "_" + activityDto.getIp());

                applicationActivity.setId(activityDto.getCloudID());
                applicationActivity.setEPKBId(activityDto.getEpkbID());

                applicationActivity.setCpuAllocatedValue(0);
                applicationActivity.setCpuRequiredMaxValue(activityDto.getRequestedCPUMax());
                applicationActivity.setCpuRequiredMinValue(activityDto.getRequestedCPUMin());
                applicationActivity.setMemAllocatedValue(0);
                applicationActivity.setMemRequiredMaxValue(activityDto.getRequestedMemoryMax());
                applicationActivity.setMemRequiredMinValue(activityDto.getRequestedMemoryMin());
                applicationActivity.setHddAllocatedValue(0);
                applicationActivity.setHddRequiredMaxValue(activityDto.getRequestedStorageMax());
                applicationActivity.setHddRequiredMinValue(activityDto.getRequestedStorageMin());
                applicationActivity.setNumberOfCoresAllocatedValue(0);
                applicationActivity.setNumberOfCoresRequiredValue(activityDto.getRequestedCores());

                applicationActivity.setCpuWeight(activityDto.getCpuWeight());
                applicationActivity.setMemWeight(activityDto.getMemWeight());
                applicationActivity.setHddWeight(activityDto.getHddWeight());

                applicationActivity.setResourceID(applicationActivity.getFrameID().getName());
                ApplicationActivityNetworkInfo networkInfo = eacmInstance.getApplicationActivityNetworkInfo(applicationActivity.getName() + "_network_info");
                if (networkInfo == null) {
                    networkInfo = eacmInstance.createApplicationActivityNetworkInfo(applicationActivity.getName() + "_network_info");
                }
                ApplicationActivityDiskInfo diskInfo = eacmInstance.getApplicationActivityDiskInfo(applicationActivity.getName() + "_disk_info");
                if (diskInfo == null) {
                    diskInfo = eacmInstance.createApplicationActivityDiskInfo(applicationActivity.getName() + "_disk_info");
                }


                networkInfo.setIpAddress(activityDto.getIp());
                networkInfo.setVncPort(activityDto.getVncPort());
                applicationActivity.setNetworkInfo(networkInfo);

                ActivityDiskDto taskDiskDto = activityDto.getDiskDtoCollection().iterator().next();

                diskInfo.setDiskType(taskDiskDto.getType());
                diskInfo.setDiskFormat(taskDiskDto.getFormat());
                diskInfo.setDiskSize(taskDiskDto.getSize());
                diskInfo.setPhysicalPath(taskDiskDto.getPath());
                applicationActivity.addDiskInfo(diskInfo);

                QoSPolicy policy = eacmInstance.createQoSPolicy(activityDto.getTaskName() + "_QoSPolicy");
                List<ContextResource> subjects = new ArrayList<ContextResource>(1);
                subjects.add(applicationActivity);
                policy.setPolicySubject(subjects);
                policy.setPolicyTarget(subjects);
                policy.setPolicyName(policy.getName());
                policy.setPolicyWeight(1f);

                applicationActivity.addActivityPolicies(policy);
                application.addActivity(applicationActivity);

                String associatedServerIP = activityDto.getAssociatedServerIP();

                if (associatedServerIP != null && associatedServerIP.length() > 0) {
                    ServiceCenterServer serviceCenterServer = eacmInstance.getServiceCenterServer(activityDto.getAssociatedServerIP());
                    serviceCenterServer.addRunningActivity(applicationActivity);
                }

            }
        }
        if (applicationsToBeAdded.size() > 0) {
            Collection<ServiceCenterServer> servers = eacmInstance.getAllServiceCenterServerInstances();
            for (ServiceCenterServer server : servers) {
                server.resetInitialValues();
            }
            //notification done from ContextMonitoringAgent CMAReceiveMEssageBevahior. Avoids
            //multiple notifications for example when a deploy and delete happen in same step
//           //notify the ContextAnalysisAgent that the context has changed and needs to be reanalyzed
//            if ((GeneralConfigurationManager.getPlanningEngine().equals("ffd") || GeneralConfigurationManager.getPlanningEngine().equals("rl")))
//                InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS, this.getClass().getName());
        }

    }
}
