/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import java.util.ArrayList;
import java.util.List;

import jade.core.Agent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.HDD;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITComputingContextPolicy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;
import ro.utcn.dsrl.utils.commonDtos.PhysicalHost;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import jade.core.behaviours.OneShotBehaviour;

public class CMAAddServersBehavior extends OneShotBehaviour {

    private EACMInstance eacmInstance;
    private List<ExtendedServerDto> serversToBeAdded;
    private ServiceCenterManagement serviceCenterManagement;


    public CMAAddServersBehavior(Agent agent, EACMInstance eacmInstance,
                                 List<ExtendedServerDto> serversToBeAdded,
                                 ServiceCenterManagement serviceCenterManagement) {
        super(agent);
        this.eacmInstance = eacmInstance;
        this.serversToBeAdded = serversToBeAdded;
        this.serviceCenterManagement = serviceCenterManagement;
    }


    public void action() {

        ArrayList<Integer> energyStates = new ArrayList<Integer>();
        energyStates.add(0);
        energyStates.add(1);

        for (ExtendedServerDto dto : serversToBeAdded) {

            String serverName = dto.getServerName();
            ServiceCenterServer server = eacmInstance.createServiceCenterServer(serverName);

            server.setEPKBId(dto.getEpkbID());

            server.setIpAddress(dto.getIpAddress());
            server.setMacAddress(dto.getMacAddress());
            server.setIsActive(false);
            server.setResourceID(server.getName());
            server.setEnergyStates(energyStates);
            server.setCurrentEnergyState(0);
            server.addResourceWorkloadProperty("Workload_Property");

            CPU cpu = eacmInstance.createCPU(serverName + "_CPU");

            cpu.setResourceID(cpu.getName());
            cpu.setCurrentEnergyState(0);

            server.setCPUWeight(0.5f);

            int coreCount = dto.getCoreNo();
            int maximumCPU = dto.getMaximumCPU();
            int optimumCPU = dto.getOptimalCPU();

            cpu.setCpuCoreCount(coreCount);
            cpu.setEnergyStates(energyStates);
            cpu.setMaximumWorkLoad((double) maximumCPU * coreCount);
            cpu.setOptimalWorkLoad((double) optimumCPU * coreCount);

            List<CPU> cpus = new ArrayList<CPU>();
            cpus.add(cpu);
            server.setCpuResources(cpus);

            MEM memory = eacmInstance.createMEM(serverName + "_Memory");

            memory.setResourceID(memory.getName());
            memory.setMaximumWorkLoad((double) dto.getMaximumMemory());
            memory.setOptimalWorkLoad((double) dto.getOptimalMemory());
            memory.setEnergyStates(energyStates);
            memory.setCurrentEnergyState(0);

            server.setMEMWeight(0.25f);

            List<MEM> mems = new ArrayList<MEM>();
            mems.add(memory);
            server.setMemResources(mems);

            HDD storage = eacmInstance.createHDD(serverName + "_Storage");

            storage.setResourceID(storage.getName());

            ITComputingContextPolicy policy =
                    eacmInstance.createITComputingContextPolicy(server.getLocalName() + "_EnergyPolicy");

            policy.addPolicySubject(server);
            policy.addPolicyTarget(server);
            policy.setPolicyWeight(0.3f);
            policy.setPolicyName(policy.getName());
            policy.setEvaluationCondition("Evaluation_condition");

            cpu.addPartOf(server);
            memory.addPartOf(server);
            storage.addPartOf(server);
            server.markInitialValues();


            PhysicalHost host = new PhysicalHost();
            host.setHostname(serverName);
            host.setIm(PhysicalHost.IM_KVM);
            host.setTm(PhysicalHost.TM_SSH);
            host.setVmm(PhysicalHost.VMM_KVM);
            host.setId(server.getId());
            try {
                host = serviceCenterManagement.addHost(host);
            } catch (ServiceCenterAccessException e) {
                GlobalLoopLogger.error(e);
            }
            server.setId(host.getId());

        }
        //notification done from ContextMonitoringAgent CMAReceiveMEssageBevahior. Avoids
            //multiple notifications for example when a deploy and delete happen in same step
//        if(serversToBeAdded.size()>0){
//            //notify the ContextAnalysisAgent that the context has changed and needs to be reanalyzed
//            InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS,this.getClass().getName());
//        }
        
    }

}
