/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import jade.core.behaviours.OneShotBehaviour;
import java.util.logging.Level;
import java.util.logging.Logger;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.commonDtos.ApplicationDto;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class CMAEACMConceptsManagementBehavior extends OneShotBehaviour implements Interruptable {
    private ContextMonitoringAgent agent;
    private EACMInstance eacmInstance;
    private ServiceCenterManagement serviceCenterManagement;
    private CAPInteractions CAPInteractions;
    private Object[] messageBody;

    private Interruptable runningBehavior;
    private Thread currentlyRunningThread;

    public CMAEACMConceptsManagementBehavior(ContextMonitoringAgent agent, EACMInstance eacmInstance,
                                        ServiceCenterManagement serviceCenterManagement,
                                        CAPInteractions CAPInteractions, Object[] messageBody) {
        super(agent);
        this.eacmInstance = eacmInstance;
        this.serviceCenterManagement = serviceCenterManagement;
        this.CAPInteractions = CAPInteractions;
        this.messageBody = messageBody;
        this.agent = agent;
    }

    @Override
    public void action() {
        currentlyRunningThread = new Thread() {

            @Override
            public void run() {
                AgentMessageType agentMessageType = (AgentMessageType) messageBody[0];
                switch (agentMessageType) {

                    case ACTIVITIES_ADDED:

                        //add the new activities
                        new CMAAddActivitiesBehavior(agent,eacmInstance, (List<ApplicationDto>) messageBody[1]).action();
                        break;

                    case ACTIVITIES_REMOVED:
                        //remove the activities
                        new CMARemoveActivitiesBehavior(agent,eacmInstance, (List<ActivityDto>) messageBody[1], serviceCenterManagement, CAPInteractions).action();

                        break;

                    case SERVERS_ADDED:
                        //add new servers
                        new CMAAddServersBehavior(agent,eacmInstance, (List<ExtendedServerDto>) messageBody[1], serviceCenterManagement).action();
                        break;

                    case SERVERS_REMOVED:

                        //remove new servers
                        new CMARemoveServersBehavior(agent,eacmInstance, (List<ExtendedServerDto>) messageBody[1], serviceCenterManagement).action();
                        break;

                    case POLICIES_ADDED:
                        //TODO: de implementat behaviors?
                        //add new policies
                        new CMAAddPoliciesBehavior();
                        break;

                    case POLICIES_REMOVED:
                        //TODO: de implementat behaviors?
                        break;

//                    case SYNHCRONYZE_WORKLOAD_WITH_EACM:
//                          new  CMAAddAlreadyRunningActivitiesToServers(agent,eacmInstance, (List<ApplicationDto>) messageBody[1]).action();
//                                    break;

                }
            }
        };
      currentlyRunningThread.start();
        try {
            currentlyRunningThread.join();
        } catch (InterruptedException ex) {
            Logger.getLogger(CMAEACMConceptsManagementBehavior.class.getName()).log(Level.SEVERE, null, ex);
        }
}

    public void terminate() {
        if (runningBehavior != null) {
            runningBehavior.terminate();
            if (currentlyRunningThread != null && currentlyRunningThread.isAlive()) {
                currentlyRunningThread.interrupt();
                try {
                    currentlyRunningThread.join();
                } catch (InterruptedException e) {
                    GlobalLoopLogger.error(e);
                }
            }
        }
    }

}
