/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import com.enterprisedt.util.debug.Logger;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.mainModule.agents.MainAgent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.actionPlanningModule.agents.ActionPlanningAgent;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.ArrayList;
import java.util.List;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class CMAReceiveMessageBehavior extends CyclicBehaviour {

    private ContextMonitoringAgent agent;
    private EACMInstance eacmInstance;
    private ServiceCenterManagement serviceCenterManagement;
    private CAPInteractions CAPInteractions;
    private PlanningEngine planningEngine;
    private boolean inMape = false;
    private boolean analysisHasTerminated;
    private boolean planningHasTerminated;
    private List<ACLMessage> pendingMessages;
    private ACLMessage pendingEACMUpdateMessage;
    private int sleepTimeToEnableSuccessfulTerminationOfAgents = 30;//000;

    public CMAReceiveMessageBehavior(ContextMonitoringAgent agent,
            EACMInstance eacmInstance, //EACMInstance remoteEACMInstance,
            ServiceCenterManagement serviceCenterManagement,
            CAPInteractions CAPInteractions, PlanningEngine planningEngine) {
        super(agent);
        this.agent = agent;
        this.eacmInstance = eacmInstance;
        this.serviceCenterManagement = serviceCenterManagement;
        this.CAPInteractions = CAPInteractions;
        this.planningEngine = planningEngine;
        pendingMessages = new ArrayList<ACLMessage>();
    }

    public void action() {

        //pauses current thread until message available
        ACLMessage message = agent.blockingReceive();

        if (message == null) {
            return;
        }

        //retrieve the content of the message
        Object[] content = null;
        try {
            content = (Object[]) message.getContentObject();
        } catch (UnreadableException e) {
            GlobalLoopLogger.error(e);
            e.printStackTrace();
        }

        //the first element of the message is the message type represented by an AgentMessageType instance
        AgentMessageType command = (AgentMessageType) content[0];

        switch (command) {

            case UPDATE_EACM_CONCEPTS:

                if (inMape) {
                    pendingMessages.add(message);
                } else {
                    //stop the MAPE loop
                    InterAgentMessageBuilder.sendMessage(agent, ContextAnalysisAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                    InterAgentMessageBuilder.sendMessage(agent, ActionPlanningAgent.class, AgentMessageType.TERMINATE_EXECUTION);

                    pendingEACMUpdateMessage = message;


                    analysisHasTerminated = false;
                    planningHasTerminated = false;
//                    CMAEACMConceptsManagementBehavior behavior = new CMAEACMConceptsManagementBehavior(agent, eacmInstance,
//                            serviceCenterManagement, CAPInteractions,
//                            new Object[]{content[1], content[2]});
//                    behavior.action();
//                    behavior.terminate();
                    inMape = true;
                }
                break;
            case UPDATE_CONTEXT_INFORMATION:

                if (inMape) {
                    pendingMessages.add(message);
                } else {
                    //stop the MAPE loop
                    InterAgentMessageBuilder.sendMessage(agent, ContextAnalysisAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                    InterAgentMessageBuilder.sendMessage(agent, ActionPlanningAgent.class, AgentMessageType.TERMINATE_EXECUTION);

                    pendingEACMUpdateMessage = message;


                    analysisHasTerminated = false;
                    planningHasTerminated = false;


                    inMape = true;
                }
                break;

            case START_ACTION_PLAN_EXECUTION:
                //n-are de ce sa raspunda la action plan executed. trebe facute in action enforcement
//                if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
//                    return;
//                }
//                ActionPlanDto actionPlanDto = (ActionPlanDto) content[1];
//                List<String> actionPlanActions = actionPlanDto.getActions();
//                Queue<ContextAction> actionsToBeExecuted = new LinkedList<ContextAction>();
//                for (String actionName : actionPlanActions) {
//                    ContextAction action = eacmInstance
//                            .getITComputingResourceAdaptationAction(actionName);
//                    action.execute(eacmInstance, planningEngine);
//                    action.execute(remoteEACMInstance, planningEngine);
//                    actionsToBeExecuted.add(action);
//                }
                break;


            case ACTION_PLAN_EXECUTION_COMPLETED:

                inMape = false;

                if (pendingMessages.size() > 0) {
                    processPendingMessages();
                }
                System.out.println("Action Execution Completed");

                //TODO: by commenting bellow the MAPE loop will start only when another event is fired
                //if uncommented, after execution, the Monitoring phase directly starts
//                agent.updateContextModel();
//                InterAgentMessageBuilder.sendMessage(agent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS);
                break;

            case EXECUTION_TERMINATED:
                if (message.getSender().getName().contains(ContextAnalysisAgent.class.getName())) {
                    analysisHasTerminated = true;
                }

                if (message.getSender().getName().contains(ActionPlanningAgent.class.getName())) {
                    planningHasTerminated = true;
                }
                if (analysisHasTerminated && planningHasTerminated) {
                    if (pendingEACMUpdateMessage != null) {
                        processMessage(pendingEACMUpdateMessage);
                    }
                    pendingEACMUpdateMessage = null;
                }
                break;
            case TERMINATE_EXECUTION:
                agent.terminate();
                InterAgentMessageBuilder.sendMessage(agent, MainAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                break;

        }


    }

    private void processMessage(ACLMessage message) {
        Object[] content = null;
        try {
            content = (Object[]) message.getContentObject();
        } catch (UnreadableException e) {
            GlobalLoopLogger.error(e);
            e.printStackTrace();
        }

        //the first element of the message is the message type represented by an AgentMessageType instance
        AgentMessageType command = (AgentMessageType) content[0];

        switch (command) {

            case UPDATE_EACM_CONCEPTS:


                CMAEACMConceptsManagementBehavior behavior = new CMAEACMConceptsManagementBehavior(agent, eacmInstance,
                        serviceCenterManagement, CAPInteractions,
                        new Object[]{content[1], content[2]});
                behavior.action();
                InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS, this.getClass().getName());
                break;
            case UPDATE_CONTEXT_INFORMATION:

                CMAContextInformationUpdateBehavior behavior1 = new CMAContextInformationUpdateBehavior(eacmInstance);
                behavior1.action();
                InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS, this.getClass().getName());
                break;
        }

    }

    private void processPendingMessages() {


        for (ACLMessage message : pendingMessages) {

            Object[] content = null;
            try {
                content = (Object[]) message.getContentObject();
            } catch (UnreadableException e) {
                Logger.getLogger(this.getClass()).error(":");
                GlobalLoopLogger.error(e);
                e.printStackTrace();
            }

            //the first element of the message is the message type represented by an AgentMessageType instance
            AgentMessageType command = (AgentMessageType) content[0];

            switch (command) {

                case UPDATE_EACM_CONCEPTS:
                    CMAEACMConceptsManagementBehavior behavior = new CMAEACMConceptsManagementBehavior(agent, eacmInstance,
                            serviceCenterManagement, CAPInteractions,
                            new Object[]{content[1], content[2]});
                    behavior.action();
                    break;

                case UPDATE_CONTEXT_INFORMATION:
                    //update monitored information
                    CMAContextInformationUpdateBehavior behavior1 = new CMAContextInformationUpdateBehavior(eacmInstance);
                    behavior1.action();

            }

        }
        pendingMessages.clear();
        InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS, this.getClass().getName());
        inMape = true;
        pendingEACMUpdateMessage = null;
        analysisHasTerminated = false;
        planningHasTerminated = false;
    }
}
