/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.QoSPolicy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.VirtualNetworkInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;
import ro.utcn.dsrl.utils.exceptions.ApplicationException;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class CMARemoveActivitiesBehavior extends OneShotBehaviour {

    private EACMInstance eacmInstance;
    private List<ActivityDto> tasksToBeDeleted;
    private ServiceCenterManagement serviceCenterManagement;
    private CAPInteractions CAPInteractions;

    public CMARemoveActivitiesBehavior(Agent agent, EACMInstance eacmInstance,
            List<ActivityDto> tasksToBeDeleted,
            ServiceCenterManagement serviceCenterManagement,
            CAPInteractions CAPInteractions) {
        super(agent);
        this.eacmInstance = eacmInstance;
        this.tasksToBeDeleted = tasksToBeDeleted;
        this.serviceCenterManagement = serviceCenterManagement;
        this.CAPInteractions = CAPInteractions;
    }

    public void action() {
//        OWLModel model = ((OntologyEACMInstance)eacmInstance;
        List<String> deletedEntries = new ArrayList<String>();

        for (ActivityDto taskDto : tasksToBeDeleted) {
            String name = taskDto.getTaskName();
            GlobalLoopLogger.getLogger().info("Deleting task " + taskDto.getTaskName() + "_" + taskDto.getIp());
            ApplicationActivity activity = eacmInstance.getApplicationActivity(taskDto.getTaskName() + "_" + taskDto.getIp());

            if (!GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
                try {
                    CAPInteractions.removeDeployedService(taskDto);
                } catch (ApplicationException e) {
                    GlobalLoopLogger.error(e);
                }
            }

            {
                for (Object p : activity.getActivityPolicies()) {
                    QoSPolicy policy = (QoSPolicy) p;
                    policy.delete();
                }


                deletedEntries.add(name);
//

                if (!activity.hasAssociatedServer()) {
                    continue;
                }
                ServiceCenterServer server = (ServiceCenterServer) activity.getAssociatedServer().iterator().next();

                server.removeRunningActivity(activity);
                VirtualTaskInfo virtualTaskInfo = new VirtualTaskInfo(activity.getLocalName());
                virtualTaskInfo.setId(activity.getId());
                VirtualNetworkInfo virtualNetworkInfo = new VirtualNetworkInfo();
                virtualNetworkInfo.setId(activity.getNetworkInfo().getId());
                virtualTaskInfo.setNetworkInfo(virtualNetworkInfo);

                try {
                    //          proxy.stopVirtualMachine(virtualTaskInfo);
                    if (!GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
                        serviceCenterManagement.deleteVirtualMachine(virtualTaskInfo);
                    }
                    Collection<ServiceCenterServer> servers = eacmInstance.getAllServiceCenterServerInstances();
                    for (ServiceCenterServer s : servers) {
                        s.resetInitialValues();
                    }
                } catch (ServiceCenterAccessException e) {
                    GlobalLoopLogger.getLogger().debug(e.toString());
                }

                GlobalLoopLogger.getLogger().info("Deleted " + activity.getName());

                activity.delete();

            }
        }

    }
}
