/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.agents.behaviors;

import java.util.Iterator;
import java.util.List;

import jade.core.Agent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.Core;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.HDD;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITComputingContextPolicy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;
import ro.utcn.dsrl.utils.commonDtos.PhysicalHost;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import jade.core.behaviours.OneShotBehaviour;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class CMARemoveServersBehavior extends OneShotBehaviour {

    private EACMInstance eacmInstance;
    private List<ExtendedServerDto> serversToBeDeleted;
    private ServiceCenterManagement serviceCenterManagement;

    public CMARemoveServersBehavior(Agent agent, EACMInstance eacmInstance,
                                    List<ExtendedServerDto> serversToBeDeleted,
                                    ServiceCenterManagement serviceCenterManagement) {
        super(agent);
        this.eacmInstance = eacmInstance;
        this.serversToBeDeleted = serversToBeDeleted;
        this.serviceCenterManagement = serviceCenterManagement;
    }


    public void action() {
        for (ExtendedServerDto dto : serversToBeDeleted) {
            String name = dto.getServerName();
            ServiceCenterServer server = eacmInstance.getServiceCenterServer(name);
            if (server == null) {
                GlobalLoopLogger.getLogger().debug("Error. Server " + name + " was not found");
                continue;
            }
            PhysicalHost physicalHost = new PhysicalHost();
            physicalHost.setHostname(server.getIpAddress());
            physicalHost.setId(server.getId());
            try {
                serviceCenterManagement.removeHost(physicalHost);
            } catch (ServiceCenterAccessException e) {
                GlobalLoopLogger.getLogger().debug(e.toString());  //To change body of catch statement use File | Settings | File Templates.
            }

            Iterator<CPU> cpuIt = server.getCpuResources().iterator();
            while (cpuIt.hasNext()) {
                CPU cpu = cpuIt.next();
                List<Core> cores = cpu.getAssociatedCores();
                Iterator<Core> coreIt = cores.iterator();
                while (coreIt.hasNext()) {
                    Core entry = coreIt.next();
                    entry.delete();
                    coreIt.remove();
                }
                cpuIt.remove();
            }
            Iterator<MEM> memIt = server.getMemResources().iterator();
            while (memIt.hasNext()) {
                MEM entry = memIt.next();
                entry.delete();
                memIt.remove();
            }

            Iterator<HDD> hddIt = server.getHddResources().iterator();
            while (hddIt.hasNext()) {
                HDD entry = hddIt.next();
                entry.delete();
                hddIt.remove();
            }
            for (ITComputingContextPolicy policy : eacmInstance.getAllITComputingContextPolicyInstances()) {
                if (policy.getName().contains(server.getLocalName())) {
                    policy.delete();
//                    SWRLFactory factory = new SWRLFactory(model);
//                    try {
//                        SWRLImp imp = factory.getImp(policy.getLocalName() + "_Rule");
//                        if (imp != null) {
//                            imp.disable();
//                            imp.deleteImp();
//                        }
//                    } catch (SWRLFactoryException e) {
//                        GlobalLoopLogger.getLogger().debug(e.toString());  //To change body of catch statement use File | Settings | File Templates.
//                    }
//                    OWLIndividual i = model.getOWLIndividual(policy.getName());
//                    Individual ontIndividual = model.getOntModel().getIndividual(policy.getName());
//                    if (ontIndividual != null) {
//                        ontIndividual.remove();
//                    }
//                    RDFResource resource = model.getRDFResource(policy.getName());
//                    if (resource != null) {
//                        resource.getKnowledgeBase().deleteFrame(resource);
//                        resource.getKnowledgeBase().deleteInstance(resource);
//                        resource.delete();
//                    }
//                    if (i != null) {
//                        i.getKnowledgeBase().deleteFrame(i);
//                        i.getKnowledgeBase().deleteInstance(i);
//                        i.delete();
//                        model.deleteInstance(model.getInstance(i.getName()));
//                    }
//                    Instance instance = model.getInstance(policy.getName());
//                    if (instance != null) {
//                        instance.getKnowledgeBase().deleteFrame(instance);
//                        instance.getKnowledgeBase().deleteInstance(instance);
//                        instance.delete();
//
//                    }
                    break;
                }
            }
//            Instance instance = model.getInstance(server.getName());
//            if (instance != null) {
//                instance.getKnowledgeBase().deleteFrame(instance);
//                instance.getKnowledgeBase().deleteInstance(instance);
//                instance.delete();
//            }
//
//            OWLIndividual i = model.getOWLIndividual(server.getName());
//            if (i != null) {
//                i.getKnowledgeBase().deleteFrame(i);
//                i.getKnowledgeBase().deleteInstance(i);
//                i.delete();
//            }
//            model.deleteInstance(model.getInstance(i.getName()));
//            Individual ontIndividual = model.getOntModel().getIndividual(server.getName());
//            if (ontIndividual != null) {
//                ontIndividual.remove();
//            }
//            RDFResource resource = model.getRDFResource(server.getName());
//            if (resource != null) {
//                resource.delete();
//                resource.getKnowledgeBase().deleteFrame(resource);
//                resource.getKnowledgeBase().deleteInstance(resource);
//            }
            GlobalLoopLogger.getLogger().info("Deleted " + server.getName());
            server.delete();
        }
        
        
        //notification done from ContextMonitoringAgent CMAReceiveMEssageBevahior. Avoids
            //multiple notifications for example when a deploy and delete happen in same step

//        if(serversToBeDeleted.size() > 0){
//             //notify the ContextAnalysisAgent that the context has changed and needs to be reanalyzed
//            InterAgentMessageBuilder.sendMessage(myAgent, ContextAnalysisAgent.class, AgentMessageType.START_CONTEXT_ANALYSIS,this.getClass().getName());
//        }

    }

}
