/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.utils.objectToDTOMappers;


import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.*;
import ro.utcn.dsrl.utils.commonDtos.ActionDto;

import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ActionToDtoMapper {

    private ActionToDtoMapper() {
    }

    public static ActionDto map(ContextAction action) {
        ActionDto dto = new ActionDto();
        dto.setActionClassName(action.getClass().getName());
        dto.setActionName(action.getLocalName());
        dto.setCost(action.getCost());
        List<String> resources = new ArrayList<String>();
        if (action instanceof DeployActivity) {
            resources.add(((DeployActivity) action).getActivity().getName());
        } else if (action instanceof MigrateActivity) {
            resources.add(((MigrateActivity) action).getActivity().getName());
        } else if (action instanceof SetServerStateActivity) {
            dto.setTargetServerState(((SetServerStateActivity) action).getTargetServerState());
        }

        for (ContextResource resource : action.getResources()) {
            resources.add(resource.getName());
        }

        dto.setContextResources(resources);

        return dto;
    }
}
