/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.utils.objectToDTOMappers;

import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ApplicationActivity;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;

import java.util.Collection;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ContextSituationToDtoMapper {

    private ContextSituationToDtoMapper() {
    }

    public static ContextSituationDto map(EACMInstance modelAccess) {
        ContextSituationDto situationDto = new ContextSituationDto();
        Collection<ServiceCenterServer> serversList = modelAccess.getAllServiceCenterServerInstances();
        Collection<ApplicationActivity> tasks = modelAccess.getAllApplicationActivityInstances();

        for (ServiceCenterServer server : serversList) {
            situationDto.addServerDto(ServerToDtoMapper.map(server));
        }

        for (ApplicationActivity activity : tasks) {
            situationDto.addTaskDto(TaskToDtoMapper.map(activity));
        }
        return situationDto;
    }
}
