/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.utils.objectToDTOMappers;

import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.actionPlanningModule.utils.actionExecutionSimulation.IActionSimulationUtil;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextAction;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.GPI_KPI_Policy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.QoSPolicy;

import java.util.List;
import java.util.Queue;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class RLNode implements Comparable {

    private Queue<ContextAction> actions;
    private List<GPI_KPI_Policy> brokenPolicies;
    private double contextEntropy = 0;
    private double rewardFunction = 0;
    public final static double gamma = 0.3d;
    private int brokenQoSPoliciesCount = 0;

    public RLNode(final Queue<ContextAction> actions) {
        this.actions = actions;

    }

    public double getContextEntropy() {
        return contextEntropy;
    }

    public List<GPI_KPI_Policy> getBrokenPolicies() {
        return brokenPolicies;
    }

    public void setBrokenPolicies(List<GPI_KPI_Policy> brokenPolicies) {
        this.brokenPolicies = brokenPolicies;
        for (GPI_KPI_Policy policy : brokenPolicies) {
            if (policy instanceof QoSPolicy) {
                brokenQoSPoliciesCount++;
            }
        }
    }

    public int getBrokenQoSPoliciesCount() {
        return brokenQoSPoliciesCount;
    }

    public void setContextEntropy(double contextEntropy) {
        this.contextEntropy = contextEntropy;
    }

    public void addActions(Queue<ContextAction> commands) {
        for (ContextAction command : commands) {
            actions.add(command);
        }
    }

    public void executeActions(EACMInstance modelAccess, IActionSimulationUtil actionSimulationUtil) {
        for (ContextAction command : actions) {
            actionSimulationUtil.simulateActionExecution(modelAccess,command);
        }
    }


    /**
     * Rewinds in the inverse order
     */
    public void undoAllActions(EACMInstance model) {

        Object[] commands = actions.toArray();

        for (int i = commands.length - 1; i >= 0; i--) {
            ContextAction command = (ContextAction) commands[i];
            command.undo(model);
        }
    }

    public Queue<ContextAction> getActions() {
        return actions;
    }

    public void setActions(final Queue<ContextAction> actions) {
        this.actions = actions;
    }

    /**
     * @return the rewardFunction
     */
    public double getRewardFunction() {
        return rewardFunction;
    }

    /**
     * @param rewardFunction the rewardFunction to set
     */
    public void setRewardFunction(double rewardFunction) {
        this.rewardFunction = rewardFunction;
    }

    public boolean equals(Object o) {
        if (o.getClass() != this.getClass()) {
            return false;
        }
        RLNode cs = (RLNode) o;
        return (cs.contextEntropy == contextEntropy) && (cs.rewardFunction == rewardFunction) && this.actions.equals(cs.actions);
    }

    public int compareTo(Object o) {
        if (o.getClass() != this.getClass()) {
            return 0;
        }
        RLNode cs = (RLNode) o;


        if (contextEntropy > cs.contextEntropy) {
            return 1;
        } else if (contextEntropy < cs.contextEntropy) {
            return -1;
        } else if (rewardFunction > cs.rewardFunction) {
            return -1;
        } else if (rewardFunction < cs.rewardFunction) {
            return 1;
        } else {
            return 0;
        }
    }

    public String toString() {
        String description;
        description = "Entropy: " + this.contextEntropy + " Reward: " + this.rewardFunction;
        return description;
    }


}
