/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.contextMonitoringModule.utils.objectToDTOMappers;


import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.Core;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ServerToDtoMapper {
    private ServerToDtoMapper() {

    }

    public static ExtendedServerDto map(ServiceCenterServer server) {
        ExtendedServerDto dto = new ExtendedServerDto();
        CPU cpu = server.getCpuResources().iterator().next();
        MEM mem = server.getMemResources().iterator().next();
        Collection<Core> cores = cpu.getAssociatedCores();
        List<Integer> freeCPU = new ArrayList<Integer>();
        for (Core core : cores) {
            freeCPU.add(core.getMaximumWorkLoad().intValue() - core.getCurrentWorkLoad().intValue());
        }

        //TODO: de gandit ce facem cu DTO-urile. Daca tinem core-uri, daca nu, cum facem
//        dto.setFreeCPU(freeCPU);

//        dto.setFreeMemory(mem.getMaximumWorkLoad().intValue() - mem.getCurrentWorkLoad().intValue());
        dto.setIpAddress(server.getIpAddress());
        dto.setMacAddress(server.getMacAddress());
        dto.setMaximumCPU(cpu.getMaximumWorkLoad().intValue());
        dto.setMaximumMemory(mem.getMaximumWorkLoad().intValue());
        dto.setOptimumCPU(cpu.getOptimalWorkLoad().intValue());
        dto.setOptimumMemory(mem.getOptimalWorkLoad().intValue());
        dto.setTotalCPU(cpu.getMaximumWorkLoad().intValue());
        dto.setTotalCPU(mem.getMaximumWorkLoad().intValue());
        dto.setUsedCPU(cpu.getCurrentWorkLoad().intValue());
        dto.setUsedMemory(mem.getCurrentWorkLoad().intValue());
        dto.setServerName(server.getName());
        dto.setCoreNo(cpu.getCpuCoreCount());
        return dto;
    }
}
