/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.ContextAccessAndProcessingAPI;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.impl.ContextAccessAndProcessing;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ContextElement;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.ContextModelAccessException;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Administrator
 */
public class ContextSnapshotSaveAndLoadTest {

    public static void main(String[] args) {
        try {
            ContextAccessAndProcessingAPI api = null;
            try {
                api = new ContextAccessAndProcessing();
                api.loadContextInstanceAndSetCurrent();
            } catch (ContextModelAccessException ex) {
                Logger.getLogger(Test.class.getName()).log(Level.SEVERE, null, ex);
            }

            EACMInstance instance = api.getCurrentContextInstance();

            //saves the current context instance to a file local respective to the GAMES-CAP client project
            api.saveLocalContextInstanceToFile("./ontology/EnergyAwareContextDatabase/BeforeInstanceCreationSnapshot.owl", instance);

            instance.createApplication("Application_instance");

            //saves the current context instance to a file local respective to the GAMES-CAP client project
            api.saveLocalContextInstanceToFile("./ontology/EnergyAwareContextDatabase/AfterInstanceCreationSnapshot.owl", instance);

            //print all instances
            System.out.println("Before load ");
            for (ContextElement element : instance.getAllContextElementInstances()) {
                System.out.println(element.getName());
            }

            //loads a local file and sets it as current EACM Instance
            //NOTE: This does not change the Context Model located on the GAMES_CAP ProtegeServer
            //      By loading a local snapshot, only the current project(from where the load has been performed) 
            //      has access to the laoded snapshot
            instance = api.loadLocalContextInstanceFromFile("./ontology/EnergyAwareContextDatabase/BeforeInstanceCreationSnapshot.owl");

            //print all instances from the loaded context snapshot
            System.out.println("After load ");
            for (ContextElement element : instance.getAllContextElementInstances()) {
                System.out.println(element.getName());
            }

            instance = api.loadLocalContextInstanceFromFile("./ontology/EnergyAwareContextDatabase/AfterInstanceCreationSnapshot.owl");

            //print all instances from the loaded context snapshot
            System.out.println("After second load ");
            for (ContextElement element : instance.getAllContextElementInstances()) {
                System.out.println(element.getName());
            }

            //reconnect to the Protege Server Context Model instance
            api.loadContextInstanceAndSetCurrent();


            System.out.println("From server ");
            for (ContextElement element : instance.getAllContextElementInstances()) {
                System.out.println(element.getName());
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
