/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.mainModule.agents;

import edu.stanford.smi.protege.exception.OntologyLoadException;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import jade.core.Agent;
import jade.wrapper.AgentContainer;
import ro.utcn.dsrl.actionEnforcementModule.actionEnforcement.ContextActionEnforcer;
import ro.utcn.dsrl.actionEnforcementModule.actionEnforcement.impl.DefaultContextActionEnforcer;
import ro.utcn.dsrl.actionEnforcementModule.agents.ActionPlanEnforcementAgent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.CAPInteractions;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.factories.impl.DefaultServiceCenterManagementFactory;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.impl.DefaultCAPInteractions;
import ro.utcn.dsrl.actionPlanningModule.agents.ActionPlanningAgent;
import ro.utcn.dsrl.actionPlanningModule.reasoning.PlanningEngine;
import ro.utcn.dsrl.actionPlanningModule.reasoning.impl.FFDPlanningEngine;
import ro.utcn.dsrl.actionPlanningModule.reasoning.impl.ReinforcementLearningPlanningEngine;
import ro.utcn.dsrl.clusterContext.contextAnalysis.ClusterContextEntropyEvaluation;
import ro.utcn.dsrl.clusterContext.contextAnalysis.ClusterServerLoadPolicyViolationEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.PolicyAnalyser;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.ContextEntropyEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.QoSPolicyViolationEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.ServerLoadPolicyViolationEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.impl.DefaultContextEntropyEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.impl.DefaultQoSPolicyViolationEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.impl.DefaultServerLoadPolicyViolationEvaluator;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.impl.DefaultPolicyAnalyser;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.ContextAccessAndProcessingAPI;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextAccessAndProcessing.impl.ContextAccessAndProcessing;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.api.EACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.CPU;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ITComputingContextPolicy;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.MEM;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.concepts.ServiceCenterServer;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.contextModel.modelAPI.OntologyEACMInstance;
import ro.utcn.dsrl.contextAccessingAndProcessingAPI.util.exceptions.ContextModelAccessException;
import ro.utcn.dsrl.utils.exposedFunctionality.rmiAPI.RMIAPI;
import ro.utcn.dsrl.utils.exposedFunctionality.rmiAPI.RMICommand;
import ro.utcn.dsrl.contextMonitoringModule.exposedFunctionality.rmiAPI.impl.RMIInterface;
import ro.utcn.dsrl.openNebulaPlugin.agents.VMPoolMonitoringAgent;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ServerInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfoToActivityDtoMapper;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import ro.utcn.dsrl.utils.misc.AgentCreator;
import ro.utcn.dsrl.workloadScheduler.WorkloadSchedulerAgent;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import ro.utcn.dsrl.contextAnalysisModule.contextAnalysis.contextMetricsComputation.impl.SWRLContextEntropyEvaluator;

import ro.utcn.dsrl.mainModule.agents.behaviors.MAReceiveMessageBehavior;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class MainAgent extends Agent {

    private EACMInstance localEacmInstance;
    private ContextAccessAndProcessingAPI contextAccessAndProcessingAPI;

    protected void setup() {

        GlobalLoopLogger.getLogger().info("Loading local ontology");
        String ontologyFile = "./eacmOntology/EnergyAwareContextModel/EACM.owl";
        File file = new File(ontologyFile);
        OWLModel model = null;
        try {
            model = ProtegeOWL.createJenaOWLModelFromURI(file.toURI().toString());
        } catch (OntologyLoadException e) {
            GlobalLoopLogger.error(e);
            GlobalLoopLogger.getLogger().info(e.toString());
            System.exit(1);
        }
        localEacmInstance = new OntologyEACMInstance(model);

        PolicyAnalyser policyAnalyser = new DefaultPolicyAnalyser();
        ServerLoadPolicyViolationEvaluator serverLoadViolationEvaluator =
                GeneralConfigurationManager.runningInClusterMode()
                ? new ClusterServerLoadPolicyViolationEvaluator()
                : new DefaultServerLoadPolicyViolationEvaluator();
        QoSPolicyViolationEvaluator qosPolicyViolationEvaluator = new DefaultQoSPolicyViolationEvaluator();
        ContextEntropyEvaluator entropyEvaluator = GeneralConfigurationManager.runningInClusterMode()
                ? new ClusterContextEntropyEvaluation(policyAnalyser, serverLoadViolationEvaluator, qosPolicyViolationEvaluator)
                : new DefaultContextEntropyEvaluator(policyAnalyser, serverLoadViolationEvaluator, qosPolicyViolationEvaluator);

        //TODO: implements the cluster context analysis with swrl if deployed on verry large service center
        ContextEntropyEvaluator analysisPhaseContextEntropyEvaluator = GeneralConfigurationManager.runningInClusterMode()
                ? new ClusterContextEntropyEvaluation(policyAnalyser, serverLoadViolationEvaluator, qosPolicyViolationEvaluator)
                : (GeneralConfigurationManager.swrlEnabled())
                ? new SWRLContextEntropyEvaluator(serverLoadViolationEvaluator, qosPolicyViolationEvaluator)
                : new DefaultContextEntropyEvaluator(policyAnalyser, serverLoadViolationEvaluator, qosPolicyViolationEvaluator);

        PlanningEngine planningEngine = null;
        if (GeneralConfigurationManager.getPlanningEngine().equals("ffd")) {
            planningEngine = new FFDPlanningEngine(entropyEvaluator, localEacmInstance);
        } else {
            if (GeneralConfigurationManager.getPlanningEngine().equals("rl")) {
                planningEngine = new ReinforcementLearningPlanningEngine(entropyEvaluator, localEacmInstance);
            }
        }
        ServiceCenterManagement serviceCenterManagement = DefaultServiceCenterManagementFactory.getInstance().createServiceCenterManagement();

        ContextActionEnforcer contextActionEnforcer = new DefaultContextActionEnforcer(serviceCenterManagement);

        CAPInteractions capInteractions = null;


        EACMInstance remoteEacmInstance = null;

        final List<ServiceCenterServer> localServiceCenterServers = new ArrayList<ServiceCenterServer>();

        if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
            try {

                List<ServerInfo> serverInfos = serviceCenterManagement.getDefinedServers();
//                for (int i = 0; i < 50; i++) {
                    for (ServerInfo serverInfo : serverInfos) {
                        ServiceCenterServer serviceCenterServer = localEacmInstance.createServiceCenterServer(serverInfo.getHostName());// + "_" + i);
                        CPU serviceCenterServerCPU = localEacmInstance.createCPU("CPU_for_"
                                + serviceCenterServer.getName().split("#")[1] );//+ "_" + i);
                        MEM serviceCenterServerMEM = localEacmInstance.createMEM("MEM_for_"
                                + serviceCenterServer.getName().split("#")[1] );//+ "_" + i);

                        int coresNo = serverInfo.getTotalCpu() / 100;

                        float optimumLoadPercentage = GeneralConfigurationManager.getResourceLoadOptimalValue() / 100;

                        Double maxCPUWorkload = new Double(serverInfo.getTotalCpu());

                        serviceCenterServerCPU.setMaximumWorkLoad(maxCPUWorkload);

                        //TODO: de luat din config file ala cu CPU optimum etc
                        serviceCenterServerCPU.setOptimalWorkLoad(optimumLoadPercentage * maxCPUWorkload);
                        serviceCenterServerCPU.setCpuCoreCount(coresNo);

                        serviceCenterServer.setIpAddress(serverInfo.getHostName());
                        serviceCenterServer.setMacAddress(serverInfo.getMacAddress());
                        serviceCenterServer.setCPUWeight(0.3f);
                        serviceCenterServer.setMEMWeight(0.3f);
                        serviceCenterServer.setHDDWeight(0.3f);


                        serviceCenterServer.setId(serverInfo.getCloudID());

                        Double totalMemory = new Double(serverInfo.getTotalMem()) / 1024;

                        serviceCenterServerMEM.setMaximumWorkLoad(totalMemory);
                        serviceCenterServerMEM.setOptimalWorkLoad(optimumLoadPercentage * totalMemory);

                        Collection<CPU> cpus = new ArrayList<CPU>();
                        Collection<MEM> mems = new ArrayList<MEM>();

                        cpus.add(serviceCenterServerCPU);
                        mems.add(serviceCenterServerMEM);

                        serviceCenterServer.setCpuResources(cpus);
                        serviceCenterServer.setMemResources(mems);

                        ITComputingContextPolicy policy = localEacmInstance.createITComputingContextPolicy(serviceCenterServer.getName().split("#")[1] + "_EnergyPolicy");// + "_" + i);
                        policy.addPolicySubject(serviceCenterServer);
                        policy.addPolicyTarget(serviceCenterServer);
                        policy.setPolicyWeight(0.3f);
                        policy.setPolicyName(policy.getName());
                        policy.setEvaluationCondition("Evaluation_condition");
                        serviceCenterServer.setServiceCenterServerPolicy(policy);
                        synchronized (localServiceCenterServers) {
                            localServiceCenterServers.add(serviceCenterServer);
                        }

                        serviceCenterServer.setIsActive(serviceCenterManagement.serverIsAlive(serviceCenterServer.getIpAddress()));

                    }
//                }
            } catch (ServiceCenterAccessException e) {
                GlobalLoopLogger.error(e);
            }
            try {
                Registry registry = LocateRegistry.createRegistry(GeneralConfigurationManager.getRMIAPIPort());
                RMIInterface rmiInterface = new RMIInterface(this, serviceCenterManagement, capInteractions);
                registry.rebind(RMIAPI.class.getName(), rmiInterface);
            } catch (RemoteException e) {
                GlobalLoopLogger.getLogger().debug(e.toString());
            }

        } else {

            try {
                GlobalLoopLogger.getLogger().info("Accessing CAP");
                contextAccessAndProcessingAPI = new ContextAccessAndProcessing();
                contextAccessAndProcessingAPI.createContextInstance();
                remoteEacmInstance = contextAccessAndProcessingAPI.getCurrentContextInstance();
            } catch (ContextModelAccessException ex) {
                GlobalLoopLogger.getLogger().info("Terminating execution");
                GlobalLoopLogger.getLogger().info(
                        "GAMES-Context Accessing and Processing API has not found on RMI at address localhost:1099");
                GlobalLoopLogger.getLogger().debug(ex.toString());
                JFrame frame = new JFrame();
                frame.setVisible(true);
                Toolkit toolkit = Toolkit.getDefaultToolkit();
                Dimension screenSize = toolkit.getScreenSize();
                frame.setLocation(screenSize.width / 2, screenSize.height / 2);
                JOptionPane.showMessageDialog(
                        frame,
                        "GAMES-Context Accessing and Processing API has not found on RMI at address localhost:1099.\n Please run the GAMES-Context Accessing and Processing API component",
                        "GAMES-Context Accessing and Processing API not running", JOptionPane.ERROR_MESSAGE);

                System.exit(1);
            }


            capInteractions = new DefaultCAPInteractions(contextAccessAndProcessingAPI);
            try {
                Registry registry = LocateRegistry.createRegistry(GeneralConfigurationManager.getRMIAPIPort());
                RMIInterface rmiInterface = new RMIInterface(this, serviceCenterManagement, capInteractions);
                registry.rebind(RMIAPI.class.getName(), rmiInterface);
            } catch (RemoteException e) {
                GlobalLoopLogger.getLogger().debug(e.toString());
            }


            Collection<ServiceCenterServer> serviceCenterServersOnRemote = remoteEacmInstance.getAllServiceCenterServerInstances();

//        final ServiceCenterServer remoteServerDefinition = serviceCenterServersOnRemote.iterator().next();
            for (ServiceCenterServer remoteServerDefinition : serviceCenterServersOnRemote) {
                final CountDownLatch countDownLatch = new CountDownLatch(1000);
                final CPU cpu = remoteServerDefinition.getCpuResources().iterator().next();
                final MEM memory = remoteServerDefinition.getMemResources().iterator().next();

//        for (int a = 0; a < GeneralConfigurationManager.getServersNo(); a++) {
//            final int i = a;
//            Thread thread = new Thread() {
//                @Override
//                public void run() {
//            System.out.println("Created " + i);
                ServiceCenterServer serviceCenterServer = localEacmInstance.createServiceCenterServer(remoteServerDefinition.getIpAddress());// + "_") + i);
                CPU serviceCenterServerCPU = localEacmInstance.createCPU("CPU_for_"
                        + serviceCenterServer.getName().split("#")[1]);//+ "_" + i);
                MEM serviceCenterServerMEM = localEacmInstance.createMEM("MEM_for_"
                        + serviceCenterServer.getName().split("#")[1]);//+ "_" + i);

                serviceCenterServer.setEPKBId(remoteServerDefinition.getEPKBId());

                int coresNo = cpu.getCpuCoreCount();

                serviceCenterServerCPU.setEPKBId(cpu.getEPKBId());

                Double maxCPUWorkload = new Double(cpu.getMaximumWorkLoad());

                serviceCenterServerCPU.setMaximumWorkLoad(maxCPUWorkload);
                serviceCenterServerCPU.setOptimalWorkLoad(maxCPUWorkload / 2);
                serviceCenterServerCPU.setCpuCoreCount(coresNo);

                serviceCenterServer.setIpAddress(remoteServerDefinition.getIpAddress());
                serviceCenterServer.setMacAddress(remoteServerDefinition.getMacAddress());
                serviceCenterServer.setCPUWeight(0.3f);
                serviceCenterServer.setMEMWeight(0.3f);
                serviceCenterServer.setHDDWeight(0.3f);

                // todo: de facut ceva cu varchar(45) pe cloud_supplier_id
                serviceCenterServer.setId(remoteServerDefinition.getId());

                Double totalMemory = memory.getMaximumWorkLoad();

                serviceCenterServerMEM.setMaximumWorkLoad(totalMemory);
                serviceCenterServerMEM.setOptimalWorkLoad(totalMemory / 2);

                Collection<CPU> cpus = new ArrayList<CPU>();
                Collection<MEM> mems = new ArrayList<MEM>();

                cpus.add(serviceCenterServerCPU);
                mems.add(serviceCenterServerMEM);

                serviceCenterServer.setCpuResources(cpus);
                serviceCenterServer.setMemResources(mems);

                ITComputingContextPolicy policy = localEacmInstance.createITComputingContextPolicy(serviceCenterServer.getName().split("#")[1] + "_EnergyPolicy");//+ "_" + i);
                policy.addPolicySubject(serviceCenterServer);
                policy.addPolicyTarget(serviceCenterServer);
                policy.setPolicyWeight(0.3f);
                policy.setPolicyName(policy.getName());
                policy.setEvaluationCondition("Evaluation_condition");
                serviceCenterServer.setServiceCenterServerPolicy(policy);
                synchronized (localServiceCenterServers) {
                    localServiceCenterServers.add(serviceCenterServer);
                }
                try {
                    serviceCenterServer.setIsActive(serviceCenterManagement.serverIsAlive(serviceCenterServer.getIpAddress()));
                } catch (ServiceCenterAccessException e) {
                    GlobalLoopLogger.getLogger().info(e.toString());
                    GlobalLoopLogger.error(e);
                }
//                    countDownLatch.countDown();
//                }
//            };
//            thread.start();
            }
        }
//        try {
//            System.err.println("Waiting ");
//            countDownLatch.await();
//        } catch (InterruptedException e) {
//            e.printStackTrace();
//            GlobalLoopLogger.error(e);
//        }


        if (GeneralConfigurationManager.runningInClusterMode() && localServiceCenterServers.size() > 0) {
            localServiceCenterServers.listIterator().next();
            //creating clusters
            int clusterSize = GeneralConfigurationManager.getClusterSize();
            int clusterIndex = 0;
            int clusterLevelIndex = 0;
            ServiceCenterServer lastCluster = null;

            while (localServiceCenterServers.size() > 1) {
                clusterIndex = 0;
                int count = localServiceCenterServers.size();
                List<ServiceCenterServer> clusters = new ArrayList<ServiceCenterServer>();
                for (int j = 0; j < count;) {

                    ServiceCenterServer cluster = localEacmInstance.createServiceCenterServer("Cluster_" + clusterLevelIndex + "_" + clusterIndex);
                    cluster.setIsCluster(true);
                    ITComputingContextPolicy policy = localEacmInstance.createITComputingContextPolicy(cluster.getName().split("#")[1] + "_EnergyPolicy");
                    policy.addPolicySubject(cluster);
                    policy.addPolicyTarget(cluster);
                    policy.setPolicyWeight(0.3f);
                    policy.setPolicyName(policy.getName());
                    policy.setEvaluationCondition("Evaluation_condition");
                    for (int i = 0; i < clusterSize; i++) {
                        if (count > j) {
//                            System.out.println("Adding to " + cluster.getName().split("#")[1]);
                            cluster.addResource(localServiceCenterServers.get(j));
                            j++;
                            //iterator.remove();
                        } else {
                            break;
                        }
                    }
                    clusters.add(cluster);
                    cluster.setServiceCenterServerPolicy(policy);
                    lastCluster = cluster;
                    clusterIndex++;
                }
                localServiceCenterServers.clear();
                localServiceCenterServers.addAll(clusters);

                clusterLevelIndex++;
            }
//            if (lastCluster != null) {
//                lastCluster.setIsRootCluster(true);
//            } else {
//                GlobalLoopLogger.getLogger().info("No cluster was generated");
//            }
            localServiceCenterServers.get(0).setIsRootCluster(true);
        }

//           for(ServiceCenterServer server : localEacmInstance.getAllServiceCenterServerInstances()){
//            System.out.println(server.getName().split("#")[1]);
//        }
//        for(ServiceCenterServer serviceCenterServer: localEacmInstance.getAllServiceCenterServerInstances()){
//            String resourcesNames = "";
//            Collection<ServiceCenterITComputingResource>  resources = serviceCenterServer.getResources();
//            for(ServiceCenterITComputingResource resource : resources){
//                resourcesNames += resource.getName().split("#")[1] + ",   ";
//            }
//            System.out.println(serviceCenterServer.getName().split("#")[1] + " : " + resourcesNames);
//        }


        AgentContainer container = (AgentContainer) getContainerController();

        MAReceiveMessageBehavior behavior = new MAReceiveMessageBehavior(this);
        addBehaviour(behavior);

        behavior.addRunningAgentName(ContextMonitoringAgent.class.getName());
        behavior.addRunningAgentName(ContextAnalysisAgent.class.getName());
        behavior.addRunningAgentName(ActionPlanningAgent.class.getName());
        behavior.addRunningAgentName(ActionPlanEnforcementAgent.class.getName());

        AgentCreator.createAgent(container, ContextMonitoringAgent.class,
                localEacmInstance, remoteEacmInstance, serviceCenterManagement,
                capInteractions, planningEngine);
        AgentCreator.createAgent(container, ContextAnalysisAgent.class,
                localEacmInstance, analysisPhaseContextEntropyEvaluator);
        AgentCreator.createAgent(container, ActionPlanningAgent.class,
                localEacmInstance, entropyEvaluator, planningEngine, capInteractions);
        AgentCreator.createAgent(container, ActionPlanEnforcementAgent.class,
                localEacmInstance, contextAccessAndProcessingAPI, contextActionEnforcer, capInteractions);
//
        //respond only to Hook if the if below is commented

        if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler() && GeneralConfigurationManager.isVMPoolingEnabled()) {
            AgentCreator.createAgent(container, VMPoolMonitoringAgent.class,
                    serviceCenterManagement);
        }

        if (GeneralConfigurationManager.isDeployedAsOpenNebulaScheduler()) {
            List<VirtualTaskInfo> pendingTasks = null;
            try {
                pendingTasks = serviceCenterManagement.getAllVirtualMachines();
            } catch (ServiceCenterAccessException e) {
                GlobalLoopLogger.error(e);
            }

            List<ActivityDto> activityDtos = new ArrayList<ActivityDto>();

            for (VirtualTaskInfo virtualTaskInfo : pendingTasks) {
                activityDtos.add(VirtualTaskInfoToActivityDtoMapper.map(virtualTaskInfo));
            }

            if (activityDtos.size() > 0) {
                Serializable[] toBeSent = new Serializable[activityDtos.size()];
                toBeSent = activityDtos.toArray(toBeSent);

                try {
                    RMIAPI rmiapi;
                    Registry registry = LocateRegistry.getRegistry(GeneralConfigurationManager.getRMIAPIPort());
                    rmiapi = (RMIAPI) registry.lookup(RMIAPI.class.getName());
                    rmiapi.processCommand(RMICommand.SERVICE_COLLECTION_DEPLOYMENT_NOTIFICATION, toBeSent);
                } catch (Exception ex) {
                    GlobalLoopLogger.error(ex);
                    ex.printStackTrace();
                }

            } else {
                //analyze intial situation
                InterAgentMessageBuilder.sendMessage(this, ContextAnalysisAgent.class,
                        AgentMessageType.START_CONTEXT_ANALYSIS);
            }

        } else {
            //analyze intial situation
            InterAgentMessageBuilder.sendMessage(this, ContextAnalysisAgent.class,
                    AgentMessageType.START_CONTEXT_ANALYSIS);
        }

        if (GeneralConfigurationManager.usesWorkloadGenerator()) {
            AgentCreator.createAgent(container, WorkloadSchedulerAgent.class, localEacmInstance, capInteractions);
        }

        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }


        GlobalLoopLogger.getLogger().info("MA started");

    }
}
