/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.mainModule.agents.behaviors;

import ro.utcn.dsrl.mainModule.agents.MainAgent;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.wrapper.ControllerException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ro.utcn.dsrl.actionEnforcementModule.agents.ActionPlanEnforcementAgent;
import ro.utcn.dsrl.actionPlanningModule.agents.ActionPlanningAgent;
import ro.utcn.dsrl.contextAnalysisModule.agents.ContextAnalysisAgent;
import ro.utcn.dsrl.contextMonitoringModule.agents.ContextMonitoringAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;
import ro.utcn.dsrl.utils.agentsCommunication.InterAgentMessageBuilder;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class MAReceiveMessageBehavior extends CyclicBehaviour {

    private MainAgent agent;
    private List<String> runningAgentsNames;

    public MAReceiveMessageBehavior(MainAgent agent) {
        super();
        this.agent = agent;
        runningAgentsNames = new ArrayList<String>();
    }

    public void addRunningAgentName(String name) {
        runningAgentsNames.add(name);
    }


    public void action() {
        ACLMessage message = agent.blockingReceive();

        if (message == null) {
            return;
        }

        Object[] content = null;
        try {
            content = (Object[]) message.getContentObject();
        } catch (UnreadableException e) {
            GlobalLoopLogger.error(e);
        }
        AgentMessageType command = null;
        try {
            command = (AgentMessageType) content[0];
        } catch (Exception err) {
            System.out.println(message);
        }
        switch (command) {
            case TERMINATE_EXECUTION:
                InterAgentMessageBuilder.sendMessage(agent, ContextMonitoringAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                InterAgentMessageBuilder.sendMessage(agent, ContextAnalysisAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                InterAgentMessageBuilder.sendMessage(agent, ActionPlanningAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                InterAgentMessageBuilder.sendMessage(agent, ActionPlanEnforcementAgent.class, AgentMessageType.TERMINATE_EXECUTION);
                break;
            case EXECUTION_TERMINATED:
                String senderName = message.getSender().getName();
                Iterator<String> agentNamesIterator = runningAgentsNames.iterator();
                while (agentNamesIterator.hasNext()) {
                    String name = agentNamesIterator.next();
                    if (senderName.contains(name)) {
                        agentNamesIterator.remove();
                        break;
                    }
                }
                if (runningAgentsNames.size() == 0) {
                    try {
                        agent.getContainerController().getPlatformController().kill();
                    } catch (ControllerException e) {
                        GlobalLoopLogger.error(e);
                    }
                }
                break;
        }
    }
}
