/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.openNebulaPlugin.agents;

import jade.core.Agent;
import ro.utcn.dsrl.actionEnforcementModule.serviceCenterManagement.ServiceCenterManagement;
import ro.utcn.dsrl.openNebulaPlugin.agents.behaviors.ReceiveMessagesVMPoolMonitoringAgentBehavior;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.exceptions.ServiceCenterAccessException;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class VMPoolMonitoringAgent extends Agent implements Interruptable {

    private List<VirtualTaskInfo> pendingVirtualMachines;
    private ServiceCenterManagement serviceCenterManagement;
    private ReceiveMessagesVMPoolMonitoringAgentBehavior behavior;

    @Override
    protected void setup() {
        Object[] args = getArguments();
        if (args != null && args.length > 0) {
            serviceCenterManagement = (ServiceCenterManagement) args[0];
        } else {
            GlobalLoopLogger.getLogger().info("VMPMA not enough parameters received.Terminating");
            this.takeDown();
        }
        behavior = new ReceiveMessagesVMPoolMonitoringAgentBehavior(this);
        addBehaviour(behavior);
        pendingVirtualMachines = new ArrayList<VirtualTaskInfo>();
    }

    public boolean hasPendingVMs() {
        return !pendingVirtualMachines.isEmpty();
    }

    public void poolForPendingVMs() {
        try {
            pendingVirtualMachines = serviceCenterManagement.getPendingVirtualMachines();
        } catch (ServiceCenterAccessException e) {
            GlobalLoopLogger.error(e);
            GlobalLoopLogger.debug(e);
        }
    }

    public VirtualTaskInfo getNextPendingVM() {
        return (!pendingVirtualMachines.isEmpty()) ? pendingVirtualMachines.remove(0) : null;
    }

    public List<VirtualTaskInfo> getPendingVirtualMachines() {
        return pendingVirtualMachines;
    }

    public void terminate() {
        behavior.terminate();
    }
}
