/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.openNebulaPlugin.agents.behaviors;

import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;

import ro.utcn.dsrl.utils.exposedFunctionality.rmiAPI.RMIAPI;
import ro.utcn.dsrl.utils.exposedFunctionality.rmiAPI.RMICommand;
import ro.utcn.dsrl.openNebulaPlugin.agents.VMPoolMonitoringAgent;
import ro.utcn.dsrl.utils.agentsCommunication.AgentMessageType;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfo;
import ro.utcn.dsrl.utils.commonDtos.VirtualTaskInfoToActivityDtoMapper;
import ro.utcn.dsrl.utils.commonInterfaces.Interruptable;
import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class ReceiveMessagesVMPoolMonitoringAgentBehavior extends CyclicBehaviour implements Interruptable {

    private VMPoolMonitoringAgent agent;
    private Timer vmPoolingTimer;

    public ReceiveMessagesVMPoolMonitoringAgentBehavior(VMPoolMonitoringAgent agent) {
        super(agent);
        this.agent = agent;
        final VMPoolMonitoringAgent a = agent;

        vmPoolingTimer = new Timer(GeneralConfigurationManager.getVMPoolingInterval(), new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                a.poolForPendingVMs();
                if (a.hasPendingVMs()) {
                    processAllVMs();
                }
            }
        });
        vmPoolingTimer.start();
    }

    @Override
    public void action() {
        ACLMessage message = agent.blockingReceive();

        if (message == null) {
            return;
        }

        Object[] content = null;
        try {
            content = (Object[]) message.getContentObject();
        } catch (UnreadableException e) {
            GlobalLoopLogger.error(e);
        }

        AgentMessageType command = (AgentMessageType) content[0];

        switch (command) {
            case ACTION_PLAN_EXECUTION_COMPLETED:
                agent.poolForPendingVMs();
                if (agent.hasPendingVMs()) {
                    processAllVMs();
                    if (vmPoolingTimer.isRunning()) {
                        vmPoolingTimer.stop();
                    }
                } else {
                    vmPoolingTimer.restart();
                }
        }
    }

    private void processNextVM() {

        VirtualTaskInfo virtualTaskInfo = agent.getNextPendingVM();
        ActivityDto activityDto = new ActivityDto();

        //TODO: de eliminat citirea scrierea din/in EPKB
        activityDto.setEpkbID("A");
        activityDto.setIp(virtualTaskInfo.getNetworkInfo().getIp());
        activityDto.setId("" + virtualTaskInfo.getId());
        activityDto.setTaskName(virtualTaskInfo.getTaskName());
        activityDto.setRequestedCPUMin((int) virtualTaskInfo.getRequestedCPU());
        activityDto.setRequestedCPUMax((int) virtualTaskInfo.getRequestedCPU());
        activityDto.setRequestedMemoryMin((int) virtualTaskInfo.getRequestedMemory());
        activityDto.setRequestedMemoryMax((int) virtualTaskInfo.getRequestedMemory());

        try {
            RMIAPI rmiapi;
            Registry registry = LocateRegistry.getRegistry(GeneralConfigurationManager.getRMIAPIPort());
            rmiapi = (RMIAPI) registry.lookup(RMIAPI.class.getName());
            rmiapi.processCommand(RMICommand.SERVICE_COLLECTION_DEPLOYMENT_NOTIFICATION, activityDto);
        } catch (Exception ex) {
            GlobalLoopLogger.error(ex);
        }

    }

    private void processAllVMs() {

        List<VirtualTaskInfo> pendingTasks = agent.getPendingVirtualMachines();

        List<ActivityDto> activityDtos = new ArrayList<ActivityDto>();

        for (VirtualTaskInfo virtualTaskInfo : pendingTasks) {
//            ActivityDto activityDto = new ActivityDto();
//
//            //TODO: de eliminat citirea scrierea din/in EPKB
//            activityDto.setEpkbID("A");
//            activityDto.setIp(virtualTaskInfo.getNetworkInfo().getIp());
//            activityDto.setCloudID(virtualTaskInfo.getId());
//            activityDto.setTaskName(virtualTaskInfo.getTaskName());
//            activityDto.setRequestedCores((float) virtualTaskInfo.getRequestedCPU());
//            activityDto.setRequestedCPUMin(GeneralConfigurationManager.getVMDefaultCPUFrequency());
//            activityDto.setRequestedCPUMax(GeneralConfigurationManager.getVMDefaultCPUFrequency());
//            activityDto.setRequestedMemoryMin((int) virtualTaskInfo.getRequestedMemory());
//            activityDto.setRequestedMemoryMax((int) virtualTaskInfo.getRequestedMemory());
//            activityDto.setCpuWeight(0.5f);
//            activityDto.setMemWeight(0.5f);
            activityDtos.add(VirtualTaskInfoToActivityDtoMapper.map(virtualTaskInfo));
        }

        if (activityDtos.size() > 0) {
            Serializable[] toBeSent = new Serializable[activityDtos.size()];
            toBeSent = activityDtos.toArray(toBeSent);

            try {
                RMIAPI rmiapi;
                Registry registry = LocateRegistry.getRegistry(GeneralConfigurationManager.getRMIAPIPort());
                rmiapi = (RMIAPI) registry.lookup(RMIAPI.class.getName());
                rmiapi.processCommand(RMICommand.SERVICE_COLLECTION_DEPLOYMENT_NOTIFICATION, toBeSent);
            } catch (Exception ex) {
                GlobalLoopLogger.error(ex);
            }
        }
        vmPoolingTimer.stop();

    }


    public void terminate() {
//        vmPoolingTimer.stop();
    }
}