/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.commonDtos;

import java.util.ArrayList;
import java.util.Collection;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ActivityDto extends GenericActivityDto implements Cloneable {
    private float cpuWeight;
    private float memWeight;
    private float hddWeight;
    private String id;
    private String epkbID;
    private int cloudID;
    private int associatedServerID;
    private String associatedServerIP;
    //in seconds
    private int life;
    
    private String vmConfiguration;

    private Collection<ActivityDiskDto> diskDtoCollection;

    private String ip;
    private int vncPort;

    private int networkID;
    private String serviceName;

    public String getVmConfiguration() {
        return vmConfiguration;
    }

    public void setVmConfiguration(String vmConfiguration) {
        this.vmConfiguration = vmConfiguration;
    }

    public String getEpkbID() {
        return epkbID;
    }

    public void setEpkbID(String epkbID) {
        this.epkbID = epkbID;
    }

        
    public String getServiceName() {
        return serviceName;
    }

    public int getCloudID() {
        return cloudID;
    }

    public void setCloudID(int cloudID) {
        this.cloudID = cloudID;
    }
    

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    {
        diskDtoCollection = new ArrayList<ActivityDiskDto>();
    }

    public int getNetworkID() {
        return networkID;
    }

    public void setNetworkID(int networkID) {
        this.networkID = networkID;
    }

    public int getLife() {
        return life;
    }

    public void setLife(int life) {
        this.life = life;
    }

    public Collection<ActivityDiskDto> getDiskDtoCollection() {
        return diskDtoCollection;
    }

    public void setDiskDtoCollection(Collection<ActivityDiskDto> diskDtoCollection) {
        this.diskDtoCollection = diskDtoCollection;
    }

    public String getIp() {
        return ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public int getVncPort() {
        return vncPort;
    }

    public void setVncPort(int vncPort) {
        this.vncPort = vncPort;
    }

    public float getCpuWeight() {
        return cpuWeight;
    }

    public void setCpuWeight(float cpuWeight) {
        this.cpuWeight = cpuWeight;
    }

    public float getMemWeight() {
        return memWeight;
    }

    public void setMemWeight(float memWeight) {
        this.memWeight = memWeight;
    }

    public float getHddWeight() {
        return hddWeight;
    }

    public void setHddWeight(float hddWeight) {
        this.hddWeight = hddWeight;
    }

    public double distanceTo(ActivityDto task) {
        double minDistance = 1000000f;
        double memory1[] = {this.getRequestedMemoryMin(), this.getRequestedMemoryMax(), this.getRequestedMemoryMax(), this.getRequestedMemoryMin(), this.getRequestedMemoryMin(), this.getRequestedMemoryMax(), this.getRequestedMemoryMax(), this.getRequestedMemoryMin()};
        double cpu1[] = {this.getRequestedCPUMin(), this.getRequestedCPUMin(), this.getRequestedCPUMax(), this.getRequestedCPUMax(), this.getRequestedCPUMin(), this.getRequestedCPUMin(), this.getRequestedCPUMax(), this.getRequestedCPUMax()};
        // double hdd1[] = {this.getRequestedStorageMax(), this.getRequestedStorageMax(), this.getRequestedStorageMax(), this.getRequestedStorageMax(), this.getRequestedStorageMin(), this.getRequestedStorageMin(), this.getRequestedStorageMin(), this.getRequestedStorageMin()};
        double memory2[] = {task.getRequestedMemoryMin(), task.getRequestedMemoryMax(), task.getRequestedMemoryMax(), task.getRequestedMemoryMin(), task.getRequestedMemoryMin(), task.getRequestedMemoryMax(), task.getRequestedMemoryMax(), task.getRequestedMemoryMin()};
        double cpu2[] = {task.getRequestedCPUMin(), task.getRequestedCPUMin(), task.getRequestedCPUMax(), task.getRequestedCPUMax(), task.getRequestedCPUMin(), task.getRequestedCPUMin(), task.getRequestedCPUMax(), task.getRequestedCPUMax()};
        //double hdd2[] = {task.getRequestedStorageMax(), task.getRequestedStorageMax(), task.getRequestedStorageMax(), task.getRequestedStorageMax(), task.getRequestedStorageMin(), task.getRequestedStorageMin(), task.getRequestedStorageMin(), task.getRequestedStorageMin()};
        double totalDistance = 0;
        for (int i = 0; i < 8; i++) {
            {
                double dist = Math.sqrt((Math.pow(memory1[i] - memory2[i], 2) + Math.pow((cpu1[i] - cpu2[i]), 2)));//+ Math.pow((hdd1[i] - hdd2[j]), 2)));
                totalDistance += dist;

            }
        }
        return totalDistance;
    }


    public String toString() {
        return "Id"+this.getId()+"ServiceName"+this.getServiceName()+"Mem"+this.getRequestedMemoryMin()+"-"+this.getRequestedMemoryMin()+"\n CPU"+this.getRequestedCPUMin()+"-"+this.getRequestedCPUMax()+"\n HDD"+this.getRequestedStorageMin()+"-"+this.getRequestedStorageMax();
    }


    public boolean equals(Object dto) {
        if (!(dto instanceof GenericActivityDto)) return false;
        ActivityDto etd = (ActivityDto) dto;

        if (etd.getCpuWeight() != cpuWeight) return false;
        if (etd.getHddWeight() != hddWeight) return false;
        if (etd.getMemWeight() != memWeight) return false;
        
        return super.equals(dto);

    }


    public Object clone() throws CloneNotSupportedException {
        ActivityDto clone = (ActivityDto) super.clone();
        clone.setRequestedCPUMax(this.getRequestedCPUMax());
        clone.setRequestedCPUMin(this.getRequestedCPUMin());
        clone.setRequestedMemoryMax(this.getRequestedMemoryMax());
        clone.setRequestedMemoryMin(this.getRequestedMemoryMin());
        clone.setRequestedStorageMax(this.getRequestedStorageMax());
        clone.setRequestedStorageMin(this.getRequestedStorageMin());
        clone.setRequestedCores(this.getRequestedCores());
        clone.setCpuWeight(this.getCpuWeight());
        clone.setMemWeight(this.getMemWeight());
        clone.setHddWeight(this.getHddWeight());
        clone.setDiskDtoCollection(this.getDiskDtoCollection());
        clone.setTaskName(this.getTaskName());
        clone.setIp(this.getIp());
        clone.setLife(this.getLife());
        clone.setNetworkID(this.getNetworkID());
        clone.setServiceName(this.getServiceName());
        clone.setVncPort(this.getVncPort());
        clone.setEpkbID(this.getEpkbID());
        clone.setVmConfiguration(this.getVmConfiguration());
        clone.setCloudID(this.getCloudID());
        return clone;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public int getAssociatedServerID() {
        return associatedServerID;
    }

    public void setAssociatedServerID(int associatedServerID) {
        this.associatedServerID = associatedServerID;
    }

    public String getAssociatedServerIP() {
        return associatedServerIP;
    }

    public void setAssociatedServerIP(String associatedServerIP) {
        this.associatedServerIP = associatedServerIP;
    }

}
