/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.commonDtos;


import ro.utcn.dsrl.utils.similarContextIdentification.SimilarContextSituationIdentification;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class ContextSituationDto implements Serializable {
    private List<ExtendedServerDto> servers;
    private List<ActivityDto> tasks;
    private List<ActionDto> actions;

    private double energyLevel;

    private UUID id;

    public static final double MAXIMUM_DISTANCE = 1000000;

    public ContextSituationDto() {
        id = UUID.randomUUID();
        servers = new ArrayList<ExtendedServerDto>();
        tasks = new ArrayList<ActivityDto>();
        actions = new ArrayList<ActionDto>();
    }

    public void addServerDto(ExtendedServerDto dto) {
        servers.add(dto);
    }

    public void addTaskDto(ActivityDto dto) {
        tasks.add(dto);
    }

    public void addActionDto(ActionDto dto) {
        actions.add(dto);
    }

    public List<ExtendedServerDto> getServers() {
        return servers;
    }

    public void setServers(List<ExtendedServerDto> servers) {
        this.servers = servers;
    }

    public List<ActivityDto> getTasks() {
        return tasks;
    }

    public void setTasks(List<ActivityDto> tasks) {
        this.tasks = tasks;
    }

    public List<ActionDto> getActions() {
        return actions;
    }

    public void setActions(List<ActionDto> actions) {
        this.actions = actions;
    }

    public double distanceTo(ContextSituationDto dto) {
        return Math.abs(dto.getEnergyLevel() - energyLevel);
    }

    public double distance(ContextSituationDto contextSituationDto) {
        int distance = 0;
        HashMap<String, String> associatedServer = new HashMap<String, String>();
        for (ExtendedServerDto dto : contextSituationDto.getServers()) {
            for (ExtendedServerDto dto1 : servers) {
                if (!associatedServer.containsKey(dto.getServerName())) {
                    if (!associatedServer.containsValue(dto1.getServerName())) {
                        associatedServer.put(dto1.getServerName(), dto.getServerName());
                        distance += Math.pow(dto.distanceTo(dto1), 2);
                    }
                }
            }
        }
        HashMap<String, String> associatedTask = new HashMap<String, String>();

        for (ActivityDto dto : contextSituationDto.getTasks()) {

            for (ActivityDto dto1 : tasks) {
                if (!associatedTask.containsKey(dto.getTaskName())) {
                    if (!associatedTask.containsValue(dto1.getTaskName())) {
                        associatedTask.put(dto.getTaskName(), dto1.getTaskName());
                        distance += Math.pow(dto.distanceTo(dto1), 2);
                    }
                }
            }
        }
        return Math.sqrt(distance);
    }


    public boolean equals(ContextSituationDto contextSituationDto) {
        //TODO: de pus threshold de similaritate cu centroidul clusterului
        if (tasks.size() != contextSituationDto.getTasks().size()) return false;
        if (servers.size() != contextSituationDto.getServers().size()) return false;
        SimilarContextSituationIdentification similarContext = SimilarContextSituationIdentification.getIdentification();
        similarContext.addServers(servers);
        similarContext.addTasks(tasks);
        similarContext.refresh();
        return similarContext.checkIfContextsSimilar(this, contextSituationDto);

    }


    public double getEnergyLevel() {
        return energyLevel;
    }

    public void setEnergyLevel(double energyLevel) {
        this.energyLevel = energyLevel;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

}
