/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.commonDtos;

import java.io.Serializable;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class GenericActivityDto implements Serializable {
    private String taskName;
    private float requestedCores;
    private int requestedCPUMax;
    private int requestedCPUMin;
    private int requestedMemoryMax;
    private int requestedMemoryMin;
    private int requestedStorageMax;
    private int requestedStorageMin;

    private int receivedCores;
    private int receivedCPU;
    private int receivedMemory;
    private int receivedStorage;

    private boolean isRunning;


    public boolean equals(Object task) {
        if (this == task) return true;

        if (task instanceof GenericActivityDto) {
            GenericActivityDto genericActivityDto = (GenericActivityDto) task;
            //if (!taskName.equals(genericActivityDto.getTaskName())) return false;
            if (requestedCores != genericActivityDto.getRequestedCores()) return false;
            if (requestedCPUMax != genericActivityDto.getRequestedCPUMax()) return false;
            if (requestedCPUMin != genericActivityDto.getRequestedCPUMin()) return false;
            if (requestedMemoryMax != genericActivityDto.getRequestedMemoryMax()) return false;
            if (requestedMemoryMin != genericActivityDto.getRequestedMemoryMin()) return false;
            if (requestedStorageMax != genericActivityDto.getRequestedStorageMax()) return false;
            if (requestedStorageMin != genericActivityDto.getRequestedStorageMin()) return false;
            if (receivedCores != genericActivityDto.getReceivedCores()) return false;
            if (receivedCPU != genericActivityDto.getReceivedCPU()) return false;
            if (receivedMemory != genericActivityDto.getReceivedMemory()) return false;
            if (receivedStorage != genericActivityDto.getReceivedStorage()) return false;
            if (genericActivityDto.isRunning() != isRunning()) return false;
        }

        return true;
    }

    public String getTaskName() {
        return taskName;
    }

    public void setTaskName(String taskName) {
        this.taskName = taskName;
    }

    public float getRequestedCores() {
        return requestedCores;
    }

    public void setRequestedCores(float requestedCores) {
        this.requestedCores = requestedCores;
    }

    public int getRequestedCPUMax() {
        return requestedCPUMax;
    }

    public void setRequestedCPUMax(int requestedCPUMax) {
        this.requestedCPUMax = requestedCPUMax;
    }

    public int getRequestedCPUMin() {
        return requestedCPUMin;
    }

    public void setRequestedCPUMin(int requestedCPUMin) {
        this.requestedCPUMin = requestedCPUMin;
    }

    public int getRequestedMemoryMax() {
        return requestedMemoryMax;
    }

    public void setRequestedMemoryMax(int requestedMemoryMax) {
        this.requestedMemoryMax = requestedMemoryMax;
    }

    public int getRequestedMemoryMin() {
        return requestedMemoryMin;
    }

    public void setRequestedMemoryMin(int requestedMemoryMin) {
        this.requestedMemoryMin = requestedMemoryMin;
    }

    public int getRequestedStorageMax() {
        return requestedStorageMax;
    }

    public void setRequestedStorageMax(int requestedStorageMax) {
        this.requestedStorageMax = requestedStorageMax;
    }

    public int getRequestedStorageMin() {
        return requestedStorageMin;
    }

    public void setRequestedStorageMin(int requestedStorageMin) {
        this.requestedStorageMin = requestedStorageMin;
    }

    public int getReceivedCores() {
        return receivedCores;
    }

    public void setReceivedCores(int receivedCores) {
        this.receivedCores = receivedCores;
    }

    public int getReceivedCPU() {
        return receivedCPU;
    }

    public void setReceivedCPU(int receivedCPU) {
        this.receivedCPU = receivedCPU;
    }

    public int getReceivedMemory() {
        return receivedMemory;
    }

    public void setReceivedMemory(int receivedMemory) {
        this.receivedMemory = receivedMemory;
    }

    public int getReceivedStorage() {
        return receivedStorage;
    }

    public void setReceivedStorage(int receivedStorage) {
        this.receivedStorage = receivedStorage;
    }

    public boolean isRunning() {
        return isRunning;
    }

    public void setRunning(boolean running) {
        isRunning = running;
    }

    public boolean equals(GenericActivityDto dto) {
        if (dto.getReceivedCores() != receivedCores) return false;
        if (dto.getReceivedCPU() != receivedCPU) return false;
        if (dto.getReceivedMemory() != receivedMemory) return false;
        if (dto.getReceivedStorage() != receivedStorage) return false;
        if (dto.getRequestedCores() != requestedCores) return false;
        if (dto.getRequestedCPUMax() != requestedCPUMax) return false;
        if (dto.getRequestedCPUMin() != requestedCPUMin) return false;
        if (dto.getRequestedMemoryMax() != requestedMemoryMax) return false;
        if (dto.getRequestedMemoryMin() != requestedMemoryMin) return false;
        if (dto.getRequestedStorageMax() != requestedStorageMax) return false;
        if (dto.getRequestedStorageMin() != requestedStorageMin) return false;
        if (!dto.getTaskName().equalsIgnoreCase(taskName)) return false;
        return true;
    }
}
