/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.commonDtos;

import ro.utcn.dsrl.utils.config.GeneralConfigurationManager;

import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public class VirtualTaskInfoToActivityDtoMapper {
    private VirtualTaskInfoToActivityDtoMapper() {

    }

    public static ActivityDto map(VirtualTaskInfo virtualTaskInfo){
        ActivityDto activityDto = new ActivityDto();
        activityDto.setCloudID(virtualTaskInfo.getId());
        activityDto.setTaskName(virtualTaskInfo.getTaskName());

        //TODO: de setat acceptable values dupa procent
        activityDto.setRequestedMemoryMax(virtualTaskInfo.getRequestedMemory());
        activityDto.setRequestedMemoryMin(virtualTaskInfo.getRequestedMemory());

        activityDto.setRequestedCores(new Double(virtualTaskInfo.getRequestedCores()).floatValue());

        //TODO; oare asta nu i cumva cu base
        activityDto.setRequestedCPUMax(new Double(virtualTaskInfo.getRequestedCPU()).intValue());
        activityDto.setRequestedCPUMin(new Double(virtualTaskInfo.getRequestedCPU()).intValue());
        activityDto.setCpuWeight(0.5f);
        activityDto.setMemWeight(0.5f);
        activityDto.setVmConfiguration(virtualTaskInfo.getVmTemplate());
        VirtualNetworkInfo virtualNetworkInfo=virtualTaskInfo.getNetworkInfo();
        activityDto.setIp(virtualNetworkInfo.getIp());
        activityDto.setAssociatedServerIP(virtualTaskInfo.getHostServerHostname());

        List<ActivityDiskDto> activityDiskDtos = new ArrayList<ActivityDiskDto>();
        List<VirtualDiskInfo> virtualDiskInfos  = virtualTaskInfo.getDiskDtos();

         for(VirtualDiskInfo virtualDiskInfo : virtualDiskInfos){
             ActivityDiskDto activityDiskDto = new ActivityDiskDto();
             activityDiskDto.setFormat(virtualDiskInfo.getFormat());
             activityDiskDto.setPath(virtualDiskInfo.getSource());
             activityDiskDto.setSize(virtualDiskInfo.getSize());
             activityDiskDto.setType(virtualDiskInfo.getType());
             activityDiskDtos.add(activityDiskDto);
         }

        activityDto.setDiskDtoCollection(activityDiskDtos);

        return  activityDto;
    }
}
