/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.commonInterfaces.rii;


import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;
import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.misc.Pair;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public interface RTEInfoInterface extends Remote, Serializable {

    public List<ExtendedServerDto> getServersInfo() throws RemoteException;

    public List<ActivityDto> getTasksInfo() throws RemoteException;

    public void setServersInfo(List<ExtendedServerDto> servers) throws RemoteException;

    public void setTasksInfo(List<ActivityDto> tasks) throws RemoteException;

    public void setInstantEnergyConsumptionInfo(Date measurementTime, Number withGames, Number withoutGames) throws RemoteException;

    //Number[0] = WithGames, Number[1] = withoutGames

    public Pair<Date, Number[]> getInstantEnergyConsumptionInfo() throws RemoteException;

    //Number[0] = WithGames, Number[1] = withoutGames

    public List<Pair<Date, Number[]>> getRecordedEnergyConsumption() throws RemoteException;

    public boolean isDataAboutTasksChanged() throws RemoteException;

    public void setDataAboutTasksChanged(boolean dataAboutTasksChanged) throws RemoteException;

    public boolean isDataAboutServersChanged() throws RemoteException;

    public void setDataAboutServersChanged(boolean dataAboutServersChanged) throws RemoteException;

    public boolean hasMessages() throws RemoteException;

    public void setHasMessages(boolean hasMessages) throws RemoteException;

    public List<Pair<Date, String>> getMessages() throws RemoteException;

    public void setMessages(List<Pair<Date, String>> messages) throws RemoteException;

    public void addMessage(Date messageTime, String message) throws RemoteException;

    public Number getEnergySavings() throws RemoteException;

    public void setEnergySavings(Number nr) throws RemoteException;

    public Date getMeasurementBeginning() throws RemoteException;

    public void setMeasurementBeginning(Date d) throws RemoteException;

    public Date getMeasurementEnding() throws RemoteException;

    public void setMeasurementEnding(Date d) throws RemoteException;

    public boolean saveFile(Serializable data, String fileName) throws RemoteException;

    public Serializable loadFile(String fileName) throws RemoteException;

    public String getCurrentlyExecutingCommand() throws RemoteException;

    public void setCurrentlyExecutingCommand(String command) throws RemoteException;

    public boolean hasExecutingCommandInfoChanged() throws RemoteException;

    public void setExecutingCommandInfoChanged(boolean val) throws RemoteException;

    public java.util.Date getCurrentTime() throws RemoteException;

    public void setInfoFromLocalLoop(List<Pair<String, Double>> freq, List<Pair<String, Integer>> states) throws RemoteException;

    public List<Pair<String, Double>> getCpuFrequencies() throws RemoteException;

    public void setCpuFrequencies(List<Pair<String, Double>> cpuFrequencies) throws RemoteException;

    public List<Pair<String, Integer>> getCpuStates() throws RemoteException;

    public void setCpuStates(List<Pair<String, Integer>> cpuStates) throws RemoteException;

}
