/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.exposedFunctionality.rmiAPI;

import java.io.Serializable;
import java.util.*;

/**
 * <br/>
 * <br/>
 * <p><strong>Technical University of Cluj Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */
public enum RMICommand implements Serializable {
    EXIT("exit"),
    START_ON_TICK("startOnTick"),
    RESET("reset"),
    CLEAR("clear"),
    //    ADD_HOST("addHost", true),
    ADD_HOSTS("addHosts", true),
    ADD_APPLICATION("addApplication", true),
    ADD_SCHEDULE("addSchedule", true),
    LIST_APPLICATIONS("listApplications"),
    LOAD_AND_PERSIST_APPLICATIONS("loadAndPersistApplications"),
    LOAD_AND_PERSIST_SERVERS("loadAndPersistSchedule"),
    LIST_HOSTS("listHosts"),
    REMOVE_APPLICATION("removeApplication"),
    REMOVE_ACTIVITY("removeActivity"),
    REMOVE_SERVICE("removeService"),
    REMOVE_HOST("removeHost"),
    REMOVE_ALL("removeAll"),
    RESCHEDULE("reschedule"),
    STOP_WORKLOAD_GENERATOR("stopWG"),
    START_WORKLOAD_GENERATOR("startWG"),
    READ_EPKB("readEPKB"),
    SERVICE_DEPLOYMENT_NOTIFICATION("serviceDeployment"),
    SERVICE_COLLECTION_DEPLOYMENT_NOTIFICATION("serviceCollectionDeployment"),
    SERVICE_UNDEPLOYMENT_NOTIFICATION("serviceTermination"),
    SERVICE_COLLECTION_UNDEPLOYMENT_NOTIFICATION("serviceTermination"),
    JMS_SERVICE_DEPLOYMENT_NOTIFICATION("jmsServiceDeployment"),
    JMS_SERVICE_UNDEPLOYMENT_NOTIFICATION("jmsServiceDeployment"),
    VM_ADDED_HOOK_ACTIVATED("vmAddedHookActivated"),
    READ_MONITORED_DATA("readMonitoredData"),
    VM_REMOVED_HOOK_ACTIVATED("vmRemovedHookActivated");

    private String command;
    private static HashMap<String, RMICommand> hashMap;
    private static HashMap<RMICommand, String> parameters;
    private boolean usesFile;

    static {
        hashMap = new HashMap<String, RMICommand>();
        parameters = new HashMap<RMICommand, String>();

        hashMap.put("exit", EXIT);
        hashMap.put("vmAddedHookActivated", VM_ADDED_HOOK_ACTIVATED);
        hashMap.put("vmRemovedHookActivated", VM_REMOVED_HOOK_ACTIVATED);

        hashMap.put("readEPKB", READ_EPKB);
        hashMap.put("loadAndPersistApplications", LOAD_AND_PERSIST_APPLICATIONS);
        hashMap.put("loadAndPersistServers", LOAD_AND_PERSIST_SERVERS);

//        hashMap.put("reset", RESET);
//        hashMap.put("clear", CLEAR);
//        hashMap.put("startOnTick", START_ON_TICK);

//        hashMap.put("addHost", ADD_HOST);
//        parameters.put(ADD_HOST, "description_file");

        hashMap.put("addHosts", ADD_HOSTS);
        parameters.put(ADD_HOSTS, "description_file_content");

//        hashMap.put("addApplication", ADD_APPLICATION);
//        parameters.put(ADD_APPLICATION, "description_file");

        hashMap.put("addScheduledTasks", ADD_SCHEDULE);
        parameters.put(ADD_SCHEDULE, "description_file");

//        hashMap.put("removeApplication", REMOVE_APPLICATION);
//        parameters.put(ADD_SCHEDULE, "name");

//        hashMap.put("removeActivity", REMOVE_ACTIVITY);
//        parameters.put(REMOVE_ACTIVITY, "name");

//        hashMap.put("removeService", REMOVE_SERVICE);
//        parameters.put(REMOVE_SERVICE, "name");

        hashMap.put("listApplications", LIST_APPLICATIONS);
        hashMap.put("listHosts", LIST_HOSTS);

        hashMap.put("removeHost", REMOVE_HOST);
        parameters.put(REMOVE_HOST, "hostname");

//        hashMap.put("removeAll", REMOVE_ALL);

//        hashMap.put("reschedule", RESCHEDULE);
//        parameters.put(RESCHEDULE, "name");

        hashMap.put("startRandomWorkloadGenerator", START_WORKLOAD_GENERATOR);

        parameters.put(START_WORKLOAD_GENERATOR, "[generationTimeMin=<intValue>] [generationIntervalMax=<intValue>]" +
                " [serviceLifeMin=<intValue>] [serviceLifeMax=<intValue>]" +
                " [generatedServicesNoMin=<intValue>][generatedServicesNoMax=<intValue>]");

        hashMap.put("stopRandomWorkloadGenerator", STOP_WORKLOAD_GENERATOR);
    }

    RMICommand(String command) {
        this.command = command;
    }

    RMICommand(String command, boolean usesFile) {
        this.usesFile = usesFile;
        this.command = command;
    }

    public boolean usesFile() {
        return usesFile;
    }

    public static List<String> getValues() {

        Iterator<String> iterator = hashMap.keySet().iterator();
        List<String> names = new ArrayList<String>();
        while (iterator.hasNext()) {
            String value = iterator.next();
            RMICommand command = hashMap.get(value);
//            if (parameters.containsKey(command)) {
//                names.add(value + " " + parameters.get(command) + "\n");
//            } else {
            names.add(value);// + "\n");
//            }
        }
        Collections.sort(names);
        return names;
    }


    public static RMICommand getCommandForString(String string) {
        return hashMap.get(string);
    }

    public static String getParameter(String commandName) {
        RMICommand command = hashMap.get(commandName);
        return parameters.get(command);
    }
}

