/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.misc;

import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class SimpleCSVWriter {
    private BufferedWriter fileWriter;
    private static DateFormat dateFormat = new SimpleDateFormat("_dd_MM_HH_mm_");

    private static Map<Class, SimpleCSVWriter> map;
    static {
         map = new HashMap<Class, SimpleCSVWriter>();
    }

    private SimpleCSVWriter(Class caller) {

        try {
            fileWriter = new BufferedWriter(new FileWriter(new File("./logs/" + caller.getName() + dateFormat.format(new Date()) + ".csv")));
        } catch (IOException e) {
            GlobalLoopLogger.error(e);
        }

    }

     private SimpleCSVWriter(String fileName) {

        try {
            fileWriter = new BufferedWriter(new FileWriter(new File("./logs/" + fileName + dateFormat.format(new Date()) + ".csv")));
        } catch (IOException e) {
            GlobalLoopLogger.error(e);
        }

    }

    public static SimpleCSVWriter getInstance(Class caller) {
        if (!map.containsKey(caller)) {
            SimpleCSVWriter simpleCSVWriter = new SimpleCSVWriter(caller);
            simpleCSVWriter.log("hours", "minutes", "seconds","servers","clusters","tasks to be deployed","total tasks no","onlineServersCount","timestamp");
            map.put(caller, simpleCSVWriter);
        }
        return map.get(caller);
    }

    public static SimpleCSVWriter getInstance(Class caller,String fileName) {
        if (!map.containsKey(caller)) {
            SimpleCSVWriter simpleCSVWriter = new SimpleCSVWriter(fileName);
            simpleCSVWriter.log("hours", "minutes", "seconds","servers","clusters","tasks to be deployed","total tasks no","onlineServersCount","timestamp");
            map.put(caller, simpleCSVWriter);
        }
        return map.get(caller);
    }

    public static SimpleCSVWriter getInstance(Class caller, String... columns) {
        if (!map.containsKey(caller)) {
            SimpleCSVWriter simpleCSVWriter = new SimpleCSVWriter(caller);
            simpleCSVWriter.log(columns);
            map.put(caller, simpleCSVWriter);
        }
        return map.get(caller);
    }

    public void log(String... values) {
        try {
            for (String value : values) {
                fileWriter.write(value + ",");
            }
            fileWriter.newLine();
            fileWriter.flush();
        } catch (IOException e) {
            GlobalLoopLogger.error(e);
        }

    }
}
