/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.misc;

import java.io.IOException;
import java.util.Map;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class SoapInvoker {

    private SoapInvoker() {
    }

    public static void performSOAPInvocation(String destinationURL,
            String operationName, String namespace)
            throws SOAPException, IOException {

        SOAPConnectionFactory soapConnFactory =
                SOAPConnectionFactory.newInstance();
        SOAPConnection connection =
                soapConnFactory.createConnection();

        //Next, create the actual message
        MessageFactory messageFactory = MessageFactory.newInstance();
        SOAPMessage message = messageFactory.createMessage();

        //Create objects for the message parts
        SOAPPart soapPart = message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();


        SOAPBody body = envelope.getBody();
        SOAPElement bodyElement = body.addChildElement(envelope.createName(operationName,
                "tns",
                namespace));

        //Add content
        SOAPElement productElement = bodyElement.addChildElement("input");

        System.out.println("\nREQUEST:\n");
        message.writeTo(System.out);
        System.out.println();

        //Send the message
        SOAPMessage reply = connection.call(message, destinationURL);
        System.out.println("\nREPLY:\n");

        //Close the connection
        connection.close();
    }

    public static void performSOAPInvocation(String destinationURL,
            String operationName, String namespace, String contentTagName, Map<String, String>... content)
            throws SOAPException, IOException {

        SOAPConnectionFactory soapConnFactory =
                SOAPConnectionFactory.newInstance();
        SOAPConnection connection =
                soapConnFactory.createConnection();

        //Next, create the actual message
        MessageFactory messageFactory = MessageFactory.newInstance();
        SOAPMessage message = messageFactory.createMessage();

        //Create objects for the message parts
        SOAPPart soapPart = message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();


        SOAPBody body = envelope.getBody();
        SOAPElement bodyElement = body.addChildElement(envelope.createName(operationName,
                "tns",
                namespace));

        //Add content
        for (Map<String, String> contentElement : content) {
            SOAPElement productElement = bodyElement.addChildElement(contentTagName);
            for (String node : contentElement.keySet()) {
                productElement.addChildElement(node).addTextNode(contentElement.get(node));
            }
        }

        System.out.println("\nREQUEST:\n");
        message.writeTo(System.out);
        System.out.println();

        //Send the message
        SOAPMessage reply = connection.call(message, destinationURL);
        System.out.println("\nREPLY:\n");

        //Close the connection
        connection.close();
    }

    public static void performSOAPInvocation(String destinationURL,
            String operationName, String namespace, String contentTagName, String content)
            throws SOAPException, IOException {

        SOAPConnectionFactory soapConnFactory =
                SOAPConnectionFactory.newInstance();
        SOAPConnection connection =
                soapConnFactory.createConnection();

        //Next, create the actual message
        MessageFactory messageFactory = MessageFactory.newInstance();
        SOAPMessage message = messageFactory.createMessage();

        //Create objects for the message parts
        SOAPPart soapPart = message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();


        SOAPBody body = envelope.getBody();
        SOAPElement bodyElement = body.addChildElement(envelope.createName(operationName,
                "tns",
                namespace));

        //Add content
        SOAPElement productElement = bodyElement.addChildElement(contentTagName);
        productElement.addTextNode(content);

        System.out.println("\nREQUEST:\n");
        message.writeTo(System.out);
        System.out.println();

        //Send the message
        SOAPMessage reply = connection.call(message, destinationURL);
        System.out.println("\nREPLY:\n");

        //Close the connection
        connection.close();
    }
}
