/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.misc;

import ro.utcn.dsrl.utils.exposedFunctionality.jaxbBindingClasses.*;
import ro.utcn.dsrl.utils.logging.GlobalLoopLogger;

import javax.xml.bind.JAXB;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: oneadmin
 * Date: 10/26/11
 * Time: 5:17 PM
 * To change this template use File | Settings | File Templates.
 */
public class WorkloadScheduleCSVTransformer {
    public static void createCSVFromWorkloadScheduleFile(String workloadSchedulePath) {
        SimpleCSVWriter simpleCSVWriter =
                SimpleCSVWriter.getInstance(WorkloadScheduleCSVTransformer.class, "delay", "runningTasksNo", "toBeDeployedTasksNo", "toBeDeletedTasksNo");
        LinkedHashMap<Long, List<ApplicationDescription>> generatedScheduleMap = new LinkedHashMap<Long, List<ApplicationDescription>>();

        try {
            WorkloadSchedule workloadSchedule = JAXB.unmarshal(new FileInputStream(workloadSchedulePath),
                    WorkloadSchedule.class);
            JaxbPair[] schedule = workloadSchedule.getApplications();
            for (JaxbPair jaxbPair : schedule) {
                if (generatedScheduleMap.containsKey(jaxbPair.getStartDelay())) {
                    generatedScheduleMap.get(jaxbPair.getStartDelay()).add(jaxbPair.getApplication());
                } else {
                    List<ApplicationDescription> serializables = new ArrayList<ApplicationDescription>();
                    serializables.add(jaxbPair.getApplication());
                    generatedScheduleMap.put(jaxbPair.getStartDelay(), serializables);
                }
            }
            Iterator<Long> keyIterator = generatedScheduleMap.keySet().iterator();
            Long firstKey = 0l;
            int runningTasksNo = 0;
            while (keyIterator.hasNext()) {
                Long currentKey = keyIterator.next();
                List<ApplicationDescription> content = generatedScheduleMap.get(currentKey);
                int toBeDeployedTasksCount = 0;
                int toBeDeletedTasksCount = 0;
                for (ApplicationDescription applicationDescription : content) {
                    for (ActivityDescription activityDescription : applicationDescription.getActivities()) {
                        for (ActivityService activityService : activityDescription.getServices()) {
                            if (activityService.getServiceName().contains("deleted")) {
                                toBeDeletedTasksCount++;
                            } else {
                                toBeDeployedTasksCount++;
                            }
                        }
                    }
                }
                simpleCSVWriter.log("" + currentKey / 1000, "" + runningTasksNo, "" + toBeDeployedTasksCount, "" + toBeDeletedTasksCount);
                runningTasksNo += toBeDeployedTasksCount;
                runningTasksNo -= toBeDeletedTasksCount;
            }
        } catch (FileNotFoundException e) {
            GlobalLoopLogger.getLogger().info(e.toString());
        }
    }

    public static void main(String[] args) {
        WorkloadScheduleCSVTransformer.createCSVFromWorkloadScheduleFile("/media/Extra/SVN_DSRL/Software/GlobalControlLoop/generatedSchedules/generated_schedule__26_10_13_47_.xml");
    }
}
