package ro.utcn.dsrl.utils.similarContextIdentification;


import ro.utcn.dsrl.utils.commonDtos.ActivityDto;
import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;
import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import ro.utcn.dsrl.utils.commonDtos.ServerInfo;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class KMeansClustering implements ClusteringAlgorithm {
    List objects;
    List<Cluster> clusters;
    public static int K = 10;
    public static final int MAXNRSTEPS = 100;

    public KMeansClustering() {
        objects = new ArrayList();
        clusters = new ArrayList<Cluster>();
    }

    public KMeansClustering(int k) {
        objects = new ArrayList();
        clusters = new ArrayList<Cluster>();
        K = k;
    }

    public Cluster getNearestCluster(Object o) {
        double smallestDist = Cluster.INFINITY;
        Cluster closestCluster = null;
        for (Cluster cl : clusters) {
            double dist = cl.distanceToCluster(o);
            if (dist < smallestDist) {
                smallestDist = dist;
                closestCluster = cl;
            }
        }
        return closestCluster;
    }

    public int getBelongingCluster(Object object) {
        int i = 0;
        for (Cluster cl : clusters) {
            if (cl.contains(object)) return i;
            i++;
        }
        return -1;
    }

    public void initializeClusters(Collection objects, Class cls) {
        this.objects.addAll(objects);
//        if (objects.isEmpty()) {
//            clusters = new ArrayList<Cluster>();
//            return;
//        }
        if (cls == ActivityDto.class) {
            List<ActivityDto> tasks = (List<ActivityDto>) objects;
            clusters = new ArrayList<Cluster>();
            int nr = 0;
            for (int i = 0; i < K; i++) {
                Cluster cl = new TasksCluster();
                int step = 0;
                while (step < tasks.size() / K && nr < tasks.size()) {
                    cl.addToCluster(tasks.get(nr));
                    step++;
                    nr++;
                }
                clusters.add(i, cl);
            }
        }
        if (cls == ExtendedServerDto.class) {
            List<ExtendedServerDto> servers = (List<ExtendedServerDto>) objects;
            clusters = new ArrayList<Cluster>();
            int nr = 0;
            for (int i = 0; i < K; i++) {
                Cluster cluster = new ServersCluster();
                int step = 0;
                while (step < servers.size() / K && nr < servers.size()) {
                    cluster.addToCluster(servers.get(nr));
                    step++;
                    nr++;
                }
                clusters.add(i, cluster);
            }

        }
        if (cls == ContextSituationDto.class) {
            List<ContextSituationDto> situations = (List<ContextSituationDto>) objects;
            clusters = new ArrayList<Cluster>();
            int nr = 0;
            for (int i = 0; i < K; i++) {
                PowerCluster cluster = new PowerCluster();
                cluster.generateRandomID();
                int step = 0;
                while (step <= (double) situations.size() / K && nr < situations.size()) {
                    cluster.addToCluster(situations.get(nr));
                    step++;
                    nr++;
                }
                clusters.add(i, cluster);
            }
        }
        int clustersNo = clusters.size();
        for (int i = 0; i < clustersNo; i++) {
            Cluster cluster = clusters.get(i);
            cluster.refreshClusterCentroid();
        }
        refreshClusters();

    }

    public int getNrOfClusters() {
        return clusters.size();  //To change body of implemented methods use File | Settings | File Templates.
    }


    public void addObjectToKnowledgeBase(Object object) {
        objects.add(object);
        if (getNearestCluster(object) == null) {
            if (object instanceof ActivityDto) {
                initializeClusters(objects, ActivityDto.class);
            } else if (object instanceof ExtendedServerDto){
                initializeClusters(objects, ExtendedServerDto.class);
               
            }    
            else
            {
                
                initializeClusters(objects, ContextSituationDto.class);
            }

        } else{
            getNearestCluster(object).addToCluster(object);
        }
        refreshClusters();
    }

    public void addObjectsToKnowledgeBase(Collection objects) {
        this.objects.addAll(objects);
        for (Object o : objects) {
            Cluster cluster = getNearestCluster(o);
            if (cluster != null) {
                cluster.addToCluster(o);
            }
        }

    }

    public void refreshClusters() {
        boolean changed = true;
        int i = 0;
        while (changed && MAXNRSTEPS > i) {
            changed = false;
            for (Cluster cl : clusters) {
                List elements = cl.getAllElements();
                List objectsToBeRemoved = new ArrayList();
                for (Object o : elements) {
                    Cluster nearest = getNearestCluster(o);
                    if (nearest != null && !nearest.equals(cl)) {
                        objectsToBeRemoved.add(o);
                        nearest.addToCluster(o);
                        changed = true;
                    }
                }
                for (Object o : objectsToBeRemoved) {
                    cl.removeFromCluster(o);
                }
            }
            if (changed) {
                for (Cluster cl : clusters) {
                    cl.refreshClusterCentroid();
                }
            }
            i++;
        }
    }

    public List<Cluster> getAllClusters() {
        return clusters;
    }

    public void setInitialClusters(List<Cluster> objects) {
        clusters = objects;
    }
}
