/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.similarContextIdentification;

import ro.utcn.dsrl.utils.commonDtos.ContextSituationDto;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class PowerCluster implements Cluster {
    private UUID id;
    private List<ContextSituationDto> contexts;
    private Object clusterCentroid;
    private double mediumClusterLevel;

    public PowerCluster() {

        contexts = new ArrayList<ContextSituationDto>();
    }

    public void generateRandomID() {
        id = UUID.randomUUID();
    }

    public void addToCluster(Object o) {
        contexts.add((ContextSituationDto) o);


    }

    public void removeFromCluster(Object o) {
        if (!(o instanceof ContextSituationDto)) return;
        if (contexts.contains(o))
            contexts.remove(o);
        else return;
    }

    public boolean refreshClusterCentroid() {
        double smallestDistance = INFINITY;
        Object intermCentroid = null;
        boolean ok = false;
        double average = 0;
        int number = 0;
        if (contexts.size() != 0) {
            for (Object context : contexts) {
                double distance = 0.0;
                int i = 0;
                for (Object contextTo : contexts) {
                    if (context instanceof ContextSituationDto && contextTo instanceof ContextSituationDto)
                        distance += ((ContextSituationDto) context).distanceTo((ContextSituationDto) contextTo);
                    else if (context instanceof PowerCluster && contextTo instanceof PowerCluster)
                        distance += ((PowerCluster) contextTo).distanceToCluster((context));
                    else if (contextTo instanceof PowerCluster)
                        distance += ((TasksCluster) context).distanceToCluster(contextTo);
                    i++;
                }
                average += ((ContextSituationDto) context).getEnergyLevel();
                number++;
                if ((distance / i) < smallestDistance) {
                    smallestDistance = distance;
                    intermCentroid = context;
                }
            }

            if (clusterCentroid != null && clusterCentroid.equals(intermCentroid)) {
                ok = true;
            } else {
                clusterCentroid = intermCentroid;
            }

        }
        if (number > 0)
            setMediumClusterLevel(average / number);
        return ok;
    }

    public List getAllElements() {
        return contexts;
    }

    public double distanceToCluster(Object o) {

        if (o instanceof PowerCluster) {
            double minDist = INFINITY;
            Cluster cluster = (Cluster) o;
            for (Object o1 : cluster.getAllElements()) {
                double d = cluster.distanceToCluster(o1);
                if (minDist > d) minDist = d;
            }
            return minDist;
        }

        if (o instanceof Double) {
            double minDist = INFINITY;
            double d = (Double) o;
            for (Object o1 : this.getAllElements()) {
                if (o1 instanceof ContextSituationDto) {
                    ContextSituationDto contextSituationDto = (ContextSituationDto) o1;
                    if (minDist > Math.abs(contextSituationDto.getEnergyLevel() - d)) {
                        minDist = Math.abs(contextSituationDto.getEnergyLevel() - d);
                    }
                }
            }
            return minDist;
        }
        ContextSituationDto context = (ContextSituationDto) o;

        if (clusterCentroid == null) refreshClusterCentroid();
        if (contexts.size() == 0) return 0;
        return context.distanceTo((ContextSituationDto) clusterCentroid);

    }

    public double distanceFromContextSit(Double d) {
        double minDist = INFINITY;
        for (Object obj : contexts) {
            if (obj instanceof ContextSituationDto) {
                ContextSituationDto contextSituationDto = (ContextSituationDto) obj;
                if (Math.abs(contextSituationDto.getEnergyLevel() - d) < minDist) {
                    minDist = Math.abs(contextSituationDto.getEnergyLevel() - d);
                }
            }
        }
        return minDist;
    }

    public Object getClusterCentroid() {
        return clusterCentroid;
    }

    public boolean equals(Object o) {
        if (!(o instanceof Cluster || o instanceof PowerCluster)) {
            return false;
        }
        PowerCluster powerCluster = (PowerCluster) o;
        //TODO: Check if need the same centroids
        if (powerCluster.getAllElements().size() != contexts.size()) {
            return false;
        }
        if (powerCluster.getAllElements().size() == 0 && contexts.size() == 0) {
            return true;
        }
        if (!clusterCentroid.equals(powerCluster.getClusterCentroid())) {
            return false;
        }
        for (Object context : powerCluster.getAllElements()) {
            if (!(context instanceof ContextSituationDto)) {
                return false;
            }
            if (!powerCluster.contains(context)) {
                return false;
            }
        }
        return true;
    }

    public boolean contains(Object o) {
        return contexts.contains(o);
    }

    public double getMediumClusterLevel() {
        return mediumClusterLevel;
    }

    public void setMediumClusterLevel(double mediumClusterLevel) {
        this.mediumClusterLevel = mediumClusterLevel;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }
}
