/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.similarContextIdentification;


import ro.utcn.dsrl.utils.commonDtos.ExtendedServerDto;

import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */


public class ServersCluster implements Cluster {
    List servers;
    Object clusterCentroid;

    public ServersCluster() {
        servers = new ArrayList<ExtendedServerDto>();
    }

    public void addToCluster(Object o) {
        servers.add((ExtendedServerDto) o);
    }

    public void removeFromCluster(Object o) {
        servers.remove(o);
    }

    public boolean refreshClusterCentroid() {
        double smallestDistance = INFINITY;
        if (servers.size() == 1) {
            clusterCentroid = servers.get(0);
            return true;
        }
        boolean ok = false;
        if (servers.size() != 0) {
            Object intermCentroid = null;
            for (Object server : servers) {
                double distance = 0.0;
                for (Object serverTo : servers) {
                    if (server instanceof ExtendedServerDto && serverTo instanceof ExtendedServerDto)
                        distance += ((ExtendedServerDto) server).distanceTo((ExtendedServerDto) serverTo);
                    else if (server instanceof ExtendedServerDto && serverTo instanceof ServersCluster)
                        distance += ((ServersCluster) serverTo).distanceToCluster((server));
                    else if (server instanceof ServersCluster)
                        distance += ((ServersCluster) server).distanceToCluster(serverTo);
                }
                if (distance < smallestDistance) {
                    smallestDistance = distance;
                    intermCentroid = server;
                }
            }

            if (clusterCentroid != null && clusterCentroid.equals(intermCentroid)) {
                ok = true;
            } else {
                clusterCentroid = intermCentroid;
            }
        }
        return ok;
    }

    public List getAllElements() {
        return servers;
    }

    public double distanceToCluster(Object o) {
        if (!(o instanceof ExtendedServerDto || o instanceof ServersCluster)) return INFINITY;
        if (o instanceof ServersCluster) {
            double minDist = INFINITY;
            Cluster cluster = (Cluster) o;
            for (Object o1 : cluster.getAllElements()) {
                double d = cluster.distanceToCluster(o1);
                if (minDist > d) minDist = d;
            }
            return minDist;
        }
        ExtendedServerDto server = (ExtendedServerDto) o;
        if (clusterCentroid == null) refreshClusterCentroid();
        if (servers.size() == 0) return 0;
        if (clusterCentroid instanceof ExtendedServerDto)
            return server.distanceTo((ExtendedServerDto) clusterCentroid);
        else
            return ((ServersCluster) clusterCentroid).distanceToCluster(server);

    }

    public Object getClusterCentroid() {
        return clusterCentroid;
    }

    public boolean equals(Object o) {
        if (!(o instanceof Cluster || o instanceof ExtendedServerDto)) return false;
        ServersCluster serversCluster = (ServersCluster) o;
        //TODO: Check if need the same centroids
        // if (!clusterCentroid.equals(serversCluster.getClusterCentroid())) return false;
        for (Object sv : serversCluster.getAllElements()) {
            if (!(sv instanceof ExtendedServerDto)) return false;
            if (!servers.contains(sv)) return false;
        }
        return true;
    }

    public boolean contains(Object o) {
        if (!(o instanceof ExtendedServerDto)) return false;
        if (servers.contains(o)) return true;
        return false;
    }


}
