/**********************************************************************************************************
 * Copyright 2012, Distributed Systems Research Laboratory, Technical University of Cluj-Napoca, Romania 
 * http://dsrl.coned.utcluj.ro/
 *  
 * Licensed under the EUPL V.1.1 
 * 
 * European Union Public Licence V. 1.1  
 * You may obtain a copy of this license at
 * 
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 *  
 **********************************************************************************************************/
package ro.utcn.dsrl.utils.similarContextIdentification;


import ro.utcn.dsrl.utils.commonDtos.ActivityDto;

import java.util.ArrayList;
import java.util.List;

/**
 * <br/>
 * <br/>
 * <p><strong>GAMES Project - Global Control Loop</strong></p>
 * <p><strong>@Author: Technical University of Cluj-Napoca</strong></p>
 * <br/>
 * <a href=http://dsrl.coned.utcluj.ro/ > Distributed Systems research Laboratory </a>
 * <br/>Coordinator: Prof. dr. ing. Ioan Salomie E-mail: Ioan.Salomie@cs.utcluj.ro
 * <br/>
 * <br/>Developers:
 * <br/>Daniel Moldovan(Daniel.Moldovan@cs.utcluj.ro)
 * <br/>Georgiana Copil(Georgiana.Copil@cs.utcluj.ro)
 */

public class TasksCluster implements Cluster {
    List tasks;

    Object clusterCentroid;

    public TasksCluster() {
        tasks = new ArrayList<ActivityDto>();
    }

    public void addToCluster(Object o) {
        tasks.add((ActivityDto) o);
    }

    public void removeFromCluster(Object o) {
        tasks.remove(o);
        if (o.equals(clusterCentroid)) refreshClusterCentroid();
    }

    public boolean refreshClusterCentroid() {
        double smallestDistance = INFINITY;
        Object intermCentroid = null;
        boolean ok = false;
        if (tasks.size() != 0) {
            for (Object task : tasks) {
                double distance = 0.0;
                int i = 0;
                for (Object taskTo : tasks) {
                    if (task instanceof ActivityDto && taskTo instanceof ActivityDto)
                        distance += ((ActivityDto) task).distanceTo((ActivityDto) taskTo);
                    else if (task instanceof ActivityDto && taskTo instanceof TasksCluster)
                        distance += ((TasksCluster) taskTo).distanceToCluster((task));
                    else if (task instanceof TasksCluster)
                        distance += ((TasksCluster) task).distanceToCluster(taskTo);
                    i++;
                }

                if ((distance / i) < smallestDistance) {
                    smallestDistance = distance;
                    intermCentroid = task;
                }
            }

            if (clusterCentroid != null && clusterCentroid.equals(intermCentroid)) {
                ok = true;
            } else {
                clusterCentroid = intermCentroid;
            }
        }
        return ok;


    }

    public List getAllElements() {
        return tasks;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public double distanceToCluster(Object o) {
        if (!(o instanceof ActivityDto || o instanceof TasksCluster)) return INFINITY;
        if (o instanceof TasksCluster) {
            double minDist = INFINITY;
            Cluster cluster = (Cluster) o;
            for (Object o1 : cluster.getAllElements()) {
                double d = cluster.distanceToCluster(o1);
                if (minDist > d) minDist = d;
            }
            return minDist;
        }
        ActivityDto task = (ActivityDto) o;
        if (clusterCentroid == null) refreshClusterCentroid();
        if (tasks.size() == 0) return 0;
        if (clusterCentroid instanceof ActivityDto)
            return task.distanceTo((ActivityDto) clusterCentroid);
        else
            return ((TasksCluster) clusterCentroid).distanceToCluster(task);

    }

    public Object getClusterCentroid() {
        return clusterCentroid;
    }

    @Override
    public boolean equals(Object o) {
        if (!(o instanceof Cluster || o instanceof TasksCluster)) return false;
        TasksCluster tasksCluster = (TasksCluster) o;
        //TODO: Check if need the same centroids
        if (tasksCluster.getAllElements().size() != tasks.size()) return false;
        if (tasksCluster.getAllElements().size() == 0 && tasks.size() == 0) return true;
        if (!clusterCentroid.equals(tasksCluster.getClusterCentroid())) return false;
        for (Object task : tasksCluster.getAllElements()) {
            if (!(task instanceof ActivityDto)) return false;
            if (!tasksCluster.contains(task)) return false;
        }
        return true;
    }

    public boolean contains(Object o) {
        if (!(o instanceof ActivityDto)) return false;

        if (tasks.contains(o)) return true;
        return false;
    }
}
